// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage;

import java.awt.Color;
import java.util.ArrayList;

import javax.swing.Box;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;
import java.awt.BorderLayout;
import java.awt.Component;

public abstract class FilterDataStorage extends InformationDataStorage
        implements RemovableDataStorage, DataStorageListener {

    private DataStorage _dataStorage = null;
    protected JPanel filterPanel = null;
    protected String filterColumnName = "";
    protected JPanel filterExpressionPanel;
    protected JLabel warningMessage = new JLabel();
    protected DataObjectComparator comparator;
    //private int filterColumnValueIndex = -1;
    /**
     * All possible values of the filterColumn (corresponds to all key values of
     * the hashtable)
     */
    public ArrayList<String> keyValues = null;

    public FilterDataStorage(DataStorageListener dsl) {
        super(dsl);
        dataTableModel = new DataTableModel(dataTable);
        dtsp = new DataTableStoragePanel(dataTableModel);
    }

    public FilterDataStorage(DataStorageListener dsl, DataStorage dataStorage) {
        this(dsl, dataStorage, null);
    }

    public FilterDataStorage(DataStorageListener dsl, DataStorage dataStorage,
            String filterColumnName) {
        super(dsl);
        //System.out.println("))))))))))))))))))FilterDS="+this+" datastorage = " + dataStorage);
        setDataStorage(dataStorage);
        // we are now the listener
        dataStorage.dsl = this;
        dataTableModel = new DataTableModel(dataTable);
        dtsp = new DataTableStoragePanel(dataTableModel);
        setColumnNames();
        setFilterColumnName(filterColumnName); // inits hashtable and/or key values
    }

    public abstract boolean isReady();

    /** returns the original DataStorage */
    public DataStorage removeItSelf() {
	DataStorage dataStorage = getDataStorage();
        dataStorage.dsl = dsl;
        dsl.dataStorageEvent(dataStorage);
        return dataStorage;
    }

    public void dataStorageEvent(DataStorage source) {
        /*if(dataStorage == null)
            dataStorage = source;*/
        Debug.message("DataStorage", "[FilterDataStorage:] dataStorageEvent <== " + source);
        // check if filterColumnName is still a valid column name

        if (isReady()) {
            setColumnNames();
            // reset the GUI ??
            setFilterColumnName(filterColumnName);// inits hashtable and/or key values
            setTable();
            initFilterExpressionPanel();
        }
    }
    
    protected ArrayList<String> dataColumnNames = null;
    protected ArrayList<String> omColumnNames = null;

    protected void setColumnNames() {
        String[] names;
        names = getDataStorage().getDefaultDataTableModel().getDataColumnNames();
        dataColumnNames = new ArrayList<String>(names.length);
        //System.out.println("NOMBRE DE DATA COL NAMES = " + names.length);
        boolean filterColumnNameIsValid = false;
        for (String n : names) {
            if (n.equals(filterColumnName)) {
                filterColumnNameIsValid = true;
            }
            dataColumnNames.add(n);
        }
        if (!filterColumnNameIsValid) {
            // <Warning message>
            warningMessage.setText(i18n.get(MonoValuedFilterDataStorage.class,
                    "warning2", "Select a column name"));
            // </Warning message>
            filterColumnName = "";
        }
        names = getDataStorage().getDefaultDataTableModel().getOMColumnNames();
        omColumnNames = new ArrayList<String>(names.length);
        for (String n : names) {
            omColumnNames.add(n);
        }
        //System.out.println("SET COLUMN NAMES" + dataColumnNames.size());
        initFilterExpressionPanel();
    }

    public JComponent getGUI() {
        filterPanel = new JPanel(new BorderLayout()); //Box.createHorizontalBox();
        filterPanel.add(filterExpressionPanel, BorderLayout.CENTER);
        Box panel = Box.createHorizontalBox();

        //addComboMode(panel);
        panel.add(filterPanel);
        Component dsGUI = getDataStorage().getGUI();
        //filterPanel.setPreferredSize(new Dimension(250,dsGUI.getHeight()));
        // add the datastorage controls

        panel.add(dsGUI);

        // wrap the panel in a flow layout panel in order to shrink the
        // components
        JPanel wrapPanel = new JPanel();
        wrapPanel.add(panel);

        //wrapPanel.add(testLabel);

        // set opacity
        wrapPanel.setBackground(Color.GRAY);
        wrapPanel.setOpaque(false);
        filterPanel.setBackground(Color.GRAY);
        filterPanel.setOpaque(false);
        filterExpressionPanel.setBackground(Color.GRAY);
        filterExpressionPanel.setOpaque(false);
        filterPanel.setVisible(false);
        filterPanel.setVisible(true);
	// wrapPanel.setBorder(BorderFactory.createTitledBorder(i18n.get(this, "name", "Filter")));

        return wrapPanel;

    }

    public java.util.Properties getProperties(String prefix, java.util.Properties properties) {
        return super.getProperties(prefix, properties);

    }

    public void setProperties(String prefix, java.util.Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        super.setProperties(prefix, properties);
    }

    protected boolean setTable() {
        ArrayList<DataObject> datas = getFilteredData();
        if (datas == null) {
            return false;
        }
        dataTable.setTable(datas, dataColumnNames, omColumnNames,
			   getDataStorage().getDataAttributesNamesAliases());
        ((DataTableStoragePanel) dtsp).fireStructureChanged();
        Debug.message("DataStorage", "[FilterDataStorage] send event to " + dsl);
        dsl.dataStorageEvent(this);
        return true;
    }

    protected abstract ArrayList<DataObject> getFilteredData();

    protected abstract void initFilterExpressionPanel();

    protected abstract void setFilterColumnName(String s);

    public DataStorage getDataStorage() {
        return _dataStorage;
    }

    public void setDataStorage(DataStorage dataStorage) {
        _dataStorage = dataStorage;
	// if (dataStorage.isReady()) // if the new dataStorage is ready autosend an event !
	//    dataStorageEvent(dataStorage);
    }
}
