// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.expr;

import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;


import fr.unicaen.openmap.app.ThemaMap;
import fr.unicaen.openmap.dataStorage.DataStorage;
import fr.unicaen.openmap.dataStorage.DataStorageListener;
import fr.unicaen.openmap.dataStorage.InformationDataStorage;
import fr.unicaen.openmap.dataStorage.RemovableDataStorage;
import fr.unicaen.openmap.dataStorage.expr.gui.AddColumnPanel;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.dataStorage.gui.MultiDataTableModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;
import org.antlr.runtime.*;
import org.antlr.runtime.tree.*;
import java.util.List;
import javax.swing.*;

public abstract class ExprDataStorage extends DataStorage
        implements RemovableDataStorage, DataStorageListener {

    public static final String exprPropertyString = "expr";
    public static final String expDSPropertyString = "exprDS";

    private DataStorage _dataStorage = null;
    protected String expr = null;
    protected CommonTree treeExpr = null;
    protected JComponent gui;

    public ExprDataStorage(DataStorageListener dsl) {
        super(dsl);
    }

    public ExprDataStorage(DataStorageListener dsl, DataStorage dataStorage) {
        super(dsl);
        if(dataStorage != null) {
            setDataStorage(dataStorage);
            // we are now the listener
            dataStorage.setDataStorageListener(this);
        }
    }

    public boolean isReady() {
	return getDataStorage() != null && getDataStorage().isReady() && treeExpr != null;
    }

    /** returns the original DataStorage */
    public DataStorage removeItSelf() {
	DataStorage dataStorage = getDataStorage();
        dataStorage.setDataStorageListener(dsl);
        dsl.dataStorageEvent(dataStorage);
        return dataStorage;
    }

    public String getExpr() { return expr; }

    public boolean setExpr(String _expr) {
        // System.out.println("EXPRESSION = " + _expr);
	Debug.message("DataStorage", "[ExprDataStorage.setExpr to: " + _expr);
	if (_expr == null)
	    _expr = "";
	expr = _expr;
	// parse expr...

        MyExprGrammarParser parser = 
	    new MyExprGrammarParser(new CommonTokenStream(
							  new ExprGrammarLexer(new ANTLRStringStream(_expr), 
									       getDefaultDataTableModel())));
	try {
	    treeExpr  = (CommonTree) parser.arg().getTree();
	} catch (org.antlr.runtime.RecognitionException e) {
	    System.err.println("Expr recognition error: " + e); 
	    treeExpr = null;
	    return false;
	}
	boolean ret = parser.getErrMsg() == null;
	Debug.message("DataStorage", "[ExprDataStorage] errmsg: "+parser.getErrMsg()+", ret: " + ret+", expr: "+ _expr + ", tree: " + treeExpr.toStringTree());
	errorLine = parser.getErrorLine();
	errorCharPosInLine = parser.getErrorCharPosInLine();
	return ret;
    }
    private int errorLine;
    private int errorCharPosInLine;
    public int getErrorLine() { return errorLine;}
    public int getErrorCharPosInLine() { return errorCharPosInLine;}

    public void dataStorageEvent(DataStorage source) {
        Debug.message("DataStorage", "[ExprDataStorage] dataStorageEvent <== " + source);

	initialize();
    }
    
    abstract public void initialize();

    public DataTableModel getDefaultDataTableModel() {	    
	if (dataTableModel == null) {
	    Debug.message("DataStorage", "[ExprDataStorage.getDefaultDataTableModel()] is null ");
	    dataTableModel = newMultiDataTableModel();
	}
	
	return dataTableModel;
    }

    abstract protected boolean setColumnNames();

    protected List<String> dataColumnNames = null;
    protected List<String> omColumnNames = null;

    protected DataTableModel newMultiDataTableModel() {
	if (! setColumnNames())
	    return null;
	return new MultiDataTableModel(this, 
				       dataColumnNames.toArray(new String[dataColumnNames.size()]), 
				       omColumnNames.toArray(new String[omColumnNames.size()]));
    }

    public void refreshGUI() { 
	Debug.message("ExprDataStorage", "ExprDataStorage.refreshGUI() not implemented !!!");
    }

    public JComponent getDataGUI(){
	if (dtsp == null) {
	    dataTableModel = getDefaultDataTableModel();
	    // and the data panel
	    dtsp = new DataTableStoragePanel(dataTableModel);
	}
	return dtsp;
    }

    public java.util.Properties getProperties(String prefix, java.util.Properties properties) {
       properties = super.getProperties(prefix, properties);
       prefix = PropUtils.getScopedPropertyPrefix(prefix) + expDSPropertyString + ".";
       properties.put(prefix + exprPropertyString, getExpr());
       return properties;
    }

    public void setProperties(String prefix, java.util.Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix + expDSPropertyString + ".");
        super.setProperties(prefix, properties);
        //System.out.println("expression pour "+prefix + exprPropertyString+"= " + properties.getProperty(prefix + exprPropertyString));
	setExpr(properties.getProperty(prefix + exprPropertyString, ""));
    }

    public DataStorage getDataStorage() {
        return _dataStorage;
    }

    public void setDataStorage(DataStorage dataStorage) {
        _dataStorage = dataStorage;
	// if (dataStorage.isReady()) // if the new dataStorage is ready autosend an event !
	//    dataStorageEvent(dataStorage);
    }
   
}

class MyExprGrammarParser extends ExprGrammarParser {
    
    public MyExprGrammarParser(TokenStream input) {
	super(input);
    }
    // public MyExprGrammarParser(TokenStream input, RecognizerSharedState state) {
    // 	super(input, state);      
    // }
    private String errmsg = null;
    public String getErrMsg() { return errmsg; }
    public void emitErrorMessage(String msg) {
	errmsg = msg;
	System.err.println(msg);
    }
    private int errorLine;
    private int errorCharPosInLine;
    public int getErrorLine() { return errorLine;}
    public int getErrorCharPosInLine() { return errorCharPosInLine;}

    public String getErrorHeader(RecognitionException e) {
	errorLine = e.line;
	errorCharPosInLine = e.charPositionInLine;
	return super.getErrorHeader(e);
    }
}
