// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.expr;

import java.util.ArrayList;
import java.util.List;

import org.antlr.runtime.tree.*;
import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.Centroid;
import fr.unicaen.openmap.dataStorage.BoundingBox;
import fr.unicaen.openmap.dataStorage.OMDataLocation;
import fr.unicaen.openmap.util.Util;
import com.bbn.openmap.geo.Geo;
import com.bbn.openmap.omGraphics.OMPoint;
import com.bbn.openmap.omGraphics.OMCircle;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.proj.Length;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;

public class Function {

    static protected I18n i18n = Environment.getI18n();
    static protected Function f = new Function(); // for i18n

    public enum FunctionRelationnalAndLogical {
	LT("<", i18n.get(f, "lt", "x < y true if x is less than y")),
	    GT(">", i18n.get(f, "gt", "x < y true if x is greater than y")),
	    LTE("<=", i18n.get(f, "lte", "x < y true if x is less than or equal to y")),
	    GTE(">=", i18n.get(f, "gte", "x >= y true if x is greater or equal to y")),
	    EQ("=", i18n.get(f, "eq", "x = y true if x equals y")),
	    NEQ("<>", i18n.get(f, "neq", "x <> y, is true if x and y differs")),
	    LAND("and", i18n.get(f, "land", "a and b, is true if both a and b are true")),
	    LOR("or", i18n.get(f, "lor", "a or b,  true if a is true or b is true"));
	
        private String sign;
	private String doc;
	public String getDoc() { return doc; }

        FunctionRelationnalAndLogical(String s, String doc){
            this.sign = s;
	    this.doc = doc;
        }

        @Override
	    public String toString(){return this.sign;}

        public static String getPattern(){return "";}
    }

    public enum Function_n_n_to_n {
        SUM("+", i18n.get(f, "sum", "x+y, computes the sum of the two numbers")),
	    MUL("*", i18n.get(f, "mul", "x*y, computes the product of the two numbers")),
	    SUB("-", i18n.get(f, "sub", "x-y, computes the difference of the two numbers")),
	    DIV(" / ", i18n.get(f, "div", "x/y, computes the quotient of the two numbers<br>Beware: put space around / in case of confusion with a LAT/LON geometry column name")),
	    MOD("%", i18n.get(f, "mod", "x%y, computes the remainder of the two numbers"));
	
        private String sign;
        private String doc;
	public String getDoc() { return doc; }

        Function_n_n_to_n(String s, String doc){
            this.sign = s;
	    this.doc = doc;
        }

        @Override
	    public String toString(){return this.sign;}

        public static String getPattern(){return "";}
    }

    public enum Function_n_to_n {
	ROUND(i18n.get(f, "round", "round(x), round x to the closest integer")),
	    FLOOR(i18n.get(f, "floor", "floor(x), returns the largest integer  that is less than or equal to x")),
	    CEIL(i18n.get(f, "ceil", "ceil(x), returns the smallest integer  that is greater than or equal to x")),
	    SIN(i18n.get(f, "sin", "sin(x), computes the trigonometric sine of an angle x")),
	    COS(i18n.get(f, "cos", "cos(x), computes the trigonometric cosine of an angle.")),
	    TAN(i18n.get(f, "tan", "tan(x), computes the trigonometric tangent of an angle x")),
	    LOG(i18n.get(f, "log", "log(x), computes the natural logarithm (base e) of x")),
	    LOG10(i18n.get(f, "log10", "log10(x), computes the base 10 logarithm of x")),
	    DEGTOKM(i18n.get(f, "degtokm", "degtokm(x), computes the length in km of an x degrees arc on the surface of the Earth. Beware, it uses a rather crude method")),
	    KMTODEG(i18n.get(f, "kmtodeg", "kmtodeg(x), give the angle of an arc of x km long. Beware, it uses the same crude method as degtokm"));

        public static String getPattern(){return "()";}
	private String doc;
	public String getDoc() { return doc; }
	Function_n_to_n(String doc){
	    this.doc = doc;
        }
        public static double eval(Function_n_to_n f, double d) {
            switch (f) {
	    case ROUND:
		return Math.round(d);
	    case FLOOR:
		return Math.floor(d);
	    case CEIL:
		return Math.ceil(d);
	    case SIN:
		return Math.sin(d);
	    case COS:
		return Math.cos(d);
	    case TAN:
		return Math.tan(d);
	    case LOG:
		return Math.log(d);
	    case LOG10:
		return Math.log10(d);
	    case DEGTOKM:
		return Length.KM.fromRadians(Length.DECIMAL_DEGREE.toRadians(d));
	    case KMTODEG:
		return Length.DECIMAL_DEGREE.fromRadians(Length.KM.toRadians(d));
            }
            return 0;
        }

    }

    static public boolean isFunction_n_to_n(String f) {
        for (Function_n_to_n ff : Function_n_to_n.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }
        return false;
    }

    public enum Function_g_to_n {

        ST_LATITUDE(i18n.get(f, "st_latitude", "st_latitude(geom) returns the latitude of the centroid of the geom")),
	    ST_LONGITUDE(i18n.get(f, "st_longitude", "st_longitude(geom) returns the longitude of the centroid of the geom")),
	    ST_AREA(i18n.get(f, "st_area", "st_area(geom) returns the area of the geom"));


        public static String getPattern(){return "()";}
	private String doc;
	public String getDoc() { return doc; }
	Function_g_to_n(String doc){
	    this.doc = doc;
        }

        public static double eval(Function_g_to_n f, OMDataLocation om) {
	    switch (f) {
	    case ST_LONGITUDE:
		if (om == null || om.getCentroid() == null)
		    return 0;
		return om.getCentroid().getLon();
	    case ST_LATITUDE:
		if (om == null || om.getCentroid() == null)
		    return 0;
		return om.getCentroid().getLat();
	    case ST_AREA:
		if (om == null) return 0;
		if (om.getOMGraphic() == null) return 0;
		// return fr.unicaen.openmap.jts.JTS.area(om.getOMGraphic());
		return SpatialFunction.areaInSQKM(om.getOMGraphic());
            }
            return 0;
        }
    }

    static public boolean isFunction_g_to_n(String f) {
        for (Function_g_to_n ff : Function_g_to_n.values())
            if (ff.toString().equals(f.toUpperCase()))
                return true;
        return false;
    }

    public enum Function_g_g_to_n {

        CENTROIDDISTANCE(i18n.get(f, "centroidDistance", "centroidDistance(geom1, geom2) returns the distance between the centroids of geom1 and geom2"));

        public static double eval(Function_g_g_to_n f, OMDataLocation om1, OMDataLocation om2) {
	    if (om1 == null || om2 == null)
		return 0;
            switch (f) {
	    case CENTROIDDISTANCE:
		if (om1.getCentroid() == null || om2.getCentroid() == null)
		    return 0;
		return Geo.distanceKM(om1.getCentroid().getLat(), om1.getCentroid().getLon(),
				      om2.getCentroid().getLat(), om2.getCentroid().getLon());
            }
            return 0;
        }

        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	Function_g_g_to_n(String doc){
	    this.doc = doc;
        }
    }

    static public boolean isFunction_g_g_to_n(String f) {
        for (Function_g_g_to_n ff : Function_g_g_to_n.values())
            if (ff.toString().equals(f.toUpperCase()))
                return true;
        return false;
    }

    public enum Function_g_g_to_g {

	ST_INTERSECTION(i18n.get(f, "st_intersection", "st_intersection(geom1, geom2) computes a geometry representing the pionts shared by geom1 and geom2")),
        ST_UNION(i18n.get(f, "st_union", "st_union(geom1, geom2) computes a geometry representing all the pints of geom1 and geom2"));

        public static OMDataLocation eval(Function_g_g_to_g f, OMDataLocation om1, OMDataLocation om2) {
	    if (om1 == null || om2 == null)
		return null;
            switch (f) {
	    case ST_UNION:
		if (om1.getOMGraphic() == null || om2.getOMGraphic() == null)
		    return null;
		return SpatialFunction.union(om1, om2);
	    case ST_INTERSECTION:
		if (om1.getOMGraphic() == null || om2.getOMGraphic() == null)
		    return null;
		return SpatialFunction.intersection(om1, om2);
            }
            return null;
        }

        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	Function_g_g_to_g(String doc){
	    this.doc = doc;
        }
    }

    static public boolean isFunction_g_g_to_g(String f) {
        for (Function_g_g_to_g ff : Function_g_g_to_g.values())
            if (ff.toString().equals(f.toUpperCase()))
                return true;
        return false;
    }

    public enum Function_n_n_to_g {

        MAKEPOINT(i18n.get(f, "makepoint", "makePoint(lat, lon) makes a point geometry given a latitude and a longitude"));

        public static OMDataLocation eval(Function_n_n_to_g f, double d1, double d2) {
            switch (f) {
	    case MAKEPOINT:
		return new OMDataLocation(
					  new OMPoint(d1, d2),
					  new Centroid(d2, d1),
					  new BoundingBox(d2, d1, d2, d1));
            }
            return null;
        }

        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	Function_n_n_to_g(String doc){
	    this.doc = doc;
        }
    }

    static public boolean isFunction_n_n_to_g(String f) {
        for (Function_n_n_to_g ff : Function_n_n_to_g.values())
            if (ff.toString().equals(f.toUpperCase()))
                return true;
        return false;
    }

    public enum Function_g_n_to_g {
	ST_BUFFER(i18n.get(f, "st_buffer", "st_buffer(geom, d) makes a geometry that is the buffer of the geom at distance d")),
	    MAKEDISC(i18n.get(f, "makeDisc", "makeDisc(geom, r) makes a disk geometry centered on the centroid of the geom and with the radius r")),
	    SIMPLIFY(i18n.get(f, "simplify", "simplify(geom, t) simplify the geom using the distance tolerance t"));
        public static OMDataLocation eval(Function_g_n_to_g f, OMDataLocation om, double d) {
	    if (om == null)
		return null;
            switch (f) {
	    case ST_BUFFER:
		{
		    if (om.getOMGraphic() == null)
			return null;
		    double lat = om.getCentroid().getLat();
		    double lon = om.getCentroid().getLon();
		    double dInRadians = Length.KM.toRadians(d);
		    return new OMDataLocation(
					      fr.unicaen.openmap.jts.JTS.buffer(om.getOMGraphic(), d),
					      om.getCentroid(),
					      new BoundingBox(lon - dInRadians, lat - dInRadians,
							      lon + dInRadians, lat + dInRadians));
		}
	    case MAKEDISC: 
		{
		    if (om.getCentroid() == null)
			return null;
		    double lat = om.getCentroid().getLat();
		    double lon = om.getCentroid().getLon();
		    double dInRadians = Length.KM.toRadians(d);
		    return new OMDataLocation(
					      new OMCircle(lat, lon, d, Length.KM),
					      om.getCentroid(),
					      new BoundingBox(lon - dInRadians, lat - dInRadians,
							      lon + dInRadians, lat + dInRadians));
		}
	    case SIMPLIFY:
		{
		    if (om.getOMGraphic() == null)
			return null;
		    return SpatialFunction.simplify(om, d);
		}
	    }
	    return null;
	}

        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	Function_g_n_to_g(String doc){
	    this.doc = doc;
        }
    }

    static public boolean isFunction_g_n_to_g(String f) {
        for (Function_g_n_to_g ff : Function_g_n_to_g.values())
            if (ff.toString().equals(f.toUpperCase()))
                return true;
        return false;
    }


    public enum Function_s_to_n {

	LENGTH(i18n.get(f, "length", "length(s), returns the number of character of the string s"));

        public static String getPattern(){return "()";}
	private String doc;
	public String getDoc() { return doc; }
	Function_s_to_n(String doc){
	    this.doc = doc;
        }
        public static double eval(Function_s_to_n f, String s) {
            switch (f) {
	    case LENGTH:
		return s.length();
            }
            return 0;
        }

    }

    static public boolean isFunction_s_to_n(String f) {
        for (Function_s_to_n ff : Function_s_to_n.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }
        return false;
    }

    public enum Function_s_to_s {

	TRIM(i18n.get(f, "trim", "trim(s), trims white spaces at both ends of the string s")),
	    ORD(i18n.get(f, "ord", "ord(s), returns the code of the first character of the string s")),
	    CHAR(i18n.get(f, "char", "char(n), returns a character whose code is n"));

        public static String getPattern(){return "()";}
	private String doc;
	public String getDoc() { return doc; }
	Function_s_to_s(String doc){
	    this.doc = doc;
        }
        public static Object eval(Function_s_to_s f, String s) {
            switch (f) {
	    case TRIM:
		return s.trim();
	    case ORD:
		return s.length() > 0 ? s.codePointAt(0) : 0;
	    case CHAR:
		return new String(Character.toChars((int)parseDouble(s)));
            }
            return 0;
        }

    }

    static public boolean isFunction_s_to_s(String f) {
        for (Function_s_to_s ff : Function_s_to_s.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }
        return false;
    }


    public enum Function_s_splus_to_s {

	CONCAT(i18n.get(f, "concat", "concat(s1, s2, ...), catenate the strings s1, s2, ..."));

        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	Function_s_splus_to_s(String doc){
	    this.doc = doc;
        }
        public static Object eval(Function_s_splus_to_s f, String ... args) {
            switch (f) {
	    case CONCAT:
		{
		    StringBuffer buffer = new StringBuffer();
		    for (String s: args)
			buffer.append(s);
		    return buffer.toString();
		}
            }
            return 0;
        }

    }

    static public boolean isFunction_s_splus_to_s(String f) {
        for (Function_s_splus_to_s ff : Function_s_splus_to_s.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }	
        return false;
    }
    
    public enum Function_s_n1or2_to_s {

	SUBSTRING(i18n.get(f, "substring", "substring(s, i[, j]), extract a substring from s starting at index i and extending to character at index j or at the end of the string s if j is omitted. Indexes start at 0."));

        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	Function_s_n1or2_to_s(String doc){
	    this.doc = doc;
        }
        public static Object eval(Function_s_n1or2_to_s f, String s, Double... args) {
	    try {
		switch (f) {
		case SUBSTRING:
		    if (args.length == 1)
			return s.substring((int)Math.round(args[0]));
		    return s.substring((int)Math.round(args[0]), (int)Math.round(args[1]));
		}
	    } catch(Exception e) {
		System.err.println("Error for "+f+"("+s+"...)");
	    }
            return f;
        }

    }

    static public boolean isFunction_s_n1or2_to_s(String f) {
        for (Function_s_n1or2_to_s ff : Function_s_n1or2_to_s.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }
        return false;
    }


    public static boolean isGeometryType(CommonTree tree) {
	return tree.getType() == ExprGrammarLexer.OMCOLUMNNAME
	    || tree.getType() == ExprGrammarLexer.FUNCTION_N_N_TO_G
	    || tree.getType() == ExprGrammarLexer.FUNCTION_G_N_TO_G;
    }

    public static Object eval(CommonTree tree, DataObject data) {
	try {
	    switch (tree.getType()) {
	    case ExprGrammarLexer.TRUE:
		return true; // new Integer(1);
	    case ExprGrammarLexer.FALSE:
		return false; // new Integer(0);
	    case ExprGrammarLexer.INT:
		return Integer.parseInt(tree.getText());
	    case ExprGrammarLexer.FLOAT:
		return Double.parseDouble(tree.getText());
	    case ExprGrammarLexer.STRING:
		return tree.getText().substring(1, tree.getText().length()-1);
		// case ExprGrammarLexer.ID:                return tree.getText();

	    case ExprGrammarLexer.DATACOLUMNNAME:
		return data.getData(tree.getText());
	    case ExprGrammarLexer.OMCOLUMNNAME:
		return data.getOMDataLocation(tree.getText());

	    case ExprGrammarLexer.POS:
		return parseDouble(eval((CommonTree) tree.getChild(0), data));
	    case ExprGrammarLexer.NEG:
		return -parseDouble(eval((CommonTree) tree.getChild(0), data));
	    case ExprGrammarLexer.MUL:
		return parseDouble(eval((CommonTree) tree.getChild(0), data)) * parseDouble(eval((CommonTree) tree.getChild(1), data));
	    case ExprGrammarLexer.DIV:
		return parseDouble(eval((CommonTree) tree.getChild(0), data)) / parseDouble(eval((CommonTree) tree.getChild(1), data));
	    case ExprGrammarLexer.MOD:
		return parseDouble(eval((CommonTree) tree.getChild(0), data)) % parseDouble(eval((CommonTree) tree.getChild(1), data));
	    case ExprGrammarLexer.ADD:
		return parseDouble(eval((CommonTree) tree.getChild(0), data)) + parseDouble(eval((CommonTree) tree.getChild(1), data));
	    case ExprGrammarLexer.SUB:
		return parseDouble(eval((CommonTree) tree.getChild(0), data)) - parseDouble(eval((CommonTree) tree.getChild(1), data));

		// for relationnal operator, we should first test for the types of the args
		// Bool, Number, Strings, ...
	    case ExprGrammarLexer.LT:
	    case ExprGrammarLexer.GT:
	    case ExprGrammarLexer.LTE:
	    case ExprGrammarLexer.GTE:
	    case ExprGrammarLexer.EQ:
	    case ExprGrammarLexer.NEQ:
		return evalRelOp(tree.getType(), eval((CommonTree) tree.getChild(0), data), eval((CommonTree) tree.getChild(1), data));

	    case ExprGrammarLexer.LAND:
		return toBoolean(eval((CommonTree) tree.getChild(0), data)) && toBoolean(eval((CommonTree) tree.getChild(1), data));
	    case ExprGrammarLexer.LOR:
		return toBoolean(eval((CommonTree) tree.getChild(0), data)) || toBoolean(eval((CommonTree) tree.getChild(1), data));
	    case ExprGrammarLexer.IF:
		return toBoolean(eval((CommonTree) tree.getChild(0), data))
		    ? eval((CommonTree) tree.getChild(1), data)
		    : eval((CommonTree) tree.getChild(2), data);

	    case ExprGrammarLexer.FUNCTION_N_TO_N:
		return Function_n_to_n.eval(Function_n_to_n.valueOf(tree.getText().toUpperCase()),
					    parseDouble(eval((CommonTree)tree.getChild(0), data)));

	    case ExprGrammarLexer.FUNCTION_G_TO_N:
		return Function_g_to_n.eval(Function_g_to_n.valueOf(tree.getText().toUpperCase()),
					    (OMDataLocation)eval((CommonTree)tree.getChild(0), data));

	    case ExprGrammarLexer.FUNCTION_G_G_TO_N:
		return Function_g_g_to_n.eval(Function_g_g_to_n.valueOf(tree.getText().toUpperCase()),
					      (OMDataLocation)eval((CommonTree)tree.getChild(0), data),
					      (OMDataLocation)eval((CommonTree)tree.getChild(1), data));
	    
	    case ExprGrammarLexer.FUNCTION_N_N_TO_G:
		return Function_n_n_to_g.eval(Function_n_n_to_g.valueOf(tree.getText().toUpperCase()),
					      parseDouble(eval((CommonTree)tree.getChild(0), data)),
					      parseDouble(eval((CommonTree)tree.getChild(1), data)));
	    case ExprGrammarLexer.FUNCTION_G_N_TO_G:
		return Function_g_n_to_g.eval(Function_g_n_to_g.valueOf(tree.getText().toUpperCase()),
					      (OMDataLocation)eval((CommonTree)tree.getChild(0), data),
					      parseDouble(eval((CommonTree)tree.getChild(1), data)));
	    case ExprGrammarLexer.FUNCTION_G_G_TO_G:
		return Function_g_g_to_n.eval(Function_g_g_to_n.valueOf(tree.getText().toUpperCase()),
					      (OMDataLocation)eval((CommonTree)tree.getChild(0), data),
					      (OMDataLocation)eval((CommonTree)tree.getChild(1), data));
	    case ExprGrammarLexer.FUNCTION_S_TO_N:
		return Function_s_to_n.eval(Function_s_to_n.valueOf(tree.getText().toUpperCase()),
					    eval((CommonTree)tree.getChild(0), data).toString());
	    case ExprGrammarLexer.FUNCTION_S_TO_S:
		return Function_s_to_s.eval(Function_s_to_s.valueOf(tree.getText().toUpperCase()),
					    eval((CommonTree)tree.getChild(0), data).toString());
	    case ExprGrammarLexer.FUNCTION_S_SPLUS_TO_S:
		{
		    String[] args = new String[tree.getChildCount()];
		    for (int i = 0; i < tree.getChildCount(); i++)
			args[i] = eval((CommonTree)tree.getChild(i), data).toString();
		    return Function_s_splus_to_s.eval(Function_s_splus_to_s.valueOf(tree.getText().toUpperCase()),
						      args);
		}
	    case ExprGrammarLexer.FUNCTION_S_N1OR2_TO_S:
		{
		    Double[] args = new Double[tree.getChildCount()-1];
		    for (int i = 1; i < tree.getChildCount(); i++)
			args[i-1] = parseDouble(eval((CommonTree)tree.getChild(i), data).toString());
		    return Function_s_n1or2_to_s.eval(Function_s_n1or2_to_s.valueOf(tree.getText().toUpperCase()), 
						      eval((CommonTree)tree.getChild(0), data).toString(),
						      args);

		}
	    }
	    System.err.println("***[Eval expr] invalid token type; "+tree.getType());
	} catch(Exception e) {
	    System.err.println("***[Eval expr] exception "+e+" for "+tree.getType());
	}
	return null;
    }

    private static Boolean evalRelOp(int op, Object child0, Object child1) { 
	// System.out.println(op+" "+(child0==null? "null" :child0.getClass().getName())+" "+(child1 ==null? "null" :child1.getClass().getName()));
	if (child0 instanceof String && child1 instanceof String) {
	    int cmp = ((String)child0).compareTo((String)child1);	    
	    // System.out.println("Compare 2 strings: "+child0 +" "+child1+" -> "+cmp);
	    switch(op) {
            case ExprGrammarLexer.LT:
                return cmp < 0;
            case ExprGrammarLexer.GT:
                return cmp > 0;
            case ExprGrammarLexer.LTE:
                return cmp <= 0;
            case ExprGrammarLexer.GTE:
                return cmp >= 0;
            case ExprGrammarLexer.EQ:
                return cmp == 0;
            case ExprGrammarLexer.NEQ:
                return cmp != 0;
	    }
	    return false;
	}
	if (child0 instanceof Boolean && child1 instanceof Boolean) {
	    int bchild0 = ((Boolean)child0) ? 1 : 0;
	    int bchild1 = ((Boolean)child1) ? 1 : 0;	    
	    switch(op) {
            case ExprGrammarLexer.LT:
                return bchild0 < bchild1;
            case ExprGrammarLexer.GT:
                return bchild0 > bchild1;
            case ExprGrammarLexer.LTE:
                return bchild0 <= bchild1;
            case ExprGrammarLexer.GTE:
                return bchild0 >= bchild1;
            case ExprGrammarLexer.EQ:
                return bchild0 == bchild1;
            case ExprGrammarLexer.NEQ:
                return bchild0 != bchild1;
	    }
	    return false;
	}
	double dchild0 = parseDouble(child0);
	double dchild1 = parseDouble(child1);
	switch(op) {
	case ExprGrammarLexer.LT:
	    return dchild0 < dchild1;
	case ExprGrammarLexer.GT:
	    return dchild0 > dchild1;
	case ExprGrammarLexer.LTE:
	    return dchild0 <= dchild1;
	case ExprGrammarLexer.GTE:
	    return dchild0 >= dchild1;
	case ExprGrammarLexer.EQ:
	    return dchild0 == dchild1;
	case ExprGrammarLexer.NEQ:
	    return dchild0 != dchild1;
	}
	return false;
    }

    static private boolean toBoolean(Object o) {
        if (o instanceof Boolean)
            return (Boolean) o;
        
        if (o instanceof String)
            return ((String) o).matches("true");
        
        return parseDouble(o) != 0;
    }

    static double parseDouble(Object o) {
	if (o instanceof Boolean) 
	    return (Boolean)o ? 1 : 0;
	return Util.parseDouble(o);    
    }

    static public boolean isFunctionName(String name) {
	return  isFunction_n_to_n(name)
	    || isFunction_g_to_n(name)
	    || isFunction_g_g_to_n(name)
	    || isFunction_n_n_to_g(name)
	    || isFunction_g_n_to_g(name);
    } 
    
    static public String getFunctionsDocs() {
	StringBuffer buffer = new StringBuffer();
	buffer.append("<h2>"+i18n.get(f, "unaryfunction","Numerical unary functions")+"</h2>");
	for (Function_n_to_n ff : Function_n_to_n.values())
	    buffer.append("<p>"+ff.getDoc()+"<p>");

	buffer.append("<h2>"+i18n.get(f, "binaryfunction","Numerical binary functions")+"</h2>");
	for (Function_n_n_to_n ff : Function_n_n_to_n.values())
	    buffer.append("<p>"+ff.getDoc()+"<p>");

	buffer.append("<h2>"+i18n.get(f, "g_to_n_function","Numerical functions from geometries")+"</h2>");
	for (Function_g_to_n ff : Function_g_to_n.values())
	    buffer.append("<p>"+ff.getDoc()+"<p>");

	buffer.append("<h2></h2>");
	// buffer.append("<h2>"+i18n.get(f, "g_g_to_n_function","Numerical functions from 2 geometries")+"</h2>");
	for (Function_g_g_to_n ff : Function_g_g_to_n.values())
	    buffer.append("<p>"+ff.getDoc()+"<p>");

	buffer.append("<h2>"+i18n.get(f, "to_g_function","Geometry constructors")+"</h2>");
	for (Function_n_n_to_g ff : Function_n_n_to_g.values())
	    buffer.append("<p>"+ff.getDoc()+"<p>");
     
	buffer.append("<h2></h2>");
	// buffer.append("<h2>"+i18n.get(f, "g_n_to_g_function","Geometry constructors with a geometry and a number")+"</h2>");
	for (Function_g_n_to_g ff : Function_g_n_to_g.values())
	    buffer.append("<p>"+ff.getDoc()+"<p>");

	for (Function_g_g_to_g ff : Function_g_g_to_g.values())
	    buffer.append("<p>"+ff.getDoc()+"<p>");

	buffer.append("<h2>"+i18n.get(f, "s_function","")+"String manipulation functions</h2>");
       for (Function_s_to_n ff : Function_s_to_n.values())
	    buffer.append("<p>"+ff.getDoc()+"<p>");

        for (Function_s_to_s ff : Function_s_to_s.values())
	    buffer.append("<p>"+ff.getDoc()+"<p>");

        for (Function_s_splus_to_s ff : Function_s_splus_to_s.values())
	    buffer.append("<p>"+ff.getDoc()+"<p>");

        for (Function_s_n1or2_to_s ff : Function_s_n1or2_to_s.values())
	    buffer.append("<p>"+ff.getDoc()+"<p>");

    	return buffer.toString();
    }

    static public List<FunctionDescription> getDescriptions() {
	List<FunctionDescription> descriptions = new ArrayList<FunctionDescription>();
	for (Function_n_to_n ff : Function_n_to_n.values())
	    descriptions.add(new FunctionDescription(ff.toString().toLowerCase(), ff.getPattern(), ff.getDoc()));
	for (Function_n_n_to_n ff : Function_n_n_to_n.values())
	    descriptions.add(new FunctionDescription(ff.toString().toLowerCase(), ff.getPattern(), ff.getDoc()));
	for (Function_g_to_n ff : Function_g_to_n.values())
	    descriptions.add(new FunctionDescription(ff.toString().toLowerCase(), ff.getPattern(), ff.getDoc()));
	for (Function_g_g_to_n ff : Function_g_g_to_n.values())
	    descriptions.add(new FunctionDescription(ff.toString().toLowerCase(), ff.getPattern(), ff.getDoc()));
	for (Function_n_n_to_g ff : Function_n_n_to_g.values())
	    descriptions.add(new FunctionDescription(ff.toString().toLowerCase(), ff.getPattern(), ff.getDoc()));
	for (Function_g_n_to_g ff : Function_g_n_to_g.values())
	    descriptions.add(new FunctionDescription(ff.toString().toLowerCase(), ff.getPattern(), ff.getDoc()));
	for (Function_g_g_to_g ff : Function_g_g_to_g.values())
	    descriptions.add(new FunctionDescription(ff.toString().toLowerCase(), ff.getPattern(), ff.getDoc()));
	for (Function_s_to_n ff : Function_s_to_n.values())
	    descriptions.add(new FunctionDescription(ff.toString().toLowerCase(), ff.getPattern(), ff.getDoc()));
        for (Function_s_to_s ff : Function_s_to_s.values())
	    descriptions.add(new FunctionDescription(ff.toString().toLowerCase(), ff.getPattern(), ff.getDoc()));
        for (Function_s_splus_to_s ff : Function_s_splus_to_s.values())
	    descriptions.add(new FunctionDescription(ff.toString().toLowerCase(), ff.getPattern(), ff.getDoc()));
        for (Function_s_n1or2_to_s ff : Function_s_n1or2_to_s.values())
	    descriptions.add(new FunctionDescription(ff.toString().toLowerCase(), ff.getPattern(), ff.getDoc()));
        for (FunctionRelationnalAndLogical ff : FunctionRelationnalAndLogical.values())
	    descriptions.add(new FunctionDescription(ff.toString().toLowerCase(), ff.getPattern(), ff.getDoc()));

	descriptions.add(new FunctionDescription("if", "(,,)",
						 i18n.get(f, "if", "if(b, x, y) gives x if b is true, y oterwise")));

	return descriptions;
    }
}
