// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors
 */
// </header>
package fr.unicaen.openmap.dataStorage.expr.gui;

import fr.unicaen.openmap.dataStorage.expr.ExprDataStorage;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;

/**
 *
 * @author jchauvea
 */
public class ExpressionPanel extends JPanel {

    private JTextArea expressionArea;
    private JButton execButton;
    private JButton clearButton;
    private JLabel labState;
    private ExprDataStorage exprDs;
    private JLabel carretLabel;
    public final static String ERROR_TAG_LEFT = "<font color=\"#ff0000\">";
    public final static String ERROR_TAG_RIGHT = "</font>";
    public final static String SPAN_TAG_LEFT = "<span>";
    public final static String SPAN_TAG_RIGHT = "</span>";
    public final static String BODY_TAG_LEFT = "<body>";
    public final static String BODY_TAG_RIGHT = "</body>";
    public final static String SPAN_TAGS_PATTERN = "<span>.*</span>";
    public final static String BODY_TAGS_PATTERN = "<body>.*</body>";
    public final static char PARENTHESIS_PATTERN = '(';

    public ExpressionPanel(ExprDataStorage ds) {
        super();
        this.exprDs = ds;
        Box box = Box.createVerticalBox();
        JPanel top = new JPanel();
        JPanel error = new JPanel();
        expressionArea = new JTextArea(1, 30);
        ComponentSizer.fixComponentSize(expressionArea, new Dimension(300, 20));
        execButton = new JButton("Exécuter");
        clearButton = new JButton("Effacer");
        JPanel pBottom = new JPanel();
        labState = new JLabel(Util.loadIcon("transparent.png"));


        execButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent ae) {

                exprDs.initialize();
                labState.setIcon(Util.loadIcon("green.png"));
            }
        });

        clearButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                expressionArea.setText("");
                labState.setIcon(Util.loadIcon("transparent.png"));
            }
        });

        add(pBottom, BorderLayout.SOUTH);
        expressionArea.addKeyListener(new KeyListener() {

            public void keyTyped(KeyEvent ke) {
            }

            public void keyPressed(KeyEvent ke) {
            }

            public void keyReleased(KeyEvent ke) {
                updateExpression();
                labState.setIcon(Util.loadIcon("transparent.png"));

            }
        });

        top.add(new JLabel("Expression"));
        top.add(expressionArea);
        top.add(execButton);
        top.add(clearButton);
        top.add(labState);

        carretLabel = new JLabel("");
        error.add(carretLabel);
        box.add(top);
        expressionArea.setText(ds.getExpr());
        add(box);
    }

    public void appendText(String textToAdd) {
        int caretPosition = expressionArea.getCaretPosition();
        int offset = textToAdd.indexOf(PARENTHESIS_PATTERN);
        if (offset == -1) {
            offset = textToAdd.length();
        } else {
            offset++;
        }
        expressionArea.insert(textToAdd, caretPosition);
        expressionArea.setCaretPosition(caretPosition + offset);
        updateExpression();
    }

    public void updateExpression() {
        String expr = expressionArea.getText();
        boolean r = exprDs.setExpr(expr);
        if (r) {
            execButton.setEnabled(true);
        } else {
            execButton.setEnabled(false);
        }
        labState.updateUI();

    }
}
