// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.gui;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;

import fr.unicaen.openmap.dataStorage.BoundingBox;


/**
 * BoundingBox GUI 
 * @author Jacques.Madelaine@info.unicaen.fr
 */

public class BoundingBoxPanel extends JPanel {
    
    protected I18n i18n = Environment.getI18n();
    
    public BoundingBoxPanel(final BoundingBox bb) {
	JPanel pan;
	{
	    pan = new JPanel();
	    pan.add(new JLabel(i18n.get(BoundingBoxPanel.class, "upperLeftLon", "Upper Left Lon:")));
	    final JSpinner field = new JSpinner(new SpinnerNumberModel(bb.getUpperLeftLon(), -180d, 180d, 0d));
	    field.addChangeListener(new ChangeListener() {
		    public void stateChanged(ChangeEvent e) {
			bb.setUpperLeftLon(((Double) field.getValue()).floatValue());
		    }
		});
	    field.setToolTipText(i18n.get(BoundingBoxPanel.class, "upperLeftLon_tip", "Upper Left Lon"));
	    pan.add(field);
	    add(pan);
	}

	{
	    pan = new JPanel();
	    pan.add(new JLabel(i18n.get(BoundingBoxPanel.class, "upperLeftLat",  "Lat:")));
	    final JSpinner field = new JSpinner(new SpinnerNumberModel(bb.getUpperLeftLat(), -90d, 90d, 0d));
	    field.addChangeListener(new ChangeListener() {
		    public void stateChanged(ChangeEvent e) {
			bb.setUpperLeftLat(((Double) field.getValue()).floatValue());
		    }
		});
	    field.setToolTipText(i18n.get(BoundingBoxPanel.class, "upperLeftLat_tip", "Upper Left Lat"));
	    pan.add(field);
	    add(pan);
	}

	{
	    pan = new JPanel();
	    pan.add(new JLabel(i18n.get(BoundingBoxPanel.class, "lowerRightLon", "Lower Right Lon:")));
	    final JSpinner field = new JSpinner(new SpinnerNumberModel(bb.getLowerRightLon(), -180d, 180d, 0d));
	    field.addChangeListener(new ChangeListener() {
		    public void stateChanged(ChangeEvent e) {
			bb.setLowerRightLon(((Double) field.getValue()).floatValue());
		    }
		});
	    field.setToolTipText(i18n.get(BoundingBoxPanel.class, "lowerRightLon_tip", "Lower Right Lon"));
	    pan.add(field);
	    add(pan);
	}

	{
	    pan = new JPanel();
	    pan.add(new JLabel(i18n.get(BoundingBoxPanel.class, "lowerRightLat", "Lat:")));
	    final JSpinner field = new JSpinner(new SpinnerNumberModel(bb.getLowerRightLat(), -90d, 90d, 0d));
	    field.addChangeListener(new ChangeListener() {
		    public void stateChanged(ChangeEvent e) {
			bb.setLowerRightLat(((Double) field.getValue()).floatValue());
		    }
		});
	    field.setToolTipText(i18n.get(BoundingBoxPanel.class, "lowerRightLat_tip", "Lower Right Lat"));
	    pan.add(field);
	    add(pan);
	}
    }
    
}
