// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * ColorPanel.java
 *
 * Created on 9 novembre 2005, 23:31
 *
 * @author Christophe.Turbout@info.unicaen.fr
 *
 */

package fr.unicaen.openmap.dataStorage.gui;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import com.bbn.openmap.omGraphics.OMColor;
import com.bbn.openmap.omGraphics.OMColorChooser;
import com.bbn.openmap.util.ColorFactory;
import fr.unicaen.openmap.dataStorage.SelectionDataStorage;
import fr.unicaen.openmap.gui.ThemaMapColorChooser;
import fr.unicaen.openmap.layer.preLayer.UnicaenLayerListener;
import fr.unicaen.openmap.util.IndexedColorButton;
import fr.unicaen.openmap.util.LabeledPairLayout;
import fr.unicaen.openmap.util.Util;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Locale;
import java.util.ArrayList;
import javax.swing.*;

/**
 * This class is used to display the content of ArrayList<SelectionMatrix>.
 *
 */
public class ColorPanel extends JPanel implements ActionListener {
    private JPanel panel = null;
    private UnicaenLayerListener unicaenLayerListener = null;
    private SelectionDataStorage selectionDataStorage= null;
    
    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     **/
    protected I18n i18n = new BasicI18n(new Locale("fr","FR"));
    
    public  ColorPanel(SelectionDataStorage storage, int selection) {
        super();
        setDoubleBuffered(true);
        setLayout(new BorderLayout());
        selectionDataStorage=storage;
        panel = new JPanel();
        panel.setLayout(new LabeledPairLayout());
         // the components
        try {
            ArrayList<Color> colors = storage.getSelectionColors(selection);
            ArrayList<String> names = storage.getSubSelectionNames(selection);
            for (int i=0; i < colors.size() ;i++){
                IndexedColorButton button = addButton(names.get(i), colors.get(i), i);
            }
        } catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
        add(panel, BorderLayout.CENTER);
    }
    
    protected Color showDialog(IndexedColorButton button) {
        Color c;
        try { c = ColorFactory.parseColor(button.getText(), true); } catch (Exception exception) { c = OMColor.clear; }
        return ThemaMapColorChooser.showDialog(button, "Select color...", c);
    }
    
    protected IndexedColorButton addButton(String name, Color color, int index) {
        IndexedColorButton button = new IndexedColorButton(color,index);
        button.addActionListener(this);
        panel.add(new JLabel(name), "label");
        panel.add(button, "field");
        return button;
    }
    
    public void actionPerformed(ActionEvent event) {
        if (event.getSource() instanceof IndexedColorButton) {
            IndexedColorButton button = (IndexedColorButton) event.getSource();
            Color newColor = showDialog(button);
            button.setColor(newColor);
            // we need to update the color with the selected value.
            selectionDataStorage.setSubSelectionColor(button.getIndex(),newColor);
            // need to refresh();
            if (selectionDataStorage!=null) selectionDataStorage.fireUpdateColors();
            if (unicaenLayerListener!=null) unicaenLayerListener.refreshGraphics();
        }
    }
    
    /**
     *  Add UnicaenLayerListener to refresh layer();
     */
    public void addUnicaenListener(UnicaenLayerListener ull){
        unicaenLayerListener=ull;
    }
    
    /**
     *  Remove the UnicaenLayerListener;
     */
    public void removeUnicaenListener(){
        unicaenLayerListener=null;
    }
    
    /**
     * Add a new color which is store at subselection index.
     */
    public void addColor(ArrayList<String> names, ArrayList<Color> colors, int subselection){
        IndexedColorButton button = addButton(names.get(subselection), colors.get(subselection), subselection);
    }
    
}
