// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * ColumnHeaderRenderer.java
 *
 * Created on 28 novembre 2005, 11:37
 *
 * @author Christophe.Turbout@info.unicaen.fr
 *
 */

package fr.unicaen.openmap.dataStorage.gui;
import fr.unicaen.openmap.dataStorage.SelectionDataStorage;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.io.BufferedReader;
import java.io.StringReader;
import java.util.ArrayList;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.table.TableCellRenderer;

/**
 * This method give use this ability to write a multiline header in a JTable.
 *
 * @author Christophe.Turbout@info.unicaen.fr
 *
 */
public class ColumnHeaderRenderer extends JPanel implements TableCellRenderer {
    
    private SelectionDataStorage selectionDataStorage= null;
    private int initcol = -1;
    private JPanel panel = null;
    
    public ColumnHeaderRenderer(SelectionDataStorage storage, int initcol){
        this.selectionDataStorage=storage;
        this.initcol=initcol;
        setOpaque(true);
        setLayout(new GridLayout(0,1, 5,5));
        setForeground(UIManager.getColor("TableHeader.foreground"));
        setBackground(UIManager.getColor("TableHeader.background"));
        setBorder(UIManager.getBorder("TableHeader.cellBorder"));
        panel = new JPanel();
        panel.setLayout(new GridLayout(0,1));
        panel.setOpaque(true);
        panel.setForeground(UIManager.getColor("TableHeader.foreground"));
        panel.setBackground(UIManager.getColor("TableHeader.background"));
        add(panel);
    }
    
    public Component getTableCellRendererComponent(JTable table, Object value,
            boolean isSelected, boolean hasFocus, int rowIndex, int vColIndex) {
        
        panel.removeAll();
        panel.invalidate();
        
        ArrayList<Color> colors = selectionDataStorage.getSelectionColors(selectionDataStorage.getCurrentSelection());
        
        panel.setFont(table.getFont());
        if (colors!=null && vColIndex>=initcol && (vColIndex-initcol)<colors.size()) panel.setBackground(colors.get(vColIndex-initcol));
        String str = (value == null) ? "" : value.toString();
        BufferedReader br = new BufferedReader(new StringReader(str));
        String line;
        ArrayList<String> values = new ArrayList<String>();
        try {
            while ((line = br.readLine()) != null) {
                values.add(line);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        int height=15;
        int width=-15;
        
        // Configure each row of the header using a separate JLabel.
        for (int i = 0; i < values.size(); i++) {
            String thisRow = values.get(i);
            JLabel label = new JLabel(thisRow,null, JLabel.CENTER);
            label.setOpaque(true);
            label.setFont(table.getFont());
            panel.add(label);
            height+=label.getMinimumSize().getHeight();
            width+=label.getMinimumSize().getWidth();
            if (colors!=null && vColIndex>=initcol && (vColIndex-initcol)<colors.size()) label.setBackground(colors.get(vColIndex-initcol));
        }
        setMinimumSize(new Dimension(width,height));
        setPreferredSize(new Dimension(width,height));
        return this;
    }
}
