// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * SelectionMatrixPanel.java
 *
 * Created on 9 novembre 2005, 23:31
 *
 * @author Christophe.Turbout@info.unicaen.fr
 *
 */

package fr.unicaen.openmap.dataStorage.gui;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.dataStorage.SelectionDataStorage;
import fr.unicaen.openmap.dataStorage.SelectionMatrix;
import fr.unicaen.openmap.util.Util;
import java.awt.*;
import java.util.ArrayList;
import java.util.Locale;
import java.util.Vector;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;



/**
 * This class is used to display the content of Vector<SelectionMatrix>, one column math one SelectionColumnMatrixModel.
 *
 */
public class SelectionColumnMatrixPanel extends JPanel implements ChangeListener{
    private JTabbedPane tabs;
    private SelectionDataStorage selectionDataStorage = null;
    private ArrayList<SelectionColumnMatrixModel> models = null;    
    private BorderLayout currentLayout = null;
    private SelectionMatrix matrix = null;
    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     **/
    protected I18n i18n = new BasicI18n(new Locale("fr","FR"));
    private ArrayList<Integer> coltosum = null;
    
    public  SelectionColumnMatrixPanel(SelectionDataStorage selectionDataStorage, int selection) {
        super();
        this.selectionDataStorage = selectionDataStorage;
        setDoubleBuffered(true);
        currentLayout = new BorderLayout();
        setLayout(currentLayout);
        models = new  ArrayList<SelectionColumnMatrixModel>();
        //  JTabpanned
        tabs = new CustomTabbedPane(selectionDataStorage);
        add(tabs, BorderLayout.CENTER);
        
        // we select column to sum manually ... we have to put it with properties
        coltosum = new ArrayList<Integer>();
        coltosum.add(2);
        coltosum.add(3);
        
        tabs.addChangeListener(this);
        // the components
        try {
            matrix = selectionDataStorage.getSelectionMatrix(selection);            
            for (int i=0;i<matrix.getNbCols();i++){
                SelectionColumnMatrixModel smm = new SelectionColumnMatrixModel(selectionDataStorage.getDefaultDataTableModel(),matrix, i,coltosum);
                models.add(smm);
                tabs.addTab(selectionDataStorage.getSubSelectionName(selection,i),new DataTableSelectionStoragePanel(selectionDataStorage,smm));
                tabs.setBackgroundAt(i,selectionDataStorage.getSubSelectionColor(selection,i));
            }
            if (matrix.getNbRows()>-1) {
                selectionDataStorage.setCurrentSubSelection(0);
                matrix.setCurrentSubSelection(0);
                tabs.setSelectedIndex(0);
            }
        } catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
    }
    
    /**
     * When state is changed, we need to update SelectionDataStorage to set the currentSubSelection.
     **/
    public void stateChanged(ChangeEvent e) {
        int selectedTab = ((JTabbedPane)e.getSource()).getSelectedIndex();
        selectionDataStorage.setCurrentSubSelection(selectedTab);
    }
    
    public void fireColumnAdded(int selection, int subselection){
        SelectionColumnMatrixModel smm = new SelectionColumnMatrixModel(selectionDataStorage.getDefaultDataTableModel(),selectionDataStorage.getSelectionMatrix(selection), subselection,coltosum);
        models.add(subselection,smm);
        DataTableSelectionStoragePanel dtssp = new DataTableSelectionStoragePanel(selectionDataStorage,smm);
        tabs.addTab(selectionDataStorage.getSubSelectionName(selection,subselection),dtssp);
        tabs.setBackgroundAt(subselection,selectionDataStorage.getSubSelectionColor(selection,subselection));
        tabs.setSelectedIndex(subselection);
    }
    
    public void fireColumnRemoved(int selection, int subselection){
        // we need to update all the model with the new culomn value.
        for (int i=subselection+1; i<models.size();i++) models.get(i).setSubSelectionValue(i-1);
        models.remove(subselection);
        tabs.remove(subselection);
        tabs.setSelectedIndex(tabs.getTabCount()-1);
    }
    
    public void fireDataChanged(int selection, int subSelection, int row, boolean state){
        Debug.message(Util.className(this),"TRACE ==> models.size() = "+models.size());
        Debug.message(Util.className(this),"TRACE ==> selection = "+selection);
        Debug.message(Util.className(this),"TRACE ==> subSelection = "+subSelection);
        Debug.message(Util.className(this),"TRACE ==> row = "+row);        
        if (row<0) models.get(subSelection).fireTableStructureChanged();
        else {
            if (state) models.get(subSelection).fireTableRowsInserted(row,row);
            else models.get(subSelection).fireTableRowsDeleted(row,row);
        }
    }
    
    public void fireUpdateColors(int selection){
        for(int i=0; i< tabs.getTabCount(); i++){
            tabs.setBackgroundAt(i,selectionDataStorage.getSubSelectionColor(selection,i));
        }
        
    }
    
     public int getSubSelectionsSize() {
        return matrix.getSubSelectionsSize();
    }
}
