// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.kernel;

import java.awt.geom.Point2D;
import java.util.ArrayList;

import com.bbn.openmap.util.Debug;

import fr.unicaen.openmap.dataStorage.GridDataStorage.GridCellID;
import fr.unicaen.openmap.dataStorage.BoundingBox;

/** Base class of :<br>
 *     Cosine, Epanechnikov, Quartic, Triangle, Triweight, Uniform
 */
abstract public class BandwidthKernel extends Kernel {

    protected ArrayList<Integer> surroundingCellOffsets;

    public BandwidthKernel() {
	this(1);
    }

    /** Constructs a new bandwidth kernel given a radius in degrees. */
    public BandwidthKernel(double radius) {
	super(radius);
    }

    public double k(double d) {
	return d > radius ? 0 : y(d);
    }

    protected abstract double y(double d);

    /**
     *  Find cells that intersects the circle defined by the radius determined upon an horizontal and a vertical resolution. Does not use bb parameter.
     */
    public void prepare(float longitudeResolution, float latitudeResolution, BoundingBox bb) {
	surroundingCellOffsets = new ArrayList<Integer>();

	double radius2 = radius*radius, lonRes = (double)longitudeResolution, latRes = (double)latitudeResolution;
	int j, i, jMax = (int)Math.floor(radius/lonRes), iMax = (int)Math.floor(radius/latRes);

	for (j = -jMax; j < jMax + 1; j++) {
	    for (i = -iMax; i < iMax + 1; i++) {
		if (Point2D.distanceSq(0, 0, (double)j*lonRes, (double)i*latRes) < radius2) {
		    surroundingCellOffsets.add(new Integer(j));
		    surroundingCellOffsets.add(new Integer(i));
		}
	    }
	}
    }

    public ArrayList<Integer> getSurroundingCellIndices(int lon, int lat) {
	ArrayList<Integer> cells = new ArrayList<Integer>();

	for (int i = 0; i < surroundingCellOffsets.size(); i += 2) {
	    cells.add(new Integer(lon + surroundingCellOffsets.get(i).intValue()    ));
	    cells.add(new Integer(lat + surroundingCellOffsets.get(i + 1).intValue()));
	}
	return cells;
    }
    
}
