// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.postgis.gui;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import fr.unicaen.openmap.dataStorage.postgis.db.DBParams;

import com.bbn.openmap.util.Debug;

import fr.unicaen.openmap.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.tree.*;
import javax.swing.event.*;

/**
 *
 */
public class DBParamsPanel extends JPanel implements ActionListener, KeyListener {

    private JPanel textFieldPanel;
    private JTextField serverHostTF;
    private JTextField serverPortTF;
    private JTextField databaseTF;
    private JTextField loginTF;
    private JPasswordField passwordTF;
    private JButton okB;
    private JButton cancelB;
    private DBParams dbParams;

    private JLabel stateLabel;

    protected I18n i18n = Environment.getI18n();
    
    /**
     *
     */
    public DBParamsPanel(DBParams dbParams) {
	super();
	this.dbParams = dbParams;
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        //setBorder(new EmptyBorder(10, 10, 10, 10));
	setMaximumSize(new Dimension(400,200));
        setMinimumSize(new Dimension(400,200));
        setPreferredSize(new Dimension(400,200));
        
	textFieldPanel = new JPanel();
        textFieldPanel.setLayout(new LabeledPairLayout());
        
        serverHostTF = addTF(i18n.get(this, "server_name","Server name"), dbParams.getServer());
        serverPortTF = addTF(i18n.get(this, "server_port","Server Port"), dbParams.getPort() + "");
        databaseTF = addTF(i18n.get(this, "database","Database"), dbParams.getDatabase() + "");
        loginTF = addTF(i18n.get(this,"login","Login"), dbParams.getLogin());
	textFieldPanel.add(new JLabel(i18n.get(this, "password","Password")), "label");
	textFieldPanel.add(passwordTF = new JPasswordField(dbParams.getPassword()), "field");

        add(textFieldPanel);
        stateLabel = new JLabel(i18n.get(this,"connection_statement", "Connection statement"),
                Util.loadIcon("green.png"),
                SwingConstants.LEFT);
        JPanel pState = new JPanel(new BorderLayout());
        pState.add(stateLabel , BorderLayout.WEST);
        add(pState);
	JPanel j = new JPanel();
	okB = new JButton(i18n.get(this,"apply","Apply"));
	okB.addActionListener(this);
	cancelB = new JButton(i18n.get(this,"cancel","Cancel"));
	cancelB.addActionListener(this);
	j.add(okB);
	j.add(cancelB);
        add(j);
    }

    /**
     *
     */
    private JTextField addTF(String name, String text) {
	JTextField tf = new JTextField(text);
        textFieldPanel.add(new JLabel(name), "label");
        textFieldPanel.add(tf, "field");
	tf.setMargin(new Insets(1, 1, 1, 3));
	tf.addKeyListener(this);
	return tf;
    }

    /**
     * Reset all displayed data and repaint the component.
     * Call this method to ensure (by example) that all
     * the fields are currently displaying the matching
     * variable value.
     */
    public void updateData() {
	serverHostTF.setText(dbParams.getServer());
	serverPortTF.setText(dbParams.getPort());
	databaseTF.setText(dbParams.getDatabase());
	loginTF.setText(dbParams.getLogin());
	passwordTF.setText(dbParams.getPassword());
    }
    
    /**
     *
     */
    public void actionPerformed(ActionEvent event) {
	// ok => a change to be made on the postgis connection
	// i.e. fire an event which must have a listener ...
	// todo ...
	try {
	    if (event.getSource() == okB) {
		dbParams.setServer(serverHostTF.getText());
		dbParams.setPort(serverPortTF.getText());
		dbParams.setDatabase(databaseTF.getText());
		dbParams.setLogin(loginTF.getText());
		dbParams.setPassword(new String (passwordTF.getPassword()));
	    } else if (event.getSource() == cancelB) {
		serverHostTF.setText(dbParams.getServer());
		serverPortTF.setText(dbParams.getPort());
		databaseTF.setText(dbParams.getDatabase());
		loginTF.setText(dbParams.getLogin());
		passwordTF.setText(dbParams.getPassword());
	    }
            dbParams.fireDBParamsChanged();
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }
    
    /**
     *
     */
    public void keyPressed(KeyEvent e) { }
    
    /**
     *
     */
    public void keyReleased(KeyEvent e)  {
	// test if all values are ok ...
	// okB.setEnabled(...);
	// cancelB.setEnabled(...);
    }
    
    /**
     *
     */
    public void keyTyped(KeyEvent e)  { }

    public void updateStateLabel(boolean connectionOk){
        stateLabel.setIcon(connectionOk ? Util.loadIcon("green.png") :  Util.loadIcon("red.png"));
    }
}
