// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.xmltable;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.util.Debug;

import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.dataStorage.postgis.db.Factory;
import fr.unicaen.openmap.util.Util;

import java.io.*;
import java.util.*;
import java.util.zip.GZIPInputStream;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

/**
 * Quick and nasty work to parse an xmltable document.
 * No validation for the moment (need to write Dtd).
 * 2004-01-13: support for automatic GZIP deflating.
 *  "  "  "  : supports (all ?) long data with StringBuffer.
 *
 * 2006-05-17: fixed error while opening the file
 */
public class XmlTableParser extends DefaultHandler {

    private ArrayList<DataObject> dataObjects = new ArrayList<DataObject>() ;
    private ArrayList<String> columnNames = new ArrayList<String>();
    private ArrayList<String> omGraphicNames = new ArrayList<String>();

    private String NO_TAG = "NO_TAG";
    private String currentTagType = NO_TAG;
    private DataObject tempDataObject = null;
    private Object tempData = null;
    private OMDataLocation tempOMDataLocation = null;
    private String tempOMDataLocationName = null;
    private String columnName = null;
    
    private StringBuffer tempStringBuffer = new StringBuffer("");
    private boolean firstObject = true;
    
    public ArrayList<DataObject> getDataObjects() {
	return dataObjects;
    }
     public ArrayList<String> getColumnNames() {
	return columnNames;
    }
     public ArrayList<String> getOMGraphicsNames() {
	return omGraphicNames;
    }
    
    public XmlTableParser(String fileName) {
	parse(fileName);
    }
    
    public void parse (String fileName) {
	FileInputStream in;
	String newFileName = fileName;
	try {
	    in = new FileInputStream(fileName);
	} catch (FileNotFoundException e) {
	    newFileName = Util.locateFile(fileName);
	    if (newFileName == null || newFileName == "") {
		System.err.println("[XmlTableDataStorage] ### File not found " + fileName + " ###");
		return;
	    }
	    try {
		in = new FileInputStream(newFileName);
	    } catch (FileNotFoundException ee) {
		System.err.println("[XmlTableDataStorage] ### File not found exception for " + fileName + " as " + newFileName);
		return;
	    }
	}
	    
	try {
	    // Try GZIP Format
	    parse(new GZIPInputStream (in));
	} catch (Exception notGZIPexception) {
	    Debug.message("DataStorage", "[XmlTableDataStorage] Fallback to plain text after trying gzip");
	    try {
		// The GZIPInputStream corrupted the FileInputStream in!
		// So we contruct a new one from the fileName  
		parse(new FileInputStream(newFileName));
	    } catch (FileNotFoundException ee) {
		System.err.println("[XmlTableDataStorage] ### File not found exception for " + fileName + " as " + newFileName);
		return;
	    }
	}
    }
    
    public void parse (InputStream file) {
	
	SAXParserFactory factory = SAXParserFactory.newInstance();
	factory.setValidating(false);
	try {
	    SAXParser saxParser = factory.newSAXParser();
	    saxParser.parse(file, this);
	} catch (SAXParseException spe) {
	    System.err.println("[XmlTableDataStorage] Parse Error, line " + spe.getLineNumber() + ": " + spe.getMessage());
	} catch (SAXException sxe) {
	    System.err.println("[XmlTableDataStorage] " + sxe.getMessage());
	} catch (ParserConfigurationException pce) {
	    System.err.println("[XmlTableDataStorage] " + pce.getMessage());
	} catch (IOException ioe) {
	    System.err.println("[XmlTableDataStorage] " + ioe.getMessage());
	}
    }

    private void debugElement(String qName, Attributes attrs) {
	System.err.println("Element: " + qName);
	for (int i = 0 ; i < attrs.getLength(); i++) {
	    String qn = attrs.getQName(i);
	    String type = attrs.getType(i);
	    String value = attrs.getValue(i);
	    System.err.println("qn: " + qn);
	    System.err.println("type: " + type);
	    System.err.println("value: " + value);
	    System.err.println("");
	}
    }

    public void startElement(String uri, String localName, String qName, Attributes attrs) throws SAXException {
	currentTagType = qName;
	
	// debugging ...
	// debugElement(qName, attrs);
	
	if (qName.equals(Dtd.DATA_OBJECT_TAG)) {
	    tempDataObject = new DataObject();
	} else if (qName.equals(Dtd.GEOMETRY_TAG)) {
	    tempOMDataLocation = new OMDataLocation();
            // new geometric object ... take column name
	    tempOMDataLocationName = attrs.getValue(Dtd.NAME_ATTR);
            if (firstObject && tempOMDataLocationName!=null) omGraphicNames.add(tempOMDataLocationName);
	}else if (qName.equals(Dtd.DATA_TAG)) {
                // we have a new data column 
                // do we have to add this column name ?
                columnName=attrs.getValue(Dtd.NAME_ATTR);
                if (firstObject && columnName!=null) columnNames.add(columnName);
        }
    }
    
    public void endElement(String uri, String localName, String qName) throws SAXException {
	// System.err.println("endElement: " + qName);
	try {
	    if (qName.equals(Dtd.DATA_OBJECT_TAG)) {
		if (tempDataObject != null) {
                    // enf of DataObject ... if this is the first, we have all informations in columNames vector.
                    firstObject=false;
		    dataObjects.add(tempDataObject);
		}
		tempDataObject = null;
	    } else if (currentTagType.equals(Dtd.THE_GEOM_TAG)) {
		if (tempOMDataLocation != null){
                    OMGraphic omg = Factory.createOMGraphic(tempStringBuffer.toString());
                    omg.putAttribute(Util.OMGraphicIndex, dataObjects.size());
		    tempOMDataLocation.setOMGraphic(omg);
                }
	    } else if (currentTagType.equals(Dtd.CENTROID_TAG)) {
		if (tempOMDataLocation != null)
		    tempOMDataLocation.setCentroid(Factory.createCentroid(tempStringBuffer.toString()));
	    } else if (currentTagType.equals(Dtd.BOUNDING_BOX_TAG)) {
		if (tempOMDataLocation != null)
		    tempOMDataLocation.setBoundingBox(Factory.createBoundingBox(tempStringBuffer.toString()));
	    } else if (qName.equals(Dtd.GEOMETRY_TAG)) {
		if (tempOMDataLocation != null && tempOMDataLocationName != null) {
		    tempDataObject.addOMDataLocation(tempOMDataLocationName, tempOMDataLocation);
		}
		tempOMDataLocation = null;
		tempOMDataLocationName = null;
	    } else if (qName.equals(Dtd.DATA_TAG)) {
                // a standard column
                if (tempDataObject!=null && columnName!=null) 
                    tempDataObject.addData(columnName,tempStringBuffer.toString().trim());
            }
	} catch (Exception e) {
	    System.err.println("[XmlTableDataStorage] " + e.getMessage());
	} finally {
	    currentTagType = NO_TAG;
	    tempStringBuffer = new StringBuffer ("");
	}
    }
    
    /**
     * This method will only concatenete buffers for 
     */
    public void characters(char buf [], int offset, int len) throws SAXException {
	tempStringBuffer.append(buf, offset, len);
    }
    
    /**
     * Error handling is reported above.
     */
    public void error(SAXParseException e) throws SAXParseException { throw e; }
    
    /**
     * Warning is handled to stderr.
     */
    public void warning(SAXParseException err) throws SAXParseException {
        System.err.println("[XmlParser] Warning, line " + err.getLineNumber() + ", " + err.getSystemId());
        System.err.println("[XmlParser] " + err.getMessage());
    }
}
