/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.unicaen.openmap.gui;

import com.bbn.openmap.Layer;
import com.bbn.openmap.LayerHandler;
import fr.unicaen.openmap.AtlasLayerHandler;
import fr.unicaen.openmap.AtlasLoader;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicDataDisplayLayerPopupMenu;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicPopupMenu;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Thematic;
import fr.unicaen.openmap.util.TooManyLegendException;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Vector;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

/**
 *Atlas LayersTree - built according to a specified directory file system
 * @author jchauvea
 */
public class AtlasLayersTree extends LayersTree {

    protected String path;
    public static HashMap<String, Layer[]> layersByPath = new HashMap<String, Layer[]>();
    protected ArrayList<AtlasLayersTree> synchronizedTrees = new ArrayList<AtlasLayersTree>();

    public AtlasLayersTree(DefaultMutableTreeNode root, String path,
            LayerHandler lh, JComponent comp) {

        super(root, lh, comp);
        this.path = path;
        buildFileSystemTree(root, new File(path));
        layersLoaded = true;
        setShowsRootHandles(true);
        expandTree(root);
        updateUI();
        removeMouseListener(this);
    }

    public AtlasLayersTree(DefaultTreeModel model, LayerHandler lh, JComponent comp) {
        super(model, lh, comp);
    }

    public void addSynchronizedTree(AtlasLayersTree tree) {
        synchronizedTrees.add(tree);
    }

    /** Add nodes from under "dir" into top. Highly recursive. */
    protected DefaultMutableTreeNode buildFileSystemTree(
            DefaultMutableTreeNode curTop, File dir) {
        String curPath = dir.getPath();
        DefaultMutableTreeNode curDir = null;
        if (curPath.contains(File.separator)) {
            String[] tab;
            if (File.separator.equals("\\")) {
                tab = curPath.split("\\" + File.separator);
            } else {
                tab = curPath.split("\\" + File.separator);
            }
            //curDir = buildNode(curPath, tab[tab.length - 1]);
            curDir = new DefaultMutableTreeNode(tab[tab.length - 1]);
        } else {
            curDir = new DefaultMutableTreeNode(curPath);
        }

        if (curTop != null) { // should only be null at root
            curTop.add(curDir);
        }
        Vector<String> ol = new Vector<String>();
        String[] tmp = dir.list();
        for (int i = 0; i < tmp.length; i++) {
            ol.addElement(tmp[i]);
        }
        Collections.sort(ol, String.CASE_INSENSITIVE_ORDER);
        File f;
        Vector<String> files = new Vector<String>();
        // Make two passes, one for Dirs and one for Files. This is #1.
        for (int i = 0; i < ol.size(); i++) {
            String thisObject = (String) ol.elementAt(i);
            String newPath;
            if (curPath.equals(".")) {
                newPath = thisObject;
            } else {
                newPath = curPath + File.separator + thisObject;
            }
            if ((f = new File(newPath)).isDirectory()) {

                buildFileSystemTree(curDir, f);

            } else {
                files.addElement(thisObject);
            }
        }
        // Pass two: for properties files.
        for (int fnum = 0; fnum < files.size(); fnum++) {
            if (files.elementAt(fnum).endsWith(AtlasLoader.PROPERTIES_EXTENSION)) {
                DefaultMutableTreeNode pNode = new DefaultMutableTreeNode(
                        files.elementAt(fnum).replace(AtlasLoader.PROPERTIES_EXTENSION, ""));

                curDir.add(pNode);
                String tmpPath = curPath + File.separator + files.elementAt(fnum);
                if (!layersLoaded) {
                    try {
                        Layer[] ls = AtlasLoader.loadLayersFromPath(tmpPath, (AtlasLayerHandler) layerHandler);

                        if (ls != null) {
                            layersByPath.put(tmpPath, ls);
                            buildTreeForLayers(pNode, ls);
                        }
                    } catch (TooManyLegendException e) {
                        JOptionPane.showMessageDialog(this, e.getMessage(), "", JOptionPane.ERROR_MESSAGE);
                        continue;
                    }
                } else {
                    //layerHandler.removeLayer(layerHandler.getLayers().length -1 );
                    buildTreeForLayers(pNode, layersByPath.get(tmpPath));
                }
            }
        }

        return curDir;
    }

    @Override
    protected void buildTreeForLayers(DefaultMutableTreeNode pnode, Layer[] layers) {
        for (Layer l : layers) {
            DefaultMutableTreeNode node = new DefaultMutableTreeNode(l);
            pnode.add(node);
            nLayers++;
            if (l instanceof ThematicDataDisplayLayer) {
                popups.put(node, new ThematicDataDisplayLayerPopupMenu(this,
                        (ThematicDataDisplayLayer) l));
                ThematicDataDisplayLayer tddl = (ThematicDataDisplayLayer) l;
                tddl.addThematicListener(this);
                for (int i = 0; i < tddl.getThematicArray().size(); i++) {
                    DefaultMutableTreeNode nodeThema = new DefaultMutableTreeNode(
                            tddl.getThematicArray().get(i));
                    node.add(nodeThema);
                    popups.put(nodeThema, new ThematicPopupMenu(this, tddl.getThematicArray().get(i).isVisible()));

                }
            } else {
                popups.put(node, new LayerPopupMenu(this, l.isVisible()));
            }
        }
    }

    @Override
    public AtlasLayerHandler getLayerHandler() {
        return (AtlasLayerHandler) super.getLayerHandler();
    }

//    public void updateSynchronizedWithNewLayer(Layer oldLayer, Layer newLayer) {
////        DefaultMutableTreeNode node = getLayerNode(oldLayer, root);
////        if (node == null) {//already updated
////            //System.out.println("UPDATEEEEEEEEEEEEEEEEEEE node = " + node);
////            return;
////        }
////        node.setUserObject(newLayer);
////        setSelectionPath(new TreePath(node.getPath()));
//        if (newLayer instanceof ThematicDataDisplayLayer) {
//            ((ThematicDataDisplayLayer) newLayer).addThematicListener(this);
//        }
//        for (AtlasLayersTree lt : synchronizedTrees) {
//            //System.out.println(Math.random());
//            //lt.updateSynchronizedWithNewLayer(oldLayer, newLayer);
//        }
//    }
    public void fireLayerSynchronization(Layer oldLayer, Layer newLayer) {
        //for (AtlasLayersTree lt : synchronizedTrees) {
        //   layerSynchronization(oldLayer, newLayer);
        //}
    }

    public void layerSynchronization(Layer oldLayer, Layer newLayer) {

        DefaultMutableTreeNode node = getObjectNode(oldLayer, root);
        node.setUserObject(newLayer);
    }

    public void removeLayerChildNodes(Layer l) {
        DefaultMutableTreeNode node = getObjectNode(l, root);
        if (node == null) {
            return;
        }
        DefaultTreeModel model = ((DefaultTreeModel) getModel());
        //DefaultMutableTreeNode node = (DefaultMutableTreeNode) getLastSelectedPathComponent();
        int nbChildren = node.getChildCount();
        if (nbChildren == 0) {
            return;
        }
        for (int i = 0; i < nbChildren; i++) {
            model.removeNodeFromParent((DefaultMutableTreeNode) node.getChildAt(0));
        }
        updateUI();
        for (AtlasLayersTree lt : synchronizedTrees) {
            lt.removeLayerChildNodes(l);
        }
    }

    /*public void selectCurrentLayer(Layer l){
    DefaultMutableTreeNode node = getLayerNode(l, root);

    if(node != null){

    }
    }*/
}
