// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui.dnd;

import java.awt.Color;
import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.PointerInfo;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.InputEvent;
import java.util.ArrayList;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.TransferHandler;

import fr.unicaen.openmap.dataStorage.gui.AddDataStorageConfigurationDialog;
import fr.unicaen.openmap.util.Util;

public class DndTransferHandler extends TransferHandler {

    protected DragArea source;
    //protected boolean received = false;
    protected DropArea dropSupport;
    protected ArrayList<DropArea> dropAreas;

    public DndTransferHandler() {
        super("text");
        dropAreas = new ArrayList<DropArea>();
    }

    @Override
    public void exportAsDrag(JComponent comp, InputEvent e, int action) {

        if (comp instanceof DragArea) {
            source = (DragArea) comp;
            source.getBox().showTargets();
            highLightDropAreas(true);
        } else {
            source = null;
        }
        super.exportAsDrag(comp, e, action);
    }

    @Override
    protected void exportDone(JComponent s, Transferable data, int action) {
        if (dropSupport != null) {
            dropSupport.unhover();
        }
        PointerInfo pointer = MouseInfo.getPointerInfo();
        Point location = pointer.getLocation();

        if (action == NONE /*&& received == false*/) {//dropped in "nowhere"
            source.getBox().externalize(location);
        }

        source.getBox().hideEmptyTargets();
        highLightDropAreas(false);

    }

    @Override
    public boolean importData(JComponent comp, Transferable t) {
        //System.out.println("IMPORT DATA");
        if (!(comp instanceof DropArea) || source == null || !(source instanceof DragArea)) {
            return false;
        } else {
            ((DropArea) comp).receive(source);
            //received = true;
            return true;
        }
    }

    public void addDropArea(DropArea da) {
        dropAreas.add(da);
    }

    private void highLightDropAreas(boolean b) {
        Color c;
        if (b) {
            c = new Color(82, 191, 35);
        } else {
            c = Color.LIGHT_GRAY;
        }
        for (DropArea da : dropAreas) {
            da.setBackground(c);
            da.updateUI();
        }
    }

    /*@Override
    public boolean importData(TransferHandler.TransferSupport support){
    if(canImport(support)){
    System.out.println(support.getComponent().getClass());
    DropArea da = (DropArea)support.getComponent();
    System.out.println("RECEIVE 1");
    try{
    da.receive(source);
    }
    catch(Exception e){
    e.printStackTrace();
    }
    received = true;
    System.out.println("RECEIVE");
    return true;
    }
    return false;


    }*/
    @Override
    public boolean canImport(TransferHandler.TransferSupport support) {

        if (support.getComponent() instanceof DropTabArea && !source.getBox().isTabbeded()) {
            return false;
        }

        if (support.getComponent() instanceof DropArea) {
            if (dropSupport != null) {
                dropSupport.unhover();
            }
            dropSupport = (DropArea) support.getComponent();
            dropSupport.hover(source.getBox().getPreferredSize());
            return true;
        } else {
            if (dropSupport != null) {
                dropSupport.unhover();
            }
            return false;
        }
    }
    //@Override
	/*public boolean canImport(JComponent cp, DataFlavor[] df){
    //System.out.println("plop");
    //System.out.println("CAN IMPORT -> DATAFLAVOR");
    //boolean res = super.canImport(cp, df);
    //System.out.println( res );
    return false;
    }*/

    @Override
    protected Transferable createTransferable(JComponent c) {
        //System.out.println("CREATE TRANSFERABLE");
        return super.createTransferable(c);
    }
    /*@Override
    public void exportToClipboard(JComponent comp, Clipboard clip, int action){
    System.out.println("EXPORT CLIPBOARD");
    super.exportToClipboard(comp, clip, action);
    }*/
}
