// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors
 */
// </header>
package fr.unicaen.openmap.gui.menu;

import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.io.*;

import com.bbn.openmap.*;
import com.bbn.openmap.gui.menu.MapHandlerMenuItem;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.gui.ThemaMapFileChooserManager;
import fr.unicaen.openmap.gui.ThemaMapPanelsManager;
import fr.unicaen.openmap.layer.utils.LayerLoader;
import fr.unicaen.openmap.util.TooManyLegendException;
import java.net.MalformedURLException;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 * This is a menu item that loads a file. This file should be a valid
 * openmap properties file, with openmap.layers set and projection
 * properties set. The components list is ignored, since it is pretty
 * dependent on order, and it's hard to write a good properties file
 * automatically with a valid components list.
 * <P>
 * If you want to change the components in an application, launch
 * OpenMap with the properties file you want.
 */
public class LoadPropertiesMenuItem extends MapHandlerMenuItem implements
        ActionListener {

    private String titleString;
    private ThemaMapPanelsManager manager;
    protected LayerHandler layerHandler;

    public LoadPropertiesMenuItem(String initString) {
        super(initString);
        titleString = initString;
        addActionListener(this);
    }

    @Override
    public void findAndInit(Object someObj) {
        super.findAndInit(someObj);
        if (someObj instanceof ThemaMapPanelsManager) {
            manager = (ThemaMapPanelsManager) someObj;
            return;
        }
        if (someObj instanceof LayerHandler) {
            layerHandler = (LayerHandler) someObj;
            return;
        }

    }

    public void actionPerformed(ActionEvent ae) {
        if (JOptionPane.showConfirmDialog(this,
                i18n.get(LoadPropertiesMenuItem.class, "accept_message", "You will lose unsaved data - Accept?"),
                i18n.get(LoadPropertiesMenuItem.class, "accept_title", "New ThemaMap Project"), JOptionPane.YES_NO_OPTION) ==
                JOptionPane.OK_OPTION) {
            for (Layer l : layerHandler.getLayers()) {
                //manager.getLayersPanel().deleteLayer(l);
                layerHandler.removeLayer(l);
            }
            layerHandler.setLayers();

            String loadString = ThemaMapFileChooserManager.propertiesLoadDialog(titleString);
            if (loadString == null || loadString.length() == 0) {
                return;
            }


            File file = new File(loadString);

            try {
                LayerLoader layerLoader = new LayerLoader(file.toURI().toURL(), layerHandler);
                String[] layers = layerLoader.findAvailableLayers();
		if (layers != null && layers.length != 0) {
		    ArrayList<String> rLayers = new ArrayList<String>(Arrays.asList(layers));
		    Collections.reverse(rLayers);
		    for (String selection : rLayers) {
			layerLoader.initLayer(selection);
		    }
		}
                Properties newProps = PropertyHandler.fetchProperties(file.toURI().toURL());

                String test = newProps.getProperty("openmap." + LayerHandler.layersProperty);
                if (test == null) {
                    throw new IOException("Doesn't seem like a valid properties file");
                }

            } catch (FileNotFoundException fnfe) {
                Debug.error(fnfe.getMessage());
            } catch (MalformedURLException fnfe) {
                Debug.error(fnfe.getMessage());
            } catch (TooManyLegendException fnfe) {
                Debug.error(fnfe.getMessage());
            } catch (IOException ioe) {
                InformationDelegator id = (InformationDelegator) mapHandler.get("com.bbn.openmap.InformationDelegator");

                if (id != null) {
                    id.displayMessage("Error while loading file ...",
                            "Error while loading file " + file.getAbsolutePath() + "\n" + ioe.getMessage());
                }
                Debug.error("Error occured loading " + file.getAbsolutePath());
            }
        }
    }
}
