// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableGraphicList;
import java.text.NumberFormat;


import java.awt.event.*;
import javax.swing.*;

import java.lang.reflect.Constructor;
import java.beans.beancontext.*;

import fr.unicaen.openmap.layer.dataDisplayLayer.*;
import fr.unicaen.openmap.layer.dataDisplayLayer.gui.*;
import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.dataStorage.gml.out.GMLWriter;
import fr.unicaen.openmap.dataStorage.gui.AdaptorButtonArray;
import fr.unicaen.openmap.dataStorage.out.CSVWriter;
import fr.unicaen.openmap.gui.ThemaMapPanel;
import fr.unicaen.openmap.gui.dnd.MovableBox;
import fr.unicaen.openmap.gui.dnd.MovableBoxesContainer;

import fr.unicaen.openmap.gui.dnd.DropTabArea;


import fr.unicaen.openmap.util.propertyEditor.DataStoragePropertyEditor;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ClassDescription;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Thematic;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicPalette;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.Refreshable;
import com.bbn.openmap.*;
import com.bbn.openmap.event.LayerStatusEvent;
import com.bbn.openmap.event.MapMouseEvent;
import com.bbn.openmap.event.MapMouseListener;
import com.bbn.openmap.event.ProjectionEvent;
import com.bbn.openmap.event.SelectMouseMode;
import com.bbn.openmap.layer.policy.BufferedImageRenderPolicy;
import com.bbn.openmap.layer.policy.RenderPolicy;
import com.bbn.openmap.layer.policy.StandardRenderPolicy;
import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.proj.*;
import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.tools.drawing.DrawingToolRequestor;
import fr.unicaen.openmap.event.LayerListener;
import com.bbn.openmap.omGraphics.event.*;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.gui.ClassicLayersTree;
import fr.unicaen.openmap.gui.ThemaMapFileChooserManager;
import fr.unicaen.openmap.layer.legendLayer.LegendLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableGraphic;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Raw;
import fr.unicaen.openmap.omGraphics.event.TMMapMouseInterpreter;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.XMLAppObject;
import fr.unicaen.openmap.util.legendable.LegendableLayer;
import fr.unicaen.openmap.util.propertyEditor.Inspector;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Component;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import javax.swing.filechooser.FileNameExtensionFilter;

/**
 * This layer visualizes thematic geolocalized data
 * <p>
 * Data visualization modes are : Raw, Label, Disc, Pie, and may be one day
 * Flow, Histograms, ...
 * <p>
 * A visu is defined with a 3-uple :<br>
 * &lt; vector of geometry columns names, vector of data columns names, mode
 * &gt;
 * <p>
 * This layer is able to cache the OMGraphics produced by generateGraphics().
 * 
 * <p>
 * Example of properties:
 * 
 * <pre>
 * test.class=fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer
 * test.dataStorageClass=fr.unicaen.openmap.dataStorage.shapefile.ShapeFileDataStorage
 * test.prettyName= Thematic Test 
 * test.renderPolicy=rp
 * test.rp.class=com.bbn.openmap.layer.policy.BufferedImageRenderPolicy
 * test.pcp.class=com.bbn.openmap.layer.policy.ListResetPCPolicy
 * test.projectionChangePolicy=pcp
 * test.shp=pays.shp
 * test.dbf=pays.dbf
 * test.visuNames= visu0 visu1 visu2
 * test.visu0.visuClassName=fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Raw
 * test.visu0.fill=true
 * test.visu0.fillColor=66002233
 * test.visu0.geometryColumnName=shape
 * test.visu0.dataColumnName=POP
 * test.visu0.lineColor=FF990000
 * test.visu1.visuClassName=fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Label
 * test.visu1.fill=true
 * test.visu1.fillColor=FFAAAAAA
 * test.visu1.geometryColumnName=shape
 * test.visu1.lineColor=FF223399
 * test.visu1.dataColumnName=LIEN
 * test.visu2.visuClassName=fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Pie
 * test.visu2.fill=true
 * test.visu2.geometryColumnName=shape
 * test.visu2.dataColumnName=POP
 * test.visu2.radiusIsSum=true
 * test.visu2.colorDataColumnNames=POP POP2 POP3
 * test.visu2.lineColor=FF990000
 * test.visu2.minSize=.1
 * test.visu2.maxSize=1
 * </pre>
 */
public class ThematicDataDisplayLayer extends SimpleDataDisplayLayer implements
        Refreshable, DrawingToolRequestor, LegendableLayer, ThematicListener {

    public final static int EXPAND_PERCENT = 10;
    private String missingErrors = "";
    private JPanel panData;
    private MovableBox sourceMovableTab, dataMovableTab, visuMovableTab;
    public static final String visuNamesPropertyString = "visuNames";
    public static final String addableVisuNamesPropertyString = "addableVisuNames";
    public static final String visuPrettyNamePropertyString = "prettyName";
    public static final String visuIconPropertyString = "icon";
    public static final String visuClassNamePropertyString = "visuClassName";
    public static final String INDEX_IN_DS = "Index in DataStorage";
    public static final String smoothPropertyString = "smooth";
    public static final String VALUE = "Value";
    private final Collection<ThematicListener> thematicsListeners = new ArrayList<ThematicListener>();
    private ThematicListener currentLayersTree;
    private final Collection<fr.unicaen.openmap.event.LayerListener> layerListeners = new ArrayList<LayerListener>();
    private int nbThematicWhichSharedGraphics = 0;
    private ArrayList<String> visuNames;
    //private Hashtable<Thematic, Component> thematicGUIS;
    private ClassDescription[] addableVisuClassDescription;
    private boolean badLegendCache = true;
    private OMResizeableGraphicList omLegendsTv;
    private LegendLayer currentLegendLayer;
    private HashMap<Thematic, OMResizeableGraphic> legendsTv = new HashMap<Thematic, OMResizeableGraphic>();
    private boolean smooth;
    //private OMGraphicList drawings = new OMGraphicList();
    //private Drawing drawing;
    /**
     * This array stores the individual thematic visualizations. They are stored
     * in reverse order: first layer in the list has its graphics on top. Tabs
     * in the gui should be in the same order.
     */
    protected ArrayList<Thematic> tvs = new ArrayList<Thematic>();

    public ArrayList<Thematic> getThematicArray() {
        return tvs;
    }

    public ThematicDataDisplayLayer() {
        Debug.message(classNameForDebug, this + " new ThematicDataDisplayLayer()");
        autoPalette = true;
        visuNames = new ArrayList<String>();
        //addableVisuClassDescription = new ArrayList<ClassDescription>();
        // be selfish about events => only the first layer will respond.
        setConsumeEvents(false);

    }

    /**
     * Compute and apply bounding box (+ n%) which contains all graphics of the layer
     */
    public void zoomOn() {
        BoundingBox bb = dataStorageIterator.getBoundingBox();
        if (bb == null) {
            super.setCenterAndProjection();
        } else {
            MapBean map = getMapBean();

            Projection projection = map.getProjection();
            Proj p = (Proj) projection;
            LatLonPoint upperLeft = new LatLonPoint.Double(bb.getUpperLeftLat(), bb.getUpperLeftLon());
            LatLonPoint lowerRight = new LatLonPoint.Double(bb.getLowerRightLat(), bb.getLowerRightLon());

            double lonCenter = (upperLeft.getLongitude() + lowerRight.getLongitude()) / 2;
            double latCenter = (upperLeft.getLatitude() + lowerRight.getLatitude()) / 2;

            float sc = ProjMath.getScale(upperLeft, lowerRight, p);
            sc *= 1.05;
            Debug.message("proj", "ZOooMM in " + bb + " [" + p.getWidth() + "x" + p.getHeight() + "] => scale=" + sc);

            p.setScale(sc);
            p.setCenter(latCenter, lonCenter);
            map.setProjection(p);
            refreshGraphics();
        }
    }

    public void addThematicListener(ThematicListener listener) {

        if (listener instanceof ClassicLayersTree) {
            if (currentLayersTree != null) {
                thematicsListeners.remove(currentLayersTree);
            }
            currentLayersTree = listener;
        }
        if (!thematicsListeners.contains(listener)) {
            thematicsListeners.add(listener);
        }
    }

    public void removeThematicListener(ThematicListener listener) {
        thematicsListeners.remove(listener);
    }

    public void fireNewThematicAdded(Thematic tv) {
        //System.out.println("FIRE NEW THEM ADDED****************************************** " + thematicsListeners.size());
        //System.out.println("TDDL listners size = ");
        for (ThematicListener tvl : thematicsListeners) {
            //System.out.println(tvl.getClass() + " - " + tvl.hashCode());
            tvl.thematicAdded(tv);
        }
        fireNeedsLegendUpdate();
    }

    public void fireNeedsLegendUpdate() {
        badLegendCache = true;
        for (LayerListener ll : layerListeners) {
            //System.out.println(ll.getClass());
            ll.layerUpdate();
        }
    }

    public void fireThematicRemoved(Thematic tv) {
        for (ThematicListener tvl : thematicsListeners) {
            tvl.thematicRemoved(tv);
        }
        fireNeedsLegendUpdate();
    }

    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);

        Debug.message(classNameForDebug,
                "[ThematicDataDisplayLayer] setProperties");
        prefix = PropUtils.getScopedPropertyPrefix(prefix);

        String vNames[] = properties.getProperty(
                prefix + visuNamesPropertyString, "").split("[ \t]");
        for (int i = 0; i < vNames.length; i++) {
            String visuPrefix = PropUtils.getScopedPropertyPrefix(prefix + vNames[i]);
            String visuClassName = properties.getProperty(visuPrefix + visuClassNamePropertyString, "");
            if ("".equals(visuClassName)) {
                System.err.println("### [ThematicDataDisplayLayer] No visuClassName specified for visu" + i + " " + visuPrefix + " " + visuClassNamePropertyString);
                continue;
            }
            Thematic tv = addNewThematicVisu(visuClassName, vNames[i]);
            tv.setProperties(visuPrefix, properties);
            fireNewThematicAdded(tv);
        }

        if (properties.getProperty(prefix + addableVisuNamesPropertyString, "").equals("")) {
            addableVisuClassDescription = null;
        } else {
            String addableVisu[] = properties.getProperty(
                    prefix + addableVisuNamesPropertyString, "").split("[ \t]");
            addableVisuClassDescription = new ClassDescription[addableVisu.length];
            for (int i = 0; i < addableVisu.length; i++) {
                String visuPrefix = PropUtils.getScopedPropertyPrefix(prefix + addableVisu[i]);
                String visuClassName = properties.getProperty(visuPrefix + visuClassNamePropertyString, "");
                String visuPrettyName = properties.getProperty(visuPrefix + visuPrettyNamePropertyString, "");
                String visuIcon = properties.getProperty(visuPrefix + visuIconPropertyString, "");
                addableVisuClassDescription[i] =
                        new ClassDescription(visuPrettyName, visuClassName, visuIcon);
            }
        }
        // Optimal use for this layer with BufferedImageRenderPolicy as
        // RenderPolicy.
        // See com.bbn.openmap.layer.OMGraphicHandlerLayer.setProperties(...)
        // for extended code.
        //        BufferedImageRenderPolicy rp = new BufferedImageRenderPolicy(this);
        //        rp.setUseImageBuffer(true);
        //        setRenderPolicy(rp);
        setSmooth(PropUtils.booleanFromProperties(properties, prefix + smoothPropertyString, false));

        if (tvs.isEmpty() && !(vNames[0].equals("##"))) {
            addDefaultThematics(properties);
        }

        // Add palette once (when Layer is created *or* if properties are set)
        // Basis property flag from com.bbn.openmap.Layer
        if (autoPalette) {
            // showPalette();
            autoPalette = false;
        }

        // always consume events
        setMouseModeIDsForEvents(new String[]{"Gestures"});
        setConsumeEvents(false);
        //zoomOn();
    }

    protected void addDefaultThematics(Properties props) {
        // default: we add a Raw as background
        String defaultName = i18n.get(Raw.class, "raw", "raw");
        Thematic rawTv = addNewThematicVisu(
                "fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Raw",
                defaultName);
        rawTv.setProperties(defaultName, props);
        fireNewThematicAdded(rawTv);
        if (configPanel != null) {
            // // configPanel.addThematicTab(rawTv, this);
            // hardcore refresh :-)
            box.getTabs().setSelectedIndex(1);
            box.getTabs().setSelectedIndex(0);

        }

    }

    public Properties getProperties(Properties properties) {
        properties = super.getProperties(properties);

        String prefix = PropUtils.getScopedPropertyPrefix(this);
        String visuNames = "";
        int i = 0;

        Hashtable<String, Thematic> legends = new Hashtable<String, Thematic>();
        // Collections.reverse(tvs);
        for (Thematic tv : tvs) {
            String tvPrefix = "visu" + i++;

            // add names in front to preserve display order !
            visuNames = tvPrefix + " " + visuNames;

            properties = tv.getProperties(prefix + tvPrefix + ".",
                    properties);

        }

        // Legends need correct prefix for all others Thematics => properties in
        // last
        for (Enumeration<String> keys = legends.keys(); keys.hasMoreElements();) {
            String legendPrefix = keys.nextElement();
            properties = legends.get(legendPrefix).getProperties(
                    prefix + legendPrefix, properties);
        }
        properties.put(prefix + smoothPropertyString, String.valueOf(smooth));
        properties.put(prefix + visuNamesPropertyString, visuNames);
        return properties;
    }

    private Constructor getVisuConstructor(String className)
            throws java.lang.ClassNotFoundException {
        Class cls = Class.forName(className);
        for (Constructor c : cls.getConstructors()) {
            java.lang.reflect.Type[] params = c.getGenericParameterTypes();
            if (params.length == 2 && params[0] == Class.forName("fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer") && params[1] == Class.forName("java.lang.String")) {
                return c;
            }
        }
        return null;
    }

    /**
     * This method do not really add but insert a new Thematic in tvs ArrayList
     * at index 0.
     */
    public Thematic addNewThematicVisu(String visuClassName, String visuName) {
        Thematic tv = null;
        try {
            tv = (Thematic) getVisuConstructor(visuClassName).newInstance(
                    new Object[]{this, visuName});
            if (tv.useSharedGraphics()) {
                nbThematicWhichSharedGraphics++;
            }
            //fireNewThematicAdded(tv);
            addMapMouseListener(0, tv.getMapMouseListener());
        } catch (Exception e) {
            System.err.println("### [ThematicDataDisplayLayer] *** construction failure for " + visuClassName);
            e.printStackTrace();
            tv = new fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Raw(
                    this, visuName);
        }
        tvs.add(0, tv);
        /*if (!(tv instanceof Legend)) {
        updateLegend(false);
        }*/
        Debug.message("ThematicDataDisplayLayer", this + " after addNewThematicVisu(" + visuClassName + ", " + visuName + ") " + tvs);

        return tv;
    }

    /**
     * This method will remove a Thematic from the internal ArrayList
     *
     * @return true if the object was removed.
     */
    public boolean deleteThematicVisu(Thematic tv) {
        if (tv == null) {
            return false;
        }
        if (tv.useSharedGraphics()) {
            nbThematicWhichSharedGraphics--;
        }
        boolean b = tvs.remove(tv);
        //configPanel.removeCurrentThematicGUI();
        configPanel.removeThematicGUI(tv.getName());
        //updateLegend(true);
        fireThematicRemoved(tv);
        return b;

    }

    public void deleteThematicVisu(int index) {
        tvs.remove(index);
    }

    /**
     * This method will move a thematic in the thematic array list in order to
     * display it accordingly (move down).
     */
    public synchronized void moveDownThematicVisu(Thematic tv) {
        moveDownThematicVisu(tvs.indexOf(tv));
    }

    /**
     * This method will move a thematic in the thematic array list in order to
     * display it accordingly (move up).
     */
    public synchronized void moveUpThematicVisu(Thematic tv) {
        moveUpThematicVisu(tvs.indexOf(tv));
    }

    /**
     * Move UP a Thematic by its index. First layer in the list has its graphics
     * on top, hence we must decrement Thematic index in the Array. This method
     * should be private.
     *
     * @see #tvs
     * @see #moveUpThematicVisu(Thematic tv)
     */
    public synchronized void moveUpThematicVisu(int index) {
        Debug.message("ThematicDataDisplayLayer", this + " before moveDownThematicVisu(" + index + ") " + tvs);
        // don't do anything if tv is already the last one.
        if (index == 0) {
            return;
        }
        Thematic tv = tvs.remove(index);
        tvs.add(index - 1, tv);
        refreshGraphicsAndGUI();
        Debug.message("ThematicDataDisplayLayer", this + " after moveDownThematicVisu(" + index + ") " + tvs);
    }

    /**
     * Move DOWN a Thematic by its index. First layer in the list has its
     * graphics on top, hence we must increment Thematic index in the Array.
     * This method should be private.
     *
     * @see #tvs
     * @see #moveDownThematicVisu(Thematic tv)
     */
    public synchronized void moveDownThematicVisu(int index) {
        Debug.message("ThematicDataDisplayLayer", this + " before moveUpThematicVisu(" + index + ") " + tvs);
        // don't do anything if tv is already the first one.
        if (index == tvs.size() - 1) {
            return;
        }
        Thematic tv = tvs.remove(index);
        tvs.add(index + 1, tv);
        refreshGraphicsAndGUI();
        Debug.message("ThematicDataDisplayLayer", this + " after moveUpThematicVisu(" + index + ") " + tvs);
    }

    /**
     * Method that we use to generate the list of drawable objects. This method
     * is called by
     *
     * @see #projectionChanged(ProjectionEvent e)
     * @see #renderDataForProjection(Projection proj, java.awt.Graphics g)
     */
    public synchronized OMGraphicList generateGraphics() {

        try {
            Debug.message(classNameForDebug, this + " generateGraphics() " + tvs);
            // get projection in order to compute the bounding box.
            Projection proj = getProjection();

            OMGraphicList omgraphics = new OMGraphicList();
            omgraphics.setTraverseMode(OMGeometryList.LAST_ADDED_ON_TOP);
            // No iterator == no data => return.
            if (dataStorageIterator == null || !dataStorageIterator.isReady()) {
                return omgraphics;
            }

            if (proj == null) {
                return omgraphics;
            }

            // omgraphics.setTraverseMode(OMGraphicList.LAST_ADDED_ON_TOP);
            initCurrentProjBB();
            int nbOMG = 0; // only for debug purposes
            Debug.message(classNameForDebug, getPropertyPrefix() + "generateGraphics()");
            // first, only non legend thema are built
            // Thematic firstShared = null;
            for (int i = tvs.size() - 1; i >= 0; i--/* Thematic tv : tvs */) {
                Thematic tv = tvs.get(i);
                if (tv.isVisible()) {
                    /*
                     * if(tv.useSharedGraphics()){ firstShared = tv; }
                     */
                    dataStorageIterator.reset(); // provide a clean iterator
                    // for
                    // each visu
                    OMGraphicList omgl = tv.getGraphics(dataStorageIterator,
                            proj);
                    /*
                     * if (tv instanceof Legend && ((Legend) tv).isEditable()) {
                     * Debug.message(classNameForDebug, this + " set mode to
                     * interactive"); interactive = true; }
                     */

                    if (omgl != null && !omgl.isEmpty()) {
                        omgl.setAppObject(new XMLAppObject(tv.getTypeName() + ":" + tv.getName()));
                        omgraphics.add(omgl);
                        Debug.message(classNameForDebug, this + " " + omgl.size() + " objects " + tv);
                        nbOMG += omgl.size();
                    }
                }
            }

            Debug.message(classNameForDebug, this + " using " + omgraphics.size() + " visu layers made of " + nbOMG + " objects.");
            return omgraphics;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public void aSharedGraphicsThematicHasChanged() {
        Thematic topVisibleShared = null;
        for (Thematic tv : tvs) {
            if (tv.isVisible() && tv.useSharedGraphics() && topVisibleShared == null) {
                topVisibleShared = tv;
            }

        }
        if (topVisibleShared != null) {
            topVisibleShared.setBadCache(true);
        }
    }
    /**
     * GUI Internationalization.
     */
    public final static String VISU_TAB_PROPERTY_NAME = "visuConfigurationTab";
    /**
     * GUI Internationalization.
     */
    public final static String DATA_TAB_PROPERTY_NAME = "dataAndSelectionTab";
    /**
     * GUI Internationalization.
     */
    public final static String DS_TAB_PROPERTY_NAME = "dataSourceTab";
    /**
     * GUI Internationalization.
     */
    public final static String DS_TAB_NEW_PROPERTY_NAME = "dataSourceTab.new";
    /**
     * GUI Internationalization.
     */
    public final static String DS_TAB_CURRENT_PROPERTY_NAME = "dataSourceTab.current";
    protected SimplePalette box = null;
    protected ThematicPalette configPanel = null;
    protected JPanel dsGUI = new JPanel();

    @Override
    public Component getGUI() {

        Debug.message("ThematicDataDisplayLayer", this + " ThematicDataDisplayLayer.getGUI()");
        if (!createPalette) {
            return null;
        }
        if (dataStorageIterator == null || !dataStorageIterator.isReady()) {
            Debug.message(classNameForDebug, this + "DataStorageIterator is null or not ready" + dataStorageIterator);
            return null;
        }

        if (box == null) {
            box = new SimplePalette();
            // box.setPreferredSize(new Dimension(1000, 200));
            if (configPanel == null) {
                configPanel = new ThematicPalette(this, dataStorageIterator);
            }

            // General dataStorage GUI : configure current DS *or* change it
            // Code to move in SimpleDataDisplayLayer *later* ?

            final JPanel panCard = new JPanel(new CardLayout());
            // final JPanel cardSource = new JPanel(new CardLayout());

            JPanel panSource = new JPanel(new BorderLayout());
            panCard.add(panSource, "pansource");
            final JPanel editorPan = new JPanel();
            JPanel wrapEditor = new JPanel();
            //editorPan.setBorder(BorderFactory.createTitledBorder(""));
            wrapEditor.add(editorPan);
            panCard.add(new JScrollPane(wrapEditor), "editor");
            editorPan.setLayout(new BorderLayout());

            final Inspector inspector = new Inspector();
            Component component = DataStoragePropertyEditor.createPropertyGuiWithoutDefaultButtons(this, inspector);

            // Replace Ok button for inspector
            JButton ok = new JButton(i18n.get(ThematicDataDisplayLayer.class,
                    "applyDS", "apply"));
            ok.setToolTipText(i18n.get(ThematicDataDisplayLayer.class,
                    "applyDS", I18n.TOOLTIP,
                    "Apply the data storage modifications."));

            ok.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent ae) {

                    int n = JOptionPane.showConfirmDialog(
                            editorPan,
                            i18n.get(
                            ThematicDataDisplayLayer.class,
                            "dialog",
                            "If you continue, the current data storage will be deleted. Do you wish to continue ?"),
                            i18n.get(ThematicDataDisplayLayer.class,
                            "dialog.title", "Warning"),
                            JOptionPane.YES_NO_OPTION);

                    // System.out.println(inspector.collectProperties());

                    if (n == JOptionPane.YES_OPTION) {
                        // DataStorage is changing
                        setProperties(inspector.collectProperties());

                        // update GUI
                        refreshGUI();

                        // update Thematics
                        refreshGraphicsAndGUI();

                        // //TODOeditorFrame.setVisible(false);
                        CardLayout cl = (CardLayout) (panCard.getLayout());
                        cl.show(panCard, "pansource");
                    }
                }
            });

            // cancel button
            JButton cancel = new JButton(i18n.get(
                    ThematicDataDisplayLayer.class, "cancel", "cancel"));
            cancel.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent event) {
                    // //TODOeditorFrame.setVisible(false);
                    CardLayout cl = (CardLayout) (panCard.getLayout());
                    cl.show(panCard, "pansource");
                }
            });

            JPanel buttonsPan = new JPanel();
            buttonsPan.add(ok);
            buttonsPan.add(cancel);
            editorPan.add(component, BorderLayout.CENTER);
            editorPan.add(buttonsPan, BorderLayout.SOUTH);
            JToolBar toolBar = new JToolBar();
            toolBar.setFloatable(false);

            JButton editorButton = new JButton(Util.loadIcon("testRecycle.png"));
            editorButton.setToolTipText(i18n.get(ThematicDataDisplayLayer.class, "changeDS",
                    I18n.TOOLTIP,
                    "Change data storage while keeping current representation."));
            editorButton.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent event) {
                    // //editorFrame.setVisible(true);
                    CardLayout cl = (CardLayout) (panCard.getLayout());
                    cl.show(panCard, "editor");
                }
            });

            JButton gmlExportButton = new JButton(Util.loadIcon("export.png"));
            gmlExportButton.setToolTipText(i18n.get(ThematicDataDisplayLayer.class, "gmlExport",
                    I18n.TOOLTIP,
                    "GML Data export"));
            gmlExportButton.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent event) {
                    String gmlFile = ThemaMapFileChooserManager.dataSaveDialog("");
                    if (gmlFile == null || gmlFile.length() == 0)
                        return;

                    GMLWriter writer = new GMLWriter(gmlFile, gmlFile.matches(".*z$"));
                    dataStorageIterator.getDataStorage().write(writer);
                }
            });

            JButton csvExportButton = new JButton(Util.loadIcon("exportCSV.png"));
            csvExportButton.setToolTipText(i18n.get(ThematicDataDisplayLayer.class, "csvExport",
                    I18n.TOOLTIP,
                    "CSV Data export"));
            csvExportButton.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent event) {
                    String csvFile = ThemaMapFileChooserManager.csvSaveDialog("");
                    if (csvFile == null || csvFile.length() == 0)
                        return;
                    CSVWriter writer = new CSVWriter(csvFile, csvFile.matches(".*z$"));
                    dataStorageIterator.getDataStorage().write(writer);
                }
            });

            toolBar.add(editorButton);
            toolBar.addSeparator();
            toolBar.addSeparator();
            toolBar.add(gmlExportButton);
            toolBar.add(csvExportButton);
            toolBar.addSeparator();
            toolBar.addSeparator();
            final AdaptorButtonArray adp = new AdaptorButtonArray(this);
            JButton[] buttons = adp.getButtons();
            for (int i = 0; i < buttons.length; i++) {
                toolBar.add(buttons[i]);
            }
            toolBar.addSeparator();
            toolBar.add(adp.getRemoveAdaptorButton());

            // Current DataStorage GUI
            dsGUI.setBorder(BorderFactory.createTitledBorder(i18n.get(
                    ThematicDataDisplayLayer.class,
                    DS_TAB_CURRENT_PROPERTY_NAME, "Current settings")));
            dsGUI.add(dataStorageIterator.getGUI());

            panSource.add(toolBar, BorderLayout.NORTH);

            JScrollPane scrollSource = new JScrollPane(dsGUI);
            panData = new JPanel(/* new BorderLayout() */new CardLayout());
            JPanel panTable = new JPanel(new BorderLayout());
            JPanel panFormat = new JPanel(new BorderLayout());
            JPanel ptopd = new JPanel(new BorderLayout());
            JPanel ptopf = new JPanel(new BorderLayout());
            JButton bFormat = new JButton("Formatter les données");
            bFormat.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent arg0) {
                    CardLayout cl = (CardLayout) (panData.getLayout());
                    cl.show(panData, "format");
                }
            });
            JButton bData = new JButton("voir les données");
            bData.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent arg0) {
                    CardLayout cl = (CardLayout) (panData.getLayout());
                    cl.show(panData, "data");
                }
            });
            ptopd.add(bFormat, BorderLayout.EAST);
            ptopf.add(bData, BorderLayout.EAST);
            if (dataStorageIterator.getDataGUI() != null) {
                panTable.add(dataStorageIterator.getDataGUI(), BorderLayout.CENTER);
            }
            panFormat.add(ptopf, BorderLayout.NORTH);
            panData.add("data", panTable);
            panSource.add(scrollSource);
            MovableBoxesContainer mbcs = new MovableBoxesContainer(
                    BoxLayout.Y_AXIS);
            sourceMovableTab = new MovableBox(i18n.get(this,
                    DS_TAB_PROPERTY_NAME, "Data Source") + " - " + getName(), i18n.get(this, DS_TAB_PROPERTY_NAME,
                    "Data Source"),
                    /* mbcs, */
                    ThemaMapPanel.transferHandler, box.getTabs());
            mbcs.addElement(sourceMovableTab);
            box.getTabs().addTab("", mbcs);
            sourceMovableTab.add(panCard);
            sourceMovableTab.showAt(0);
            mbcs.updateGUI();

            MovableBoxesContainer mbcd = new MovableBoxesContainer(
                    BoxLayout.Y_AXIS);
            dataMovableTab = new MovableBox("", "",
                    ThemaMapPanel.transferHandler, box.getTabs());
            String records = " " + dataStorageIterator.getDataStorage().getCount() + " " +
                    i18n.get(this, "lines", "line(s)");
            dataMovableTab.setNames(i18n.get(this,
                    DATA_TAB_PROPERTY_NAME, "Data") + " - " + getName() + records, i18n.get(this, DATA_TAB_PROPERTY_NAME,
                    "Data") + records);
            mbcd.addElement(dataMovableTab);
            box.getTabs().addTab("", mbcd);
            dataMovableTab.add(panData);
            dataMovableTab.showAt(1);
            mbcd.updateGUI();

            if (dataStorageIterator.getDataGUI() != null) {
                MovableBoxesContainer mbcv = new MovableBoxesContainer(
                        BoxLayout.Y_AXIS);
                visuMovableTab =
                        new MovableBox(i18n.get(this, VISU_TAB_PROPERTY_NAME, "Visu Configuration") + " - " +
                        getName(), i18n.get(this, VISU_TAB_PROPERTY_NAME, "Visu Configuration"),
                        /* mbcv, */
                        ThemaMapPanel.transferHandler, box.getTabs());
                mbcv.addElement(visuMovableTab);
                box.getTabs().addTab("", mbcv);
                visuMovableTab.add(configPanel);
                visuMovableTab.showAt(2);
                mbcv.updateGUI();
            }

            int tdaIndex = box.getTabs().getTabCount();
            box.getTabs().add(new JPanel());
            box.getTabs().setComponentAt(tdaIndex, null);
            DropTabArea tda = new DropTabArea(ThemaMapPanel.transferHandler,
                    box.getTabs());
            box.getTabs().setTabComponentAt(tdaIndex, tda);
            //SimplePalette.customTabbedPaneUI(box.getTabs(), tdaIndex);
            for (int i = 0; i < box.getTabs().getTabCount() - 1; i++) {
                box.getTabs().setSelectedIndex(i);
            }

            //thematic tab
            box.getTabs().add(new JPanel());
            JComponent comp = configPanel.getVisuManager().asTool();
            comp.setBorder(BorderFactory.createEmptyBorder(0, 100, 0, 0));
            box.getTabs().setTabComponentAt(box.getTabs().getTabCount() - 1, comp);
            //SimplePalette.customTabbedPaneUI(box.getTabs(),box.getTabs().getTabCount() - 1);
            //            box.getTabs().addChangeListener(new ChangeListener() {
            //
            //                public void stateChanged(ChangeEvent e) {
            //                    ArrayList<Integer> unDecoratedIndices = new ArrayList<Integer>();
            //                    for (int i = 0; i < 3; i++) {
            //                        unDecoratedIndices.add(box.getTabs().getTabCount() - 1 - i);
            //                    }
            //                    SimplePalette.customTabbedPaneUI(box.getTabs(), unDecoratedIndices, box.getTabs().getTabCount() - 3);
            //                }
            //            });
            //refresh Button
            box.getTabs().add(new JPanel());
            box.getTabs().setTabComponentAt(box.getTabs().getTabCount() - 1, configPanel.getRefreshButton());
            box.getTabs().updateUI();
            //            ArrayList<Integer> unDecoratedIndices = new ArrayList<Integer>();
            //            for (int i = 0; i < 3; i++) {
            //                unDecoratedIndices.add(box.getTabs().getTabCount() - 1 - i);
            //            }
            //            SimplePalette.customTabbedPaneUI(box.getTabs(), unDecoratedIndices, box.getTabs().getTabCount() - 3);

        }
        return box;

    }

    @Override
    synchronized protected void refreshGUI() {
        if (dataStorageIterator != null && dataStorageIterator.getGUI() != null) {
            // Data GUI
            box.setVisible(false);
            if (dataStorageIterator.getDataGUI() != null) {
                panData.removeAll();
                panData.add(dataStorageIterator.getDataGUI(),
                        BorderLayout.CENTER);
            }
            configPanel.setDataStorageIterator(dataStorageIterator);
            for (Thematic tv : tvs) {
                tv.refreshGUI(dataStorageIterator);
            }

            // refresh DataStorage GUI
            dsGUI.removeAll();
            dsGUI.add(dataStorageIterator.getGUI());
            dsGUI.updateUI();
            box.setVisible(true);
        }
    }

    @Override
    protected void refreshDSGUI() {
        if (dataStorageIterator != null && dataStorageIterator.getGUI() != null) {
            // Data GUI
            if (box == null)
                return;
            box.setVisible(false);
            if (dataStorageIterator.getDataGUI() != null) {
                panData.removeAll();
                panData.add(dataStorageIterator.getDataGUI(),
                        BorderLayout.CENTER);
            }
            configPanel.setDataStorageIterator(dataStorageIterator);
            // refresh DataStorage GUI
            dsGUI.removeAll();
            dsGUI.add(dataStorageIterator.getGUI());
            dsGUI.updateUI();
            box.setVisible(true);
        }
    }

    /**
     * Remove all GUI panels from their parents - Usefull when a tabbed panel is
     * detached from the parent tabbedPane
     */
    public void removeGUI() {
        removeBox(sourceMovableTab);
        removeBox(dataMovableTab);
        removeBox(visuMovableTab);
    }

    private void removeBox(MovableBox b) {
        if (b == null) {
            return;
        }
        if (b.isExternalized()) {
            b.getParentFrame().dispose();
        } else {
            b.getParentBoxContainer().removeElement(b);
            b.getParentBoxContainer().updateGUI();
        }
    }

    /**
     * Refresh graphics and GUI. Use cached graphics if possible.
     */
    public void refreshGraphicsAndGUI() {
        Debug.message(classNameForDebug, this + "refreshGraphics() ==>");
        for (Thematic tv : tvs) {
            tv.refreshGUI(dataStorageIterator);
        }
        super.refreshGraphics();
        Debug.message(classNameForDebug, this + "refreshGraphics() <==");
    }

    public void refreshInteractiveAndBadCacheTVS() {
        for (Thematic tv : tvs) {
            if (tv.isInteractiveAndBadCache()) {
                tv.refreshGUI(dataStorageIterator);
                tv.refresh();
            }
        }
        super.refreshGraphics();
    }

    public void refreshInteractiveGraphics() {
        Debug.message(classNameForDebug, this + "repaint() ==>");
        //repaint();
        Debug.message(classNameForDebug, this + "repaint() <==");
    }

    /**
     * Check if thematic are ready
     *
     * @return
     */
    public boolean isReady() {
        boolean ready = true;
        // String errors = "";
        for (Thematic tv : tvs) {
            if (!tv.isReady()) {
                //missingErrors += tv.getMissingInfos();
                ready = false;
            }
        }
        return ready;
    }
    /**
     * Bounding boxes resp. east or west of the date line (lon 180 or -180
     * meridien)
     */
    protected BoundingBox eastCurrentProjBB, westCurrentProjBB;

    /**
     * inits the bounding boxes used for clipping
     * <p>
     * Is there still a pb if lowerRightLat &gt; upperLeftLon ?
     */
    public void initCurrentProjBB() {

        eastCurrentProjBB = westCurrentProjBB = null;
        // don't need bb for scale ~ max scale (nothing to crop)
        if ((getProjection().getMaxScale() - getProjection().getScale()) > getProjection().getScale() / 10) {
            return;
        }

        westCurrentProjBB = new BoundingBox((LatLonPoint) (getProjection().getUpperLeft()), (LatLonPoint) (getProjection().getLowerRight()));

        // test if the date line crosses the bb
        LatLonPoint center = getProjection().getCenter();
        // I assume that date line crosses the box if the following is false :
        // westCurrentProjBB.getUpperLeftLon() <= center.getLongitude() <=
        // westCurrentProjBB.getLowerRightLon()
        if (!(westCurrentProjBB.getUpperLeftLon() <= center.getLongitude() && center.getLongitude() <= westCurrentProjBB.getLowerRightLon())) {
            // cut out the box in 2 pieces around that 180° meridian
            // eastCurrentProjBB =
            // new BoundingBox(westCurrentProjBB.getLowerRightLon(),
            // westCurrentProjBB.getUpperLeftLat(),
            // 180, westCurrentProjBB.getLowerRightLat());
            // westCurrentProjBB =
            // new BoundingBox(-180, westCurrentProjBB.getUpperLeftLat(),
            // westCurrentProjBB.getUpperLeftLon(),
            // westCurrentProjBB.getLowerRightLat());
            eastCurrentProjBB = new BoundingBox(westCurrentProjBB.getUpperLeftLon(), westCurrentProjBB.getUpperLeftLat(),
                    180, westCurrentProjBB.getLowerRightLat());
            westCurrentProjBB = new BoundingBox(-180, westCurrentProjBB.getUpperLeftLat(), westCurrentProjBB.getLowerRightLon(),
                    westCurrentProjBB.getLowerRightLat());
        }
        // System.out.println("<== init WBB: " + westCurrentProjBB + ", EBB: " +
        // eastCurrentProjBB);
    }

    /**
     * test if the given bb overlaps at least one of eastCurrentProjBB or
     * westCurrentProjBB
     */
    public boolean overlapsCurrentProjBB(BoundingBox bb) {
        return bb == null || westCurrentProjBB == null || westCurrentProjBB.intersects(bb) || eastCurrentProjBB == null || eastCurrentProjBB.intersects(bb);
    }

    /**
     * Legend visualization is allways done with paint in order to allow mouse
     * edition
     */
    public void paint(java.awt.Graphics g) {
        Projection projection = getProjection();
        if (projection == null) {
            return;
        }
        super.paint(g);
        for (Thematic tv : tvs) {
            if (tv.isVisible() && tv.getInteractiveGraphics().size() > 0) {
                dataStorageIterator.reset();
                OMGraphicList omgl = tv.getInteractiveGraphics();
                omgl.setVisible(true);
                omgl.generate(projection);
                omgl.render(g);
            }
        }
    }

    /**
     * @see Layer#projectionChanged(com.bbn.openmap.event.ProjectionEvent pe)
     */
    public void projectionChanged(ProjectionEvent pe) {
        Projection proj = setProjection(pe);
        if (proj != null) {
            // Use the projection to gather OMGraphics in the layer,
            // and prepare the layer so that in the paint() method,
            // the OMGraphics get rendered.

            // Call any methods that kick off work to build graphics
            // here...
            // needsToRefresh = true;
            // fixed = false;
            refreshGraphicsAndGUI();
            // You get the paint() methods called by calling repaint():
            // repaint();
        }
        fireStatusUpdate(LayerStatusEvent.FINISH_WORKING);
    }

    /** Forward this event to all visus */
    public void dataStorageEvent(DataStorage source) {
        Debug.message("ThematicDataDisplayLayer", this + " caught a dataStorage event from " + source);

        for (Thematic tv : tvs) {
            tv.setBadCache(true);
            tv.dataStorageEvent(source);
        }
        refreshGraphicsAndGUI();

        if (source instanceof EditableDataStorage) {
            ((EditableDataStorage) source).setRequestor(this);
            ((EditableDataStorage) source).getDrawingTool().setCanvas((JComponent) getMapBean());
        }

        if (dataMovableTab != null) {//update data tab label
            String records = " " + source.getCount() + " " +
                    i18n.get(this, "lines", "line(s)");
            dataMovableTab.setNames(i18n.get(this,
                    DATA_TAB_PROPERTY_NAME, "Data") + " - " + getName() + records, i18n.get(this, DATA_TAB_PROPERTY_NAME,
                    "Data") + records);
        }
    }

    @Override
    public DataStorageIterator getDataStorageIterator() {
        return dataStorageIterator;
    }

    @Override
    public synchronized MapMouseListener getMapMouseListener() {
        if (getMouseModeIDsForEvents() != null) {
            // use StandardMapMouseInterpreter from OMGraphicHandlerLayer
            StandardMapMouseInterpreter smmi = (StandardMapMouseInterpreter) super.getMapMouseListener();
            smmi.setMouseTimerInterval(0);
            if (!thematicMapMouseListener.containsMouseListener(smmi)) {
                thematicMapMouseListener.addMapMouseListener(smmi);
            }
        }
        return thematicMapMouseListener;
    }

    public synchronized MapMouseInterpreter getMouseEventInterpreter() {
        if (getMouseModeIDsForEvents() != null && mouseEventInterpreter == null) {
            setMouseEventInterpreter(new TMMapMouseInterpreter(this));
        }

        return mouseEventInterpreter;
    }

    public void addMapMouseListener(MapMouseListener m) {
        if (m != null && !thematicsListeners.contains(m)) {
            thematicMapMouseListener.addMapMouseListener(m);
        }
    }

    public void addMapMouseListener(int index, MapMouseListener m) {
        if (m != null) {
            thematicMapMouseListener.addMapMouseListener(index, m);
        }
    }

    public void addLayerListener(LayerListener ll) {
        if (!layerListeners.contains(ll)) {
            layerListeners.add(ll);
        }
    }

    public void removeLayerListener(LayerListener ll) {
        layerListeners.remove(ll);
    }

    public void removeMapMouseListener(MapMouseListener m) {
        if (m != null) {
            thematicMapMouseListener.removeMapMouseListener(m);
        }
    }
    private ThematicMapMouseListener thematicMapMouseListener = new ThematicMapMouseListener();

    public OMResizeableGraphicList getLegend(LegendLayer legendLayer) {
        if (currentLegendLayer != legendLayer) {
            badLegendCache = true;
            currentLegendLayer = legendLayer;
        }

        legendsTv.clear();
        omLegendsTv = new OMResizeableGraphicList(this, legendLayer.getMouseControlPointsListener(), getName());
        omLegendsTv.setGuiOrientation(BoxLayout.X_AXIS);
        addLayerListener(legendLayer);
        for (Thematic tv : tvs) {
            if (!tv.isVisible()) {
                continue;
            }
            OMResizeableGraphic omLegend = tv.getLegend(legendLayer, legendLayer.getMouseControlPointsListener());
            if (omLegend == null) {
                continue;
            }

            legendsTv.put(tv, omLegend);
            omLegendsTv.add(omLegend);
        }

        badLegendCache = false;

        return omLegendsTv;
    }

    public void thematicAdded(Thematic thema) {
        badLegendCache = true;
        fireNeedsLegendUpdate();
    }

    public void thematicChanged(Thematic thema) {
        badLegendCache = true;
        fireNeedsLegendUpdate();
    }

    public void thematicRemoved(Thematic thema) {
        badLegendCache = true;
        fireNeedsLegendUpdate();
    }

    public void setLegendProperties(String prefix, Properties properties) {
        for (Map.Entry<Thematic, OMResizeableGraphic> currentEntry : legendsTv.entrySet()) {
            OMResizeableGraphic omLegend = currentEntry.getValue();
            Thematic tv = currentEntry.getKey();
            omLegend.setProperties(prefix + getName() + "." + tv.getName(), properties);
        }
    }

    public Properties getLegendProperties(String prefix, Properties properties) {
        for (Map.Entry<Thematic, OMResizeableGraphic> currentEntry : legendsTv.entrySet()) {
            OMResizeableGraphic omLegend = currentEntry.getValue();
            Thematic tv = currentEntry.getKey();
            properties = omLegend.getProperties(prefix + getName() + "." + tv.getName(), properties);
        }
        return properties;
    }

    public void animationModeChanged(MonoValuedFilterDataStorage ads) {
        //throw new UnsupportedOperationException("Not supported yet.");
    }

    public class ThematicMapMouseListener implements MapMouseListener {

        ArrayList<MapMouseListener> mouseListeners = new ArrayList<MapMouseListener>();

        public void addMapMouseListener(MapMouseListener m) {
            mouseListeners.add(m);
        }

        public void addMapMouseListener(int index, MapMouseListener m) {
            mouseListeners.add(index, m);
        }

        public void removeMapMouseListener(MapMouseListener m) {
            mouseListeners.remove(m);
        }

        public boolean containsMouseListener(MapMouseListener mml) {
            return mouseListeners.contains(mml);
        }

        /**
         * Return a list of the modes that are interesting to the
         * MapMouseListener. We can use mouse in Gestures mode to select
         * OMGraphics.
         */
        public String[] getMouseModeServiceList() {
            return new String[]{SelectMouseMode.modeID};
        }

        /**
         * Invoked when a mouse button has been pressed on a component.
         *
         * @param e
         *            MouseEvent
         * @return true (the listener is always able to process the event).
         */
        public boolean mousePressed(MouseEvent e) {
            // interactive mode can be efficient (if on) -- Layer's
            // omGraphicList will be not repaint with a correct RenderPolicy.
            // See refreshGraphics.
            Debug.message(classNameForDebug, this + "mousePressed() --> repaint()");

            for (MapMouseListener m : mouseListeners) {
                //System.out.println(" FORWAD PRESSED TO " + m.getClass());
                if (m.mousePressed(e)) {
                    return true;
                }
            }

            return false;
        }

        /**
         * Invoked when a mouse button has been released on a component. A
         * change (move all - resize border) is over.
         *
         * @param e
         *            MouseEvent
         * @return true if the listener was able to process the event.
         */
        public boolean mouseReleased(MouseEvent e) {

            for (MapMouseListener m : mouseListeners) {
                //System.out.println(m.hashCode());
                if (m.mouseReleased(e)) {
                    return true;
                }
            }

            return false;
        }

        /**
         * Invoked when the mouse has been clicked on a component.
         *
         * @param e
         *            MouseEvent
         * @return true if the listener was able to process the event.
         */
        public boolean mouseClicked(MouseEvent e) {
            for (MapMouseListener m : mouseListeners) {
                m.mouseClicked(e);
            }
            return false;
        }

        /**
         * Invoked when the mouse enters a component.
         *
         * @param e
         *            MouseEvent
         */
        public void mouseEntered(MouseEvent e) {
            for (MapMouseListener m : mouseListeners) {
                m.mouseEntered(e);
            }
        }

        /**
         * Invoked when the mouse exits a component.
         *
         * @param e
         *            MouseEvent
         */
        public void mouseExited(MouseEvent e) {
            // Debug.message("Legend", this + " mouseExited("+e.getX()+",
            // "+e.getY()+")");
            for (MapMouseListener m : mouseListeners) {
                m.mouseExited(e);
            }
        }

        /**
         * Invoked when a mouse button is pressed on a component and then
         * dragged.
         *
         * @param e
         *            MouseEvent
         * @return true if the listener was able to process the event.
         */
        public boolean mouseDragged(MouseEvent e) {
            Debug.message(classNameForDebug, this + "mouseDragged() --> repaint()");
            repaint();
            // }

            for (MapMouseListener m : mouseListeners) {
                if (m.mouseDragged(e)) {
                    return true;
                }
            }
            return false;
        }

        /**
         * Handle a mouse cursor moving without the button being pressed.
         */
        public void mouseMoved() {
        }

        /**
         * Invoked when the mouse button has been moved on a component (with no
         * buttons down).
         *
         * @param e
         *            MouseEvent
         * @return true if the listener was able to process the event.
         */
        public boolean mouseMoved(MouseEvent e) {
            for (MapMouseListener m : mouseListeners) {
                if (m.mouseMoved(e)) {
                    return true;
                }
            }
            return false;
        }
    }

    /*
     * Get the MapBean through the BeanContext. Set the layer property
     * "addToBeanContext" to true in order to be able to get it.
     */
    public MapBean getMapBean() {
        BeanContext mapHandler = getBeanContext();
        if (mapHandler == null) {
            return null;
        }

        Iterator it = mapHandler.iterator();
        while (it.hasNext()) {
            Object o = it.next();
            if (o instanceof MapBean) {
                return (MapBean) o;
            }
        }
        return null;
    }

    /** Returns the width of the application window through the MapBean. */
    public int getAppWindowWidth() {
        MapBean mb = getMapBean();
        return mb == null ? -1 : mb.getWidth();
    }

    /** Returns the width of the application window through the MapBean. */
    public int getAppWindowHeight() {
        MapBean mb = getMapBean();
        return mb == null ? -1 : mb.getHeight();
    }

    @Override
    public String getToolTipTextFor(OMGraphic omg) {
        Object o;
        if ((o = omg.getAttribute(VALUE)) != null) {
            if (o instanceof Number) {
                NumberFormat df = NumberFormat.getNumberInstance();
                //highlightInDataTable((Integer) o);
                return "<html><body>" + df.format((Number) o) + "</body></html>";
                // return String.format("<html>! %d !</html>", o);
            }
            // System.out.println(o.getClass());
            //highlightInDataTable((Integer) o);
            return "<html><body>" + o.toString() + "</body></html>";

        }
        if ((o = omg.getAttribute(INDEX_IN_DS)) != null) {
            String s = dataStorageIterator.getDataStorage().getElementAtInHTML(
                    (Integer) o);
            highlightInDataTable((Integer) o);
            return s;
        }
        return "<html><body>NO DATA</body></html>";
    }

    private void highlightInDataTable(int index) {
        if (dataStorageIterator.getDataStorage() instanceof EditableDataStorage)
            ((EditableDataStorage) dataStorageIterator.getDataStorage()).highlightInDataTable(index);

    }

    @Override
    public List<Component> getItemsForOMGraphicMenu(final OMGraphic omg) {
        if (!(dataStorageIterator.getDataStorage() instanceof EditableDataStorage))
            return super.getItemsForOMGraphicMenu(omg);
        final EditableDataStorage eds = (EditableDataStorage) dataStorageIterator.getDataStorage();
        JMenuItem itemDelete = new JMenuItem(i18n.get(this, "delete", "Delete Graphic"));
        JMenuItem itemDeleteAll = new JMenuItem(i18n.get(this, "deleteAssociated", "Delete Graphic and associated data"));
        JMenuItem itemEdit = new JMenuItem(i18n.get(this, "edit", "Edit Graphic"));
        itemDelete.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                if (JOptionPane.showConfirmDialog(null,
                        i18n.get(ThematicDataDisplayLayer.this, "deleteConfirm", "Do you really want to delete this graphic?"),
                        i18n.get(ThematicDataDisplayLayer.this, "deleting", "deleting a graphic"),
                        JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
                    Object o;
                    if ((o = omg.getAttribute(INDEX_IN_DS)) != null) {
                        if (eds.removeOMGraphicAt((Integer) o, omg)) {
                            setNeedsToRegenerateGraphics(true);
                            refreshGraphics();
                        }
                    }
                }
            }
        });

        itemDeleteAll.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                if (JOptionPane.showConfirmDialog(null,
                        i18n.get(ThematicDataDisplayLayer.this, "deleteConfirm", "Do you really want to delete this graphic?"),
                        i18n.get(ThematicDataDisplayLayer.this, "deleting", "deleting a graphic"),
                        JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
                    Object o;
                    if ((o = omg.getAttribute(INDEX_IN_DS)) != null) {
                        eds.removeDataObjectAt((Integer) o);
                    }
                }
            }
        });

        itemEdit.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                Object o;
                if ((o = omg.getAttribute(INDEX_IN_DS)) != null) {
                    eds.editOMGraphicAt((Integer) o, omg);
                    refreshGraphics();

                }
            }
        });

        ArrayList<Component> components = new ArrayList<Component>();
        components.add(itemDelete);
        components.add(itemDeleteAll);
        components.add(itemEdit);
        return components;
    }
//    public List<Component> getItemsForMapMenu(MapMouseEvent mme){
//
//       StandardMapMouseInterpreter smmi = (StandardMapMouseInterpreter) super.getMapMouseListener();
//       OMGraphic omg = smmi.getGeometryUnder(mme);
//
//        System.out.println("GET ITEM FOR MAP MENU -> " + omg);
//        return new ArrayList<Component>();
//    }

    public String getMissingErrors() {
        return missingErrors;
    }

    public void setMissingErrors(String missingErrors) {
        this.missingErrors = missingErrors;
    }

    public ArrayList<String> getVisuNames() {
        return visuNames;
    }

    public void drawingComplete(OMGraphic omg, OMAction action) {
        if (getDataStorageIterator().getDataStorage() instanceof EditableDataStorage) {
            ((EditableDataStorage) getDataStorageIterator().getDataStorage()).addOMGraphic(omg);
        }
        repaint();
    }

    public void setNeedsToRegenerateGraphics(boolean b) {
        for (Thematic tv : tvs) {
            tv.setBadCache(true);
        }

    }

    public ThematicPalette getThematicPalette() {
        if (configPanel == null) {
            configPanel = new ThematicPalette(this, dataStorageIterator);
        }
        return configPanel;
    }

    @Override
    public void animationAdded(MonoValuedFilterDataStorage ads) {
        setDataStorageIterator(new DataStorageIteratorAdapter(ads));

    }

    @Override
    public void animationRemoved(MonoValuedFilterDataStorage ads) {
        DataStorage ds = getDataStorageIterator().getDataStorage();
        setDataStorageIterator(new DataStorageIteratorAdapter(
                ((RemovableDataStorage) ds).removeItSelf()));
        for (LayerListener ll : layerListeners) {
            ll.titleUpdate("");
        }

    }

    @Override
    public void animationChanged(MonoValuedFilterDataStorage ads) {
        String name = ads.getFilterColumnName() + " : " + ads.getFilterColumnValue();
        // System.out.println("TDDL animation changed " + name + "\nlisteners = "+layerListeners.size());
        for (LayerListener ll : layerListeners) {
            ll.titleUpdate(name);
        }
    }

    public boolean isAnimated() {
        return getDataStorageIterator().getDataStorage() instanceof MonoValuedFilterDataStorage;
    }

    public ClassDescription[] getAddableVisuClassDescription() {
        return addableVisuClassDescription;
    }

    public void setSmooth(boolean b) {
        smooth = b;
        RenderPolicy rp = null;
        if (b) {
            rp = new StandardRenderPolicy();
        } else {
            rp = new BufferedImageRenderPolicy(this);
            //((BufferedImageRenderPolicy) rp).setUseImageBuffer(true);
        }
        setRenderPolicy(rp);
        refreshGraphics();
    }
}
