// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import com.bbn.openmap.omGraphics.OMCircle;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.proj.coords.LatLonPoint;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.OffsetableListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.OffsetableRaw;
import java.awt.Point;
import java.awt.geom.Point2D;

public class OMDisc extends OMOffsetable/*OMCircle*/ implements HaveWidthAndData, Comparable<OMDisc> {

    private OMCircle omCircle;

    /** Creates an OMDisc with a Lat-lon center and a lat-lon axis. It has a RENDERTYPE_LATLON. */
    public OMDisc(OffsetableRaw offRaw, String id, float latPoint, float lonPoint, int xOffset, int yOffset, float radius, String data, boolean v, boolean l) {
        //super(latPoint, lonPoint, radius);
        super(offRaw, id, latPoint, lonPoint, xOffset, yOffset, v, l);
        omCircle = new OMCircle(latPoint, lonPoint, radius);
        omCircle.putAttribute(ThematicDataDisplayLayer.VALUE, data);
        renderType = OMDisc.RENDERTYPE_OFFSET;
        //renderType = omCircle.getRenderType();
        putData(data);
        add(omCircle);
        drawJoiningLine();
        
    }

    public OMDisc(OffsetableRaw offRaw, float lat, float lon, float radius, String data) {
        this(offRaw, "", lat, lon, 0, 0, radius, data, false, false);
    }

    public OMDisc(OffsetableRaw offRaw, float lat, float lon, float radius, String data, boolean v, boolean l) {
        this(offRaw, "", lat, lon, 0, 0, radius, data, v, l);
    }

    /** Create a OMDisc, positioned with a lat-lon center and x-y diameter. */
    public OMDisc(OffsetableRaw offRaw, String id, float latPoint, float lonPoint, int xOffset, int yOffset, int diameter, String data, boolean v, boolean l) {
        super(offRaw, id, latPoint, lonPoint, xOffset, yOffset, v, l);
        renderType = OMDisc.RENDERTYPE_OFFSET;
        omCircle = new OMCircle(latPoint, lonPoint, xOffset, yOffset, diameter, diameter);
        omCircle.putAttribute(ThematicDataDisplayLayer.VALUE, data);
        putData(data);
        add(omCircle);
        drawJoiningLine();

    }

    public OMDisc(float lat, float lon, int diameter, String data) {
	this(null, "", lat, lon, 0, 0, diameter, data, false, false);
    }

    public OMCircle getOMCircle() {
        return omCircle;
    }

    public int compareTo(OMDisc omd) {
        /*System.out.println(OMDisc.RENDERTYPE_LATLON);
        System.out.println(OMDisc.RENDERTYPE_OFFSET);
        System.out.println(OMDisc.RENDERTYPE_UNKNOWN);
        System.out.println(OMDisc.RENDERTYPE_XY);*/
        int renderT = omCircle.getRenderType();
        if (renderT != omd.getRenderType()) {
            System.err.println("### Trying to compare to different rendered type discs: " + renderT +" - " + omd.getRenderType());
        }
        if (renderT == RENDERTYPE_LATLON || renderT == RENDERTYPE_OFFSET) {
            return (int) Math.signum(omCircle.getRadius() - omd.getOMCircle().getRadius());
        }
        return (int) Math.signum(omCircle.getWidth() - omd.getOMCircle().getWidth());
    }

    private void putData(String data) {
        putAttribute(ThematicDataDisplayLayer.VALUE, data);
    }

    public String getData() {
        return (String) getAttribute(ThematicDataDisplayLayer.VALUE);
    }

    public int getWidth() {
        return omCircle.getWidth();
    }

    @Override
    public void computeAnchors() {
        int width = omCircle.getWidth();

        Point2D bottom = new Point(0, width / -2 - MARGIN);
        Point2D top = new Point(0, width / 2 + MARGIN);
        Point2D left = new Point(width / -2 - MARGIN, 0);
        Point2D right = new Point(width / 2 + MARGIN, 0 + MARGIN);

        anchors = new Anchors(top, left, bottom, right);
    }

    @Override
    public void translate(int x, int y) {
        super.translate(x, y);
        omCircle.setOffX(getX());
        omCircle.setOffY(getY());
    }
}
