/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMLine;
import com.bbn.openmap.proj.coords.LatLonPoint;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Offseter;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.OffsetableListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.OffsetableRaw;
import java.awt.geom.Point2D;
import java.util.ArrayList;

/**
 *
 * @author dje
 */
public abstract class OMOffsetable extends OMGraphicList {

    public static final int MARGIN = 2;
    protected OffsetableRaw offsetableRaw;
    public OMGraphicList controlPoints = new OMGraphicList();
    private Offseter offseter;
    private boolean startDrag = false;
    private int x0;
    private int y0;
    //public ArrayList<OffsetableListener> offsetableListeners = new ArrayList<OffsetableListener>();
    protected String id;
    protected boolean cpVisibility; // control points visibility
    protected boolean lineVisibility;
    protected OMOffsetableControlPoint controlPoint;
    protected Anchors anchors;
    private int xOffset, yOffset;
    private float lat, lon;

    public enum Position {

        NORTH, WEST, SOUTH, EAST
    };

    public OMOffsetable() {
        id = "";
        cpVisibility = false;
    }

    public OMOffsetable(OffsetableRaw offRaw, String id, float lat, float lon, int offX, int offY, boolean cpVisibility, boolean lineVisibility) {
        this.offsetableRaw = offRaw;
        if(offRaw != null)//by example, is null when drawn is an overview panel
            setOffseter(offRaw.getOffseter());
        this.id = id;
        this.xOffset = offX;
        this.yOffset = offY;
        this.lat = lat;
        this.lon = lon;
        this.cpVisibility = cpVisibility;
        add(controlPoints);
        LatLonPoint latLonPoint = new LatLonPoint.Float(lat, lon);
        controlPoint = new OMOffsetableControlPoint(latLonPoint, this, offX, offY);
        //controlPoint.setAppObject(appObject);
        this.lineVisibility = lineVisibility;
        if (cpVisibility) {
            controlPoints.add(controlPoint);
        }
    }

    public void updatePosition(int x, int y) {
        if(startDrag){
            x0 = x - getX();
            y0 = y - getY();
            startDrag = false;
        }
        //System.out.println(getClass() + " -> UPDATE POSITION!!!!!");
        offsetableRaw.setBadCache(true);
        translate(x - x0 - getX(), y - y0 - getY());
        //setLocation(x - x0, y - y0);
        //setRenderType(OMGraphic.RENDERTYPE_XY);
        offseter.setOffset(id, x - x0, y - y0);
        //setLocation(getX() + offseter.getX(id) , getY() + offseter.getY(id));
        /*for (OffsetableListener off : offsetableListeners) {
            off.offsetableMoved(id, lat, lon, x, y);
        }*/
        offsetableRaw.getLayer().repaint();
    }

    public void startDrag() {
        if (offsetableRaw != null) {
            startDrag = true;
            offsetableRaw.addInteractiveGraphic(this);
        }
    }

    public void endDrag() {
        if (offsetableRaw != null) {
            startDrag = false;
            //setRenderType(OMGraphic.RENDERTYPE_OFFSET);
            //offseter.offsetableMoved(id, lat, lon, getX(), getY());
            offsetableRaw.clearInteractiveGraphics();
            offsetableRaw.setBadCache(true);
            offsetableRaw.getLayer().setNeedsToRegenerateGraphics(true);
            offsetableRaw.getLayer().refreshGraphicsAndGUI();
        }
    }

    public void setOffseter(Offseter off) {
        offseter = off;
    }

    public void setControlPointVisibility(boolean cpVisibility) {
        this.cpVisibility = cpVisibility;
    }

    private Position getObjectPosition(int x, int y) {
        if (x >= 0 && y <= x && y >= -x) {
            return Position.EAST;
        }
        if (x <= 0 && y <= -x && y >= x) {
            return Position.WEST;
        }
        if (y >= 0 && x <= y && x >= -y) {
            return Position.NORTH;
        }
        return Position.SOUTH;
    }

    public class Anchors {

        private Point2D top, left, bottom, right;

        public Anchors(Point2D top, Point2D left, Point2D south, Point2D right) {
            this.top = top;
            this.left = left;
            this.bottom = south;
            this.right = right;
        }

        public Point2D getTopAnchor() {
            return top;
        }

        public Point2D getLeftAnchor() {
            return left;
        }

        public Point2D getBottomAnchor() {
            return bottom;
        }

        public Point2D getRightAnchor() {
            return right;
        }
    }

    public void drawJoiningLine() {
        if (lineVisibility) {
            computeAnchors();
            Point2D point = null;
            Position pos = getObjectPosition(xOffset, yOffset);
            switch (pos) {
                case NORTH:
                    point = anchors.getBottomAnchor();
                    break;
                case SOUTH:
                    point = anchors.getTopAnchor();
                    break;
                case WEST:
                    point = anchors.getRightAnchor();
                    break;
                case EAST:
                    point = anchors.getLeftAnchor();
                    break;
            }

            add(new OMLine(lat, lon, 0, 0, (int) (xOffset + point.getX()), (int) (yOffset + point.getY())));
        }
    }

    public abstract void computeAnchors();
    public void translate(int x, int y){
        xOffset += x;
        yOffset += y;
    }
    public int getX(){return xOffset;}
    public int getY(){return yOffset;}
    public float getLat(){return lat;}
    public float getLon(){return lon;}
    //public abstract Box getBox();
//    public Box getBox(){return box;}
}
