// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import java.awt.geom.Arc2D;

import com.bbn.openmap.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.OffsetableRaw;
import java.awt.Point;
import java.awt.geom.Point2D;

public class OMPie extends OMOffsetable/*OMGraphicList*/ implements HaveWidthAndData, Comparable<OMPie> {

    private float width;
    private OMGraphicList arcs;

    public OMPie(OffsetableRaw offRaw, String id, float latPoint, float lonPoint, int xOffset, int yOffset, float radius, int nbParts, double[] angles, ColorScale colorScale, String data, boolean v, boolean l) {
        super(offRaw, id, latPoint, lonPoint, xOffset, yOffset, v, l);
        this.width = 2 * radius;
        arcs = new OMGraphicList();
        putData(data);
        float startAngle = 0.0f;
        for (int i = 0; i < nbParts; i++) {
            float angle = (float) angles[i];
            //// full lat/lon render type
            OMArc arc = new OMArc(latPoint, lonPoint, radius, startAngle, angle);
            arc.setArcType(Arc2D.PIE);
            arc.setFillPaint(colorScale.getColor(i));
            arcs.add(arc);
            startAngle += angle;
        }
        this.add(arcs);
        drawJoiningLine();
    }

    /**
     * XY RENDER TYPE attached to a latlon point
     * @param latPoint
     * @param lonPoint
     * @param diameter
     * @param nbParts
     * @param angles
     * @param colorScale
     * @param data
     */
    public OMPie(OffsetableRaw offRaw, String id, float latPoint, float lonPoint, int xOffset, int yOffset, int diameter, int nbParts, double[] angles, ColorScale colorScale, String data, boolean v, boolean l) {
        super(offRaw, id, latPoint, lonPoint, xOffset, yOffset, v, l);
        this.width = diameter;
        arcs = new OMGraphicList();
        putData(data);
        float startAngle = 0.0f;
        for (int i = 0; i < nbParts; i++) {
            float angle = (float) angles[i];
            // RENDERTYPE_XY
            OMArc arc = new OMArc(latPoint, lonPoint, xOffset, yOffset, (int) (width), (int) (width), startAngle, angle);
            arc.setArcType(Arc2D.PIE);
            arc.setFillPaint(colorScale.getColor(i));
            arcs.add(arc);
            startAngle += angle;
        }
        this.add(arcs);
        drawJoiningLine();
    }

    /**
     * Full XY RENDER TYPE
     * @param x
     * @param y
     * @param diameter
     * @param nbParts
     * @param angles
     * @param colorScale
     * @param data
     */
    public OMPie(int x, int y, int diameter, int nbParts, double[] angles, ColorScale colorScale, String data) {
        this.width = diameter;
        putData(data);
        arcs = new OMGraphicList();
        float startAngle = 0.0f;
        for (int i = 0; i < nbParts; i++) {
            float angle = (float) angles[i];
            // RENDERTYPE_XY
            OMArc arc = new OMArc(x, y, (int) (width), (int) (width), startAngle, angle);
            arc.setArcType(Arc2D.PIE);
            arc.setFillPaint(colorScale.getColor(i));
            arcs.add(arc);
            startAngle += angle;
        }
        this.add(arcs);
    }

    public void setFillPaint(ColorScale colorScale) {
        for (int i = 0; i < size(); i++) {
            getOMGraphicAt(i).setFillPaint(colorScale.getColor(i));
        }
    }

    public int compareTo(OMPie omd) {
        return (int) Math.signum(this.width - omd.width);
    }

    public int getWidth() {
        return (int) width;
    }

    private void putData(String data) {
        putAttribute(ThematicDataDisplayLayer.VALUE, data);
    }

    public String getData() {
        return (String) getAttribute(ThematicDataDisplayLayer.VALUE);
    }

    @Override
    public void computeAnchors() {

        Point2D bottom = new Point(0, (int) width / -2 - MARGIN);
        Point2D top = new Point(0, (int) width / 2 + MARGIN);
        Point2D left = new Point((int) width / -2 - MARGIN, 0);
        Point2D right = new Point((int) width / 2 + MARGIN, 0);
        anchors = new Anchors(top, left, bottom, right);
    }


    @Override
    public void translate(int x, int y) {
        super.translate(x, y);
        for (OMGraphic omg : arcs) {
            OMArc oma = (OMArc) omg;
            oma.setOffX(getX());
            oma.setOffY(getY());
        }
    }
}
