// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import java.awt.Paint;
import javax.swing.*;
import java.util.Properties;

public class OMResizeableColoredProportionalSymbolLegend extends OMResizeableFrame {

    OMResizeableFrame rpl;
    OMResizeableColorScaleLegend scl;
    
    public OMResizeableColoredProportionalSymbolLegend(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, Resizer resizer, MinMaxValues minMaxValues, int nbEdges, float startAngle, int starCoef, String columnName, Paint  lineColor, Paint fillColor, ColorScale colorScale, String[] colorNames, String colorDataColumnName, String name) {
	super(r, ml, xAnchor, yAnchor, 0, 0, true, name); // we'll set width and height at the end of the ctor.
	init(resizer, minMaxValues, nbEdges, startAngle, starCoef, columnName, lineColor, fillColor, colorScale, colorNames, colorDataColumnName);
    }

    public void init(Resizer resizer, MinMaxValues minMaxValues, int nbEdges, float startAngle, int starCoef, String columnName, Paint  lineColor, Paint fillColor, ColorScale colorScale, String[] colorNames, String colorDataColumnName) {

	this.clear();
	rpl = nbEdges > 20 
	    ? new OMResizeableDiscLegend
	    (refreshable, ml, 0, 0, resizer, minMaxValues, columnName, 
	     lineColor, fillColor, colorDataColumnName)
	    : new OMResizeableRegularPolygonLegend
	    (refreshable, ml, 0, 0, resizer, minMaxValues, nbEdges, startAngle,
	     starCoef, columnName, lineColor, fillColor, colorDataColumnName);
	scl = new OMResizeableColorScaleLegend(refreshable, ml, 0, rpl.getHeight()+10, 
					       colorScale, colorNames, 
					       colorDataColumnName, colorDataColumnName);
	// scl.showBorder(false);
	int width = rpl.getWidth()>scl.getWidth()?rpl.getWidth():scl.getWidth();
	int height = rpl.getHeight()+scl.getHeight();
	this.add(rpl);
	this.add(scl);
	this.extendBorder(width, height+20);
    }

    //===============================================================
    // PROPERTIES
    //===============================================================
    public static final String POLYS_PROPERTY_NAME = "polys";
    public static final String COLORS_PROPERTY_NAME = "colors";

    public void setProperties(String prefix, Properties properties) {
 	super.setProperties(prefix, properties);
	prefix = PropUtils.getScopedPropertyPrefix(prefix);

	rpl.setProperties(prefix + POLYS_PROPERTY_NAME, properties);
	scl.setProperties(prefix + COLORS_PROPERTY_NAME, properties);
    }
    
    public Properties getProperties(String prefix, Properties properties) {
	properties = super.getProperties(prefix, properties);
	prefix = PropUtils.getScopedPropertyPrefix(prefix);

	properties = rpl.getProperties(prefix + POLYS_PROPERTY_NAME, properties);
	properties = scl.getProperties(prefix + COLORS_PROPERTY_NAME, properties);

	return properties;
    }



     protected void visuFrameRefreshGUI() {
	if (guiPanel == null)
	    return;
	guiPanel.removeAll();
        rpl.getGUI();
        scl.getGUI();
        ThematicConfPanel confp = ThematicConfPanel.createThematicConfPanel(getName());
        guiContent = Box.createVerticalBox();
        guiContent.add(getToolGUI());
	// pb null pointer excetion
        guiContent.add(rpl.getGuiContent());
        guiContent.add(scl.getGuiContent());
        confp.setContent(new JScrollPane(
                guiContent,
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
        guiPanel.add(confp);
	guiPanel.updateUI();
    }

}
