// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import java.awt.Color;
import java.awt.Paint;
import java.awt.event.*;
import javax.swing.*;
import java.util.Properties;

public class OMResizeableHistogramLegend extends OMResizeableFrame {

    protected OMResizeableColorScaleLegend scl;
    private boolean mustBeReseted;
    public static final String mustBeResetedPropertyString = "mustBeReseted";
    public static final String histoLegendPropertyString = "histoLegend";
    private MinMeanMaxValuesAndSizes mmmValuesAndSizes;

    public OMResizeableHistogramLegend(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, Resizer resizer, MinMaxValues minMaxValues, double barsWidth, Paint lineColor, Paint fillColor, ColorScale colorScale, String[] colorNames, String name) {
        //super(r, ml, xAnchor, yAnchor, 0, 0, true, name, false); // we'll set width and height at the end of the ctor.
        super(r, ml, xAnchor, yAnchor, 0, 0, name, false);
        mmmValuesAndSizes = new MinMeanMaxValuesAndSizes(resizer, minMaxValues);
        init(resizer, minMaxValues, barsWidth, lineColor, fillColor, colorScale, colorNames);
    }

    public void init(Resizer resizer, MinMaxValues minMaxValues, double barsWidth, Paint lineColor, Paint fillColor, ColorScale colorScale, String[] colorNames) {

        clear();
        //omBars.clear();

        final int offset = 15;
        int x0 = getXAnchor() + offset / 2, y0 = getYAnchor() + offset;

        // title ???

//        MinMeanMaxValuesAndSizes minMeanMaxValuesAndSizes =
//                new MinMeanMaxValuesAndSizes(resizer, minMaxValues);
        //System.out.println("Histogram Legend MinMeanMaxValuesAndSizes "+minMedianMaxValuesAndSizes);

        int polygonIndex = mmmValuesAndSizes.getSize() - 1;
        //OMResizeableText[] usualNames = new OMResizeableText[polygonIndex + 1];

        int maxHeight = mmmValuesAndSizes.getSize(0);
        y0 += maxHeight;
        for (int i = 0; i <= polygonIndex; i++) {
            int height = mmmValuesAndSizes.getSize(i);
            //System.out.println("Histogram Legend " + i + " [" + x0 + ", " + y0 + "] " + height + " -- " + (minMeanMaxValuesAndSizes.getValue(i)));

            OMResizeableGroup group = new OMResizeableGroup(refreshable, ml, x0, y0, 0, 0, "Elem " + i);

            OMResizeableRect rect = new OMResizeableRect(refreshable, ml, x0, y0 - height, (int) barsWidth, height, fillColor, lineColor);

            int fontsize = 10;
            OMResizeableText text =
                    new OMResizeableText(refreshable, ml,
                    (int) (x0 + barsWidth + offset), y0 - height,
                    mmmValuesAndSizes.getValue(i),
                    OMText.JUSTIFY_LEFT, "bar value " + i, true, true);
            x0 += barsWidth * 3 / 2;

            group.add(rect);
            group.add(text);
            //usualNames[i] = text;
            //omBars.add(rect);
            //omGroups.add(group);
            add(group);
        }


        //customizedTexts.setTexts(usualNames);
        //customizedTexts.setTitle(title);

        scl = new OMResizeableColorScaleLegend(refreshable, ml, getXAnchor(), y0 + offset,
                colorScale, colorNames,
                "", "");
        scl.showBorder(false);
        int width = scl.getWidth();
        int height = y0 + offset + scl.getHeight();
        this.add(scl);
        this.extendBorder(width, height + 20);
    }
    //===============================================================
    // PROPERTIES
    //===============================================================
    public static final String BARS_TOTAL_PROPERTY_NAME = "bars.total";
    public static final String BAR_PROPERTY_NAME = "bar";
    public static final String GROUPS_PROPERTY_NAME = "groups";
    public static final String COLORS_PROPERTY_NAME = "colors";
    //protected CustomizedLegendText customizedTexts = new CustomizedLegendText();
    //protected ArrayList<OMResizeableRect> omBars = new ArrayList<OMResizeableRect>();
    //protected ArrayList<OMResizeableGroup> omGroups = new ArrayList<OMResizeableGroup>();

    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + histoLegendPropertyString + ".";
//
//        int total = PropUtils.intFromProperties(properties, prefix + BARS_TOTAL_PROPERTY_NAME, 0);
//        for (int i = 0; i < total && i < omBars.size(); i++) {
//            omGroups.get(i).setProperties(prefix + GROUPS_PROPERTY_NAME + Integer.toString(i), properties);
//            omBars.get(i).setProperties(prefix + BAR_PROPERTY_NAME + Integer.toString(i), properties);
//        }
//        customizedTexts.setProperties(prefix, properties);
        mustBeReseted = PropUtils.booleanFromProperties(properties, prefix + mustBeResetedPropertyString, false);
        scl.setProperties(prefix + COLORS_PROPERTY_NAME, properties);
    }

    public Properties getProperties(String prefix, Properties properties) {
                properties = super.getProperties(prefix, properties);
        if (mustBeReseted) {
            properties = removeAnchorFromProperties(prefix, properties);
            properties = removeSizeFromProperties(prefix, properties);
        }
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + histoLegendPropertyString + ".";

//        properties.put(prefix + BARS_TOTAL_PROPERTY_NAME, Integer.toString(omBars.size()));
//        for (int i = 0; i < omBars.size(); i++) {
//            properties = omGroups.get(i).getProperties(prefix + GROUPS_PROPERTY_NAME + Integer.toString(i), properties);
//            properties = omBars.get(i).getProperties(prefix + BAR_PROPERTY_NAME + Integer.toString(i), properties);
//        }

        properties = scl.getProperties(prefix + COLORS_PROPERTY_NAME, properties);
        properties.put(prefix + mustBeResetedPropertyString, String.valueOf(mustBeReseted));
        return properties;
    }

    public JComponent getToolGUI() {
        JComponent c = super.getToolGUI();
        final JCheckBox resetLegend = new JCheckBox(i18n.get(OMResizeableDiscLegend.class,"align","auto align"));
        resetLegend.setBorder(BorderFactory.createEmptyBorder(0,25,0,0));
        resetLegend.setSelected(mustBeReseted);
        resetLegend.setBackground(Color.WHITE);
        resetLegend.setOpaque(false);
        //ComponentSizer.fixComponentSize(resetLegend, new Dimension(20,20));
        resetLegend.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mustBeReseted = resetLegend.isSelected();
            }
        });
        resetLegend.setToolTipText("Reset bars positions according to their new sizes");
        c.add(resetLegend);
        return c;
    }

//    public Properties removeAnchorFromProperties(String prefix, Properties props) {
//        //props = grou.removeAnchorFromProperties(prefix + FRAME_PROPERTY_NAME, props);
//        for (int i = 0; i < omGroups.size(); i++) {
//            //omGroups.get(i).setProperties(prefix + GROUPS_PROPERTY_NAME + Integer.toString(i), properties);
//            props = omGroups.get(i).removeAnchorFromProperties(prefix + GROUPS_PROPERTY_NAME + Integer.toString(i), props);
//        }
//        for (int i = 0; i < omBars.size(); i++) {
//            //omGroups.get(i).setProperties(prefix + GROUPS_PROPERTY_NAME + Integer.toString(i), properties);
//            props = omBars.get(i).removeAnchorFromProperties(prefix + BAR_PROPERTY_NAME + Integer.toString(i), props);
//        }
//        customizedTexts.removeAnchorFromProperties(prefix, props);
//        return props;
//    }
}
