// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.Color;
import java.awt.Paint;
import java.awt.event.*;
import javax.swing.*;
import java.awt.Dimension;
import java.util.Properties;
import java.util.ArrayList;

/** 
 * OMResizeableRegularPolygonLegend construct a legend for colored proportionnal polygons.
 * <pre>
 *     ..........--------value
 *     ..........
 *     ..........
 *     ..........
 *     ..........
 *     ..........
 * </pre>
 * The asciiart is copyrighted Yannick Lange ;-)
 * @author Jacques.Madelaine@info.unicaen.fr
 */
public class OMResizeableRegularPolygonLegend extends OMResizeableFrame {

    private boolean mustBeReseted;
    private MinMeanMaxValuesAndSizes minMeanMaxValuesAndSizes;

    public static final String multiRectLegendPropertyString = "multiRectLegend";
    public static final String mustBeResetedPropertyString = "mustBeReseted";
    
    /** if columnName is null, draw only 1 polygon (because they are of fixed size. */
    public OMResizeableRegularPolygonLegend(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, Resizer resizer, MinMaxValues minMaxValues, int nbEdges, float startAngle, int starCoef, String columnName, Paint  lineColor, Paint fillColor, String name) {
	super(r, ml, xAnchor, yAnchor, 0, 0, true, name); // we'll set width and height at the end of the ctor.
	//this.resizer = resizer;
        minMeanMaxValuesAndSizes =
		    new MinMeanMaxValuesAndSizes(resizer, minMaxValues);
	init(minMaxValues, nbEdges, startAngle, starCoef, columnName, lineColor, fillColor);
    }

    public void init(MinMaxValues minMaxValues, int nbEdges, float startAngle, int starCoef, String columnName, Paint  lineColor, Paint fillColor) {

	clear();
	final int offset = 15; // we should compute this offset from the extend of the current font !!!

	int x0 = getXAnchor() + offset;
	int y0 = getYAnchor() + offset;
 
	// place the title on top
	OMResizeableText title = 
	    new OMResizeableText(refreshable, ml, x0, y0+offset/2,
				 columnName, OMText.JUSTIFY_CENTER, "polygon title");
	title.setFontStyle(java.awt.Font.BOLD);
	this.add(title);
	int fontsize = title.getFontSize();
	//System.out.println("*Resizer: "+resizer+"\n1.0E9 -> "+resizer.getSize(1E9));

	//int biggestRadius = (int)Math.round(resizer.getSize(minMaxValues.getMax()));


        //0 index is the biggest polygon
	int biggestRadius = minMeanMaxValuesAndSizes.getSize(0);
	y0 += 2 * fontsize;

	int polygonIndex = minMeanMaxValuesAndSizes.getSize() - 1;
	if (columnName == null && polygonIndex > 1)
	    polygonIndex = 0; 

	OMResizeableText[] usualNames = new OMResizeableText[polygonIndex + 1];

	double trueRadiusFactor = getTrueRadiusFactor(nbEdges, startAngle);
	// in reverse order so that the smallest polygon is on top.
	for(int i = polygonIndex; i >= 0; i--) {
	    int radius = minMeanMaxValuesAndSizes.getSize(i);
	    // Debug.message("Legend2", "LegendPolygon ["+(x0 + biggestRadius)+", "+ (y0 + 2*biggestRadius - radius) + "] " + radius + " " + biggestRadius+ " ["+(x0+biggestRadius)+", "+ (y0+2*biggestRadius-2*radius)+"] "+(minMeanMaxValuesAndSizes.getValue(i)));
	    OMResizeableGroup group = 
		new OMResizeableGroup(refreshable, ml, 
				      x0, (int)Math.round(y0+2*biggestRadius-radius*trueRadiusFactor), 0, 0, "Elem "+i);

	    OMResizeableRegularPolygon poly =
		new OMResizeableRegularPolygon(refreshable, null, 
					       x0+biggestRadius, 
					       (int)Math.round(y0+2*biggestRadius-radius*trueRadiusFactor), 
					       nbEdges, radius, startAngle, starCoef, 
					       lineColor, fillColor);
	    // to add: OMResizeableLine
	    OMResizeableText text = 
		new OMResizeableText(refreshable, ml, 
				     x0+2*biggestRadius+offset, 
				     (int)Math.round(y0+2*biggestRadius-2*radius*trueRadiusFactor)+fontsize,
				     minMeanMaxValuesAndSizes.getValue(i), 
				     OMText.JUSTIFY_LEFT, "polygon value "+i);
	    group.add(poly);
	    group.add(text);
            this.add(group);
	}
	int frameWidth = 2*biggestRadius+100;
	title.translate(Math.round(frameWidth/2), 0);

	this.extendBorder(x0+frameWidth, y0+2*biggestRadius+offset);
    }

    /** Factor to apply to radius in order to bottom align the polygons */
    private double getTrueRadiusFactor(int nbEdges, float startAngle) {
	double r = 0;
	double angle = startAngle;
	double angleIncr = 2*Math.PI/nbEdges;
	for(int i = 0; i < nbEdges; i++, angle += angleIncr) { 
	    double y = Math.sin(angle);
	    // System.out.println(y);
	    if (r < y)
		r = y;
	}
	// System.out.println(r);
	return r;
    }

    //===============================================================
    // PROPERTIES
    //===============================================================
    public static final String POLYS_TOTAL_PROPERTY_NAME = "polys.total";
    public static final String POLYS_PROPERTY_NAME = "polys";
    public static final String GROUPS_PROPERTY_NAME = "groups";

//    protected ArrayList<OMResizeableRegularPolygon> omPolys = new ArrayList<OMResizeableRegularPolygon>();
//    protected ArrayList<OMResizeableGroup> omGroups = new ArrayList<OMResizeableGroup>();

    public static final String FRAME_PROPERTY_NAME = "frame";
//    protected OMResizeableGroup groups;



    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + multiRectLegendPropertyString + ".";
        minMeanMaxValuesAndSizes.setProperties(prefix, properties);
        mustBeReseted = PropUtils.booleanFromProperties(properties, prefix + mustBeResetedPropertyString, false);
    }
    
    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        //System.out.println("props avant = " + properties);
        if (mustBeReseted) {
            properties = removeAnchorFromProperties(prefix, properties);
            properties = removeSizeFromProperties(prefix, properties);
        }
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + multiRectLegendPropertyString + ".";
        properties = minMeanMaxValuesAndSizes.getProperties(prefix, properties);
        properties.put(prefix + mustBeResetedPropertyString, String.valueOf(mustBeReseted));
        return properties;
    }

    public JComponent getToolGUI(){
        JComponent c = super.getToolGUI();
        JButton resetLegend = new JButton(Util.loadIcon("undo.png"));
        resetLegend.setBackground(Color.WHITE);
        resetLegend.setOpaque(false);
        ComponentSizer.fixComponentSize(resetLegend, new Dimension(20,20));
        resetLegend.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                mustBeReseted = true;
            }
        });
        resetLegend.setToolTipText("Reset polygons positions according to their new sizes");
        //c.add(resetLegend);
        return c;
    }

}
