// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.util.Colors;
import fr.unicaen.openmap.util.FontDialog;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.text.JTextComponent;
import javax.swing.plaf.basic.BasicButtonUI;
import java.awt.*;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Properties;

public class OMResizeableText extends OMResizeableGraphic {

    private OMText omText;
    private Color color = Color.BLACK;

    public OMResizeableText(Refreshable r, MouseControlPointsListener ml, int px1, int py1, String stuff) {
        this(r, ml, px1, py1, stuff, OMText.JUSTIFY_LEFT);
    }

    public OMResizeableText(Refreshable r, MouseControlPointsListener ml, int px1, int py1, String stuff, int just) {
        this(r, ml, px1, py1, stuff, just, null);
    }

    public OMResizeableText(Refreshable r, MouseControlPointsListener ml, int px1, int py1, String stuff, int just, String nameForGUI) {
        this(r, ml, px1, py1, stuff, just, nameForGUI, true);
    }

    public OMResizeableText(Refreshable r, MouseControlPointsListener ml, int px1, int py1, String stuff, int just, String nameForGUI, boolean withControlPoints){
        this(r, ml, px1, py1, stuff, just, nameForGUI, withControlPoints, false);
    }

    public OMResizeableText(Refreshable r, MouseControlPointsListener ml, int px1, int py1, String stuff, int just, String nameForGUI, boolean withControlPoints, boolean ra) {
	super(r, ml, nameForGUI, ra);
	omText = new OMText(px1, py1, stuff, just);
	setJustify(just);
	if (withControlPoints)
	    addOMControlPoint(new OMControlNPoint
			      (px1, py1, 
			       OMResizeableGraphic.OMControlPointRadius, 
			       Color.yellow, Color.red, refreshable.getMapBean()));
	
	add(omText);
	// System.out.println("### new "+Util.className(this)+" : "+ml+" name:"+getName()+" "+stuff);
    }

    public void translate(int dx, int dy) {
        // System.out.println("TRANSLATE TEXT "+dx+","+dy + " for " + this);
        omText.setX(omText.getX() + dx);
        omText.setY(omText.getY() + dy);
        for (int i = 0; i < omControlPoints.size(); i++) {
            ((OMControlPoint) omControlPoints.getOMGraphicAt(i)).translate(dx, dy);
        }
    }

    /** 
     * 
     */
    //nothing to do as we don't have angle control point. 
    // * <br/> We could imagine a sub class that will change its font size 
    public void extend(int dw, int dh) {
        //// An example of false good idea :
        //// Extend message comes from a frame (no OMControlAngle) => we translate the text
        // translate(0, dh);
        //// It works fine for text below an embedded frame, but not for the one above !
        //// Must solve this problem in OMResizeableFrame.extend
    }

    public int getXAnchor() {
        return omText.getX();
    }

    public int getYAnchor() {
        return omText.getY();
    }
    protected int width = -1;
    protected int height = -1;

    protected void initWidthHeight() {
        Rectangle2D rect = Util.getStringBounds(getText(), getFont());
        width = (int) (rect.getWidth() * 1.1);
        height = (int) rect.getHeight();
    }

    public int getWidth() {
        if (width < 0) {
            initWidthHeight();
        }
        return width;
    }

    public int getHeight() {
        if (height < 0) {
            initWidthHeight();
        }
        return height;

    }

    protected void updateFromGUI() {
        if (textComponent != null) {
            setText(textComponent.getText());
        }
    }

    public String getText() {
        return omText.getData() == null ? "" : omText.getData();
    }

    public void setText(String text) {
        omText.setData(text);
        width = height = -1;
    }

    public Font getFont() {
        return omText.getFont();
    }

    public void setFont(Font font) {
        omText.setFont(font);
    }

    public void setLinePaint(Paint p) {
        omText.setLinePaint(p);
    }

    public void setJustify(int just) {
        omText.setJustify(just);
    }

    public int getJustify() {
        return omText.getJustify();
    }
    private static final String[] names = {"left", "center", "right"};
    private static final ImageIcon[] justifyIcons = {Util.loadIcon("visu/stock_text_left.png"),
        Util.loadIcon("visu/stock_text_center.png"),
        Util.loadIcon("visu/stock_text_right.png")};

    private class JustifyRadioButton extends JRadioButton implements ActionListener {

        private MouseListener buttonMouseListener = new MouseAdapter() {

            public void mouseEntered(MouseEvent e) {
                setBorderPainted(true);
            }

            public void mouseExited(MouseEvent e) {
                setBorderPainted(false);
            }
        };
        int just;

        public JustifyRadioButton(int just) {
            super();
            this.just = just;
            // use setIcon to ensure size is ok. PreferredSize is just bullshit.
            setIcon(Util.loadIcon("visu/stock_text_" + names[just] + ".png"));
            setToolTipText("Justify " + names[just]);
            addActionListener(this);
            setOpaque(false);
            setPreferredSize(new Dimension(20, 20));
            setBorder(BorderFactory.createEtchedBorder());
            setBorderPainted(false);
            addMouseListener(buttonMouseListener);
            setUI(new BasicButtonUI());
        }

        public void actionPerformed(ActionEvent e) {
            setJustify(just);
            refreshGraphics();
        }

        public void paintComponent(Graphics g) {
            try {
                if (isSelected()) {
                    g.setColor(new Color(100, 200, 150));
                    //g.setColor(color);
                    g.fillRect(2, 2, 16, 16);
                }
                justifyIcons[just].paintIcon(this, g, 2, 2);
            } catch (Exception e) {
                super.paintComponent(g);
            }
        }
    }

    protected JComponent getJustificationGUI() {
        Box box = Box.createHorizontalBox();
        ButtonGroup group = new ButtonGroup();
        for (int i = 0; i < 3; i++) {
            JRadioButton rb = new JustifyRadioButton(i);
            if (i == getJustify()) {
                rb.setSelected(true);
            }
            group.add(rb);
            box.add(rb);
        }
        return box;
    }

    public void updateText(String text) {
        setText(text);
        refreshable.refreshInteractiveGraphics();
    }
    protected JTextComponent textComponent;

    public JComponent getGUI() {
        //Box box = Box.createHorizontalBox();
        JPanel p = new JPanel();
        JComponent tg = getToolGUI();
        //tg.setBorder(BorderFactory.createLineBorder(Color.BLUE));
        p.add(tg);
        //tg.add(box);
        //box.setBorder(BorderFactory.createLineBorder(Color.RED));
        JLabel lab = new JLabel(getName());
        ComponentSizer.fixComponentSize(lab, new Dimension(100,25));
        //lab.setPreferredSize(new Dimension(200,20));
        //lab.setMaximumSize(new Dimension(200,20));
        p.add(lab);

        //box.add(Box.createHorizontalStrut(5));
        textComponent = new JTextField(getText());
        ComponentSizer.fixComponentSize(textComponent, new Dimension(100 , 25));
        textComponent.setForeground(color);
        //textComponent.addKeyListener(listener);


        ((JTextField) textComponent).addKeyListener(new KeyListener() {

            public void keyPressed(KeyEvent e) {
                //setText(textComponent.getText());
                // ((ThematicDataDisplayLayer)refreshable).repaint();
            }

            public void keyReleased(KeyEvent e) {
                setText(textComponent.getText());
                refreshable.refreshInteractiveGraphics();
                //refreshGraphics();

            }

            public void keyTyped(KeyEvent e) {
                //setText(textComponent.getText());
                //refreshGraphics();
            }
        });

        p.add(textComponent);

        final FontDialog fontDialog = new FontDialog(omText.getFont(), true, getJustificationGUI());
        JButton fontButton = new JButton("", Util.loadIcon("visu/fonts.png"));
        ComponentSizer.fixComponentSize(fontButton, new Dimension(25,25));
        fontButton.setPreferredSize(new Dimension(20, 20));
        fontButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent event) {
                fontDialog.setVisible(true);

                if (!fontDialog.isCanceled()) {
                    omText.setFont(fontDialog.getSelectedFont());
                    omText.setLinePaint(fontDialog.getFontColor());
                    textComponent.setFont(fontDialog.getSelectedFont());
                    textComponent.setForeground(fontDialog.getFontColor());
                    color = fontDialog.getFontColor();
                    refreshGraphics();
                }
            }
        });
        fontButton.setBackground(Color.WHITE);
        fontButton.setOpaque(false);
        fontButton.setBorder(null);
        p.add(fontButton);

        //p.add(getJustificationGUI());
        //ComponentSizer.fixComponentSize(box, new Dimension(300, 25));
        //JPanel wrap = new JPanel(new BorderLayout());
        //wrap.add(box, BorderLayout.NORTH);
        //ComponentSizer.fixComponentSize(p, new Dimension(330,35));
        return p;
    }
    /**
     * Property name for font name.
     */
    public final static String FONT_PROPERTY_NAME = "font";
    /**
     * Property name for font color
     */
    public final static String FONT_COLOR_PROPERTY_NAME = "font.color";
    /**
     * Property name for font size.
     */
    public final static String FONT_SIZE_PROPERTY_NAME = "font.size";
    /**
     * Property name for font style.
     */
    public final static String FONT_STYLE_PROPERTY_NAME = "font.style";
    /**
     * Property name for x anchor.
     */
    //    public final static String X_ANCHOR_PROPERTY_NAME = "anchor.x";
    /**
     * Property name for y anchor.
     */
    //    public final static String Y_ANCHOR_PROPERTY_NAME = "anchor.y";
    /**
     * Property name for x anchor.
     */
    public final static String JUSTIFY_PROPERTY_NAME = "justify";
    /**
     * Property name for data used in OMText.
     */
    public final static String DATA_PROPERTY_NAME = "data";

    public void setFontStyle(int s) {
	omText.setFont(omText.getFont().deriveFont(s));
    }

    public void setFontSize(float s) {
	omText.setFont(omText.getFont().deriveFont(s));
    }

    public int getFontSize() {
	return omText.getFont().getSize();
    }

    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);

        //Font settings
        omText.setFont(new Font(properties.getProperty(prefix + FONT_PROPERTY_NAME, omText.getFont().getName()),
                PropUtils.intFromProperties(properties, prefix + FONT_STYLE_PROPERTY_NAME, omText.getFont().getStyle()),
                PropUtils.intFromProperties(properties, prefix + FONT_SIZE_PROPERTY_NAME, omText.getFont().getSize())));
        color = Colors.parseColor(properties.getProperty(prefix + FONT_COLOR_PROPERTY_NAME, "000000"));
        omText.setLinePaint(color);
        //Justify
        setJustify(PropUtils.intFromProperties(properties, prefix + JUSTIFY_PROPERTY_NAME, omText.getJustify()));

	//Text
	setText(properties.getProperty(prefix + DATA_PROPERTY_NAME, getText()));

        // System.out.println("Resizeable text setProps " + omText.getData() + " - visible = " + getVisible());
    }

    public Properties getProperties(String prefix, Properties properties) {
        super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        //System.out.println("SET TEXT PROPERTIES 2" + prefix);
        //Font settings
        properties.put(prefix + FONT_PROPERTY_NAME, getFont().getName());
        properties.put(prefix + FONT_COLOR_PROPERTY_NAME, Integer.toHexString(color.getRGB()));
        properties.put(prefix + FONT_STYLE_PROPERTY_NAME, String.valueOf(getFont().getStyle()));
        properties.put(prefix + FONT_SIZE_PROPERTY_NAME, String.valueOf(getFont().getSize()));
        //Justify
        properties.put(prefix + JUSTIFY_PROPERTY_NAME, Integer.toString(getJustify()));

        //Text
        properties.put(prefix + DATA_PROPERTY_NAME, getText());
        //properties.put(prefix + VISIBILITY_PROPERTY_NAME, visible);
        //System.out.println("Resizeable text getProps" + omText.getData() + " - visible = " + visible);
        return properties;
    }

    //    public void setTextProperties(String prefix, Properties props){
    //        setProperties(prefix, props);
    //    }

    /*public void addKeyListener(int index, String[] names, Pie pie){
    listener = new OMResizeableTextListener(this, index, names, pie);
    }*/
    public String toString() {
	return super.toString() + " name: " + getName() + " " + getText();
    }

//    public static OMResizeableText contructOMResiezableText(OMLabel oml, Refreshable r, MouseControlPointsListener ml, Projection p){
//        OMText omt = oml.getOMText();
//        System.out.println("LATLON "+omt.getLat()+","+omt.getLon());
//        Point2D pt = p.forward(omt.getLat(), omt.getLon());
//        System.out.println("XY "+pt.getX()+","+pt.getY());
//        return new OMResizeableText(r, ml,(int)pt.getX(), (int)pt.getY() - 10, omt.getData());
//    }
}
