// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;

import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMPoint;

import fr.unicaen.openmap.dataStorage.Centroid;
import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.dataStorage.OMDataLocation;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ColorizerPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicGUI;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMDisc;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMRegularPolygon;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMSizeableGraphicList;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.Refreshable;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableColoredProportionalSymbolLegend;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableGraphic;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableFrame;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Colorizer;
import fr.unicaen.openmap.util.classification.ClassificationListener;
import fr.unicaen.openmap.util.classification.Quantile;
import fr.unicaen.openmap.util.gui.ComponentSizer;

public class ColoredProportionalSymbol extends ProportionalSymbol implements
        ClassificationListener {

    public static final String colorDataColumnNamePropertyString = "colorDataColumnName";
    protected String colorDataColumnName = "";
    protected Colorizer colorizer = new Colorizer();
    //private JComponent panelGUI;
    JComboBox dataList;

    public ColoredProportionalSymbol(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(ColoredProportionalSymbol.class,
                "coloredProportionalSymbol", "Colored Proportional Symbol"));
    }

    public ColoredProportionalSymbol(ThematicDataDisplayLayer layer,
            String name, String typeName) {
        super(layer, name, typeName);
        colorizer.addClassificationListener(this);
    }

    @Override
    public JComponent getGUI(final DataStorageIterator dsi) {
        thematicGUI = (ThematicGUI) super.getGUI(dsi);
        this.proportionalSymbolPanel.removePicto();// no pictogramm in colored
        JPanel mainPan = new JPanel();
        mainPan.setLayout(new BoxLayout(mainPan, BoxLayout.X_AXIS));

        final JPanel coloredRegularpolygonPanel = new JPanel();

        try {
            graphicsPanelToggleControlsDisplay(false, true, false, false, false);

            coloredRegularpolygonPanel.setLayout(new BoxLayout(
                    coloredRegularpolygonPanel, BoxLayout.Y_AXIS));

            final ColorizerPanel colorizerPanel = colorizer.getGUI();
            dataList = new JComboBox(dsi.getDataColumnNames());
            if (colorDataColumnName != null & !"".equals(colorDataColumnName)) {
                dataList.setSelectedIndex(dsi.getIndexForColumnName(colorDataColumnName));
            } else {
                if (dsi.getDataColumnNames().length > 0) {
                    colorDataColumnName = dsi.getDataColumnNames()[0];
                }
            }

            dataList.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    try {
                        int index = ((JComboBox) e.getSource()).getSelectedIndex();
                        setColorDataColumnName(dsi.getRealColumnName(index));
                        // colorizerPanel.refreshPanel();
                    } catch (ArrayIndexOutOfBoundsException ex) {
                    }
                }
            });
            ComponentSizer.fixComponentSize(dataList, new Dimension(120, 20));
            JPanel pData = new JPanel();
            pData.add(new JLabel(i18n.get(this, "colorDataColumnName",
                    "Color Data Column Name: ")));
            pData.add(dataList);
            pData.add(colorizerPanel.getClassifList());
            JPanel wrapColorizerPanel = new JPanel(new BorderLayout());
            wrapColorizerPanel.add(pData, BorderLayout.NORTH);
            wrapColorizerPanel.add(colorizerPanel, BorderLayout.CENTER);
            coloredRegularpolygonPanel.add(wrapColorizerPanel);
            ThematicConfPanel classifConfPanel =
                    ThematicConfPanel.createThematicConfPanel(i18n.get(this, "classification_color", "Classification"), false);
            classifConfPanel.setContent(coloredRegularpolygonPanel);
            JPanel wrapClassConf = new JPanel();
            wrapClassConf.add(classifConfPanel);
            thematicGUI.add(wrapClassConf);
            thematicGUI.addThematicConfigurationPanel(classifConfPanel);
            if (colorizer.getClassification() == null) {
                Quantile quantile = new Quantile(null, 4);
                quantile.reInit(dsi, colorDataColumnName);
                colorizer.setClassification(quantile);
            }
        } catch (Exception e) {
        }

        return thematicGUI;
    }

    @Override
    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        //minMedianMaxValuesAndSizes.setProperties(prefix, properties);
        colorizer.setProperties(prefix, properties);
        colorDataColumnName = properties.getProperty(prefix + colorDataColumnNamePropertyString, "color");
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        //properties = minMedianMaxValuesAndSizes.getProperties(prefix, properties);
        properties = colorizer.getProperties(prefix, properties);
        properties.put(prefix + colorDataColumnNamePropertyString,
                colorDataColumnName);
        return properties;
    }

    @Override
    public OMGraphicList generateGraphics(DataStorageIterator dsi) {

        resizer.setMinMaxValue(dsi);
        colorizer.setData(dsi, colorDataColumnName);

        //OMGraphicList omgl = new OMGraphicList();
        OMSizeableGraphicList omglDiscs = new OMSizeableGraphicList<OMDisc>();
        //OMGraphicList ompl = new OMGraphicList(); // for the OMPoint (Discs of
        // null sizes)
        float startAngleInRadians = getStartAngleInRadians();

        boolean offVis = offsetsOn ? offseter.isVisible() : false;
        boolean offLineVis = offsetsOn ? offseter.isLineVisible() : false;

        while (dsi.hasNext()) {
            DataObject data = dsi.next();
            OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
            if (omdl == null) {
                continue;
            }

            Centroid centroid = omdl.getCentroid();
            if (centroid == null) {
                continue;
            }

            double size = resizer.getSize(data);
            float lat = (float) centroid.getLat();
            float lon = (float) centroid.getLon();
            int offX = 0;
            int offY = 0;
            String key = "";
            if (offsetsOn) {
                key = offseter.buildKey(data);
                offX = offseter.getX(key);
                offY = offseter.getY(key);
            }
            OMGraphic p;

            if (getShowNullAsPoint() && ((int) (2 * size)) < 2) { // null sized disc => OMPoint
                p = new OMPoint((float) centroid.getLat(), (float) centroid.getLon());
                omglDiscs.add(p);
            } else {

//                int offX = 0;
//                int offY = 0;
//                String key = offseter.buildKey(data);
////                if (offseter.isIndividual()) {
////                    key = offseter.buildKey(data);
////                    if (!offseter.contains(key)) {
////                        offseter.insert(key, lat, lon);
////                    }
//                offX = offseter.getX(key);
//                offY = offseter.getY(key);
////                }
                if (nbEdges < 21)//draw poly
                {
                    p = new OMRegularPolygon(this, key, lat, lon,
                            nbEdges, (float) size,
                            offX, offY, startAngleInRadians,
                            starCoef, resizer.getString(data), offVis, offLineVis);
                } else {//draw circle

                    p = new OMDisc(this, key, lat, lon,
                            offX, offY,
                            (int) Math.round(size * 2),
                            resizer.getString(data), offVis, offLineVis);
                }
                omglDiscs.add(p);
            }

            p.setLinePaint(drawingAttributes.getLinePaint());
            Object colorValue = data.getData(colorDataColumnName);
            p.setFillPaint(colorizer.getColor(colorValue));
            p.setStroke(drawingAttributes.getStroke());
        }
        omglDiscs.sort();
        initMinMaxValues(omglDiscs); // used for the legend
//        omgl.add(omglDiscs);
//        omgl.add(ompl);

        return omglDiscs;
    }

    public void classificationChanged(Object source) {
        if (source != colorizer) {
            colorizer.setData(getLayer().getDataStorageIterator(),
                    colorDataColumnName);
        }
        refresh();
    }

    public void setColorDataColumnName(String name) {
        colorDataColumnName = name;
        refresh();
    }

    public boolean isReady() {
        return super.isReady() && !colorDataColumnName.equals("");
    }

    public String getMissingInfos() {
        String mess = super.getMissingInfos();
        if (colorDataColumnName.equals("")) {
            mess += i18n.get(this, "colorDataColumnName",
                    "Color data column name") + "\n";
        }
        return mess;
    }

    public OMResizeableGraphic getLegend(Refreshable r,
            MouseControlPointsListener ml) {
        if (minMaxValues == null) {
            return null;
        }
        OMResizeableFrame frame = new OMResizeableColoredProportionalSymbolLegend(r, ml, 0, 0,
                resizer, minMaxValues, nbEdges, getStartAngleInRadians(),
                starCoef, getDataColumnName(), drawingAttributes.getLinePaint(), new Color(0, 0, 0, 0),
                colorizer.getColorScale(), colorizer.getColorsTexts(), colorDataColumnName, getFullName());
        frame.setVisuFrame(true);
        return frame;
    }
}
