// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import fr.unicaen.openmap.event.RefreshableGUI;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;

import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;

import com.bbn.openmap.omGraphics.*;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;

/**
 * Disc visualization mode.
 */
public class Disc extends OffsetableRaw implements ResizerListener {

    public static final String zoomableSymbolPropertyString = "zoomableSymbol";
    public static final String showNullAsPointPropertyString = "showNullAsPoint";

    protected SymbolResizer resizer = new SymbolResizer(new DataSquareRootColumnValueComputer("data"));
    protected JCheckBox showNullAsPointCB;
    public String getDataColumnName() { return resizer.getDataColumnName(); }
    public void setDataColumnName(String name) { resizer.setDataColumnName(name); }

    //protected Offseter offset = new Offseter();
    private MinMaxValues minMaxValues;
    /// zoomableSymbol is now always false
    protected boolean zoomableSymbol = false;
    public boolean getZoomableSymbol() { return zoomableSymbol; }
    protected void setZoomableSymbol(boolean b) { 
	if (b != zoomableSymbol) {
	    zoomableSymbol = b; 
	    // fireChangeZoomable(); could change maxSize 40 -> 1  or 1 -> 40 !!!
	}
    }
    
    protected boolean showNullAsPoint;// = true;
    protected boolean getShowNullAsPoint() { return showNullAsPoint; }
    protected void setShowNullAsPoint(boolean b) {
	if (showNullAsPoint != b) {
	    showNullAsPoint = b;
	    refresh();
	}
    }

    public Disc(ThematicDataDisplayLayer layer, String name) {
	this(layer, name, i18n.get(Disc.class, "disc", "Disc"));
    }
    public Disc(ThematicDataDisplayLayer layer, String name, String typeName) { 
	super(layer, name, typeName); 
	resizer.addResizerListener(this);
    }

    public void setProperties(String prefix, Properties properties) {
	super.setProperties(prefix, properties);

	setZoomableSymbol(Util.isTrue(properties.getProperty(prefix + zoomableSymbolPropertyString, "false")));
	setShowNullAsPoint(Util.isTrue(properties.getProperty(prefix + showNullAsPointPropertyString, "false")));
	resizer.setProperties(prefix, properties);
    }
    
    public Properties getProperties(String prefix, Properties properties) {
	properties = super.getProperties(prefix, properties);
	
	properties.put(prefix + zoomableSymbolPropertyString, new Boolean(getZoomableSymbol()).toString());
	properties.put(prefix + showNullAsPointPropertyString, new Boolean(getShowNullAsPoint()).toString());
	properties = resizer.getProperties(prefix, properties);

	return properties;
    }
    
    public OMGraphicList generateGraphics(DataStorageIterator dsi) {
 
	resizer.setMinMaxValue(dsi);
	OMSizeableGraphicList omgl = new OMSizeableGraphicList<OMDisc>();
	OMGraphicList ompl = new OMGraphicList(); // for the OMPoint (Discs of null sizes)

	while (dsi.hasNext()) {
	    DataObject data = dsi.next();
	    OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
	    if (omdl == null)
		continue;

	    Centroid centroid = omdl.getCentroid();
	    if (centroid == null)
		continue;
	    float radius = (float)resizer.getSize(data);

	    OMGraphic disc;
	    // 0.001 is to be adapted with LatLon Graphics
	    if (((int)(2*radius)) < 2 && getShowNullAsPoint()) { // null sized disc => OMPoint
		disc = new OMPoint((float)centroid.getLat(), (float)centroid.getLon());
		ompl.add(disc);
	    } else {
//                float lat = (float) centroid.getLat();
//                float lon = (float) centroid.getLon();
                int offX = 0;
                int offY = 0;
//                if (offseter.isIndividual()) {
//                    key = offseter.buildKey(data);
//                    if (!offseter.contains(key)) {
//                        offseter.insert(key, lat, lon);
//                    }
//                }
                String key = offseter.buildKey(data);
                offX = offseter.getX(key);
                offY = offseter.getY(key);
		disc = getZoomableSymbol() 
		    ? new OMDisc(this, key, (float)centroid.getLat(), (float)centroid.getLon(),
                    offX, offY,
                    radius, resizer.getString(data), offseter.isVisible(), offseter.isLineVisible())
		    : new OMDisc(this, key, (float)centroid.getLat(), (float)centroid.getLon(),
                    offX, offY,
		    (int)(2*radius), resizer.getString(data),offseter.isVisible(), offseter.isLineVisible());
		omgl.add(disc);
	    }
	    disc.setLinePaint(drawingAttributes.getLinePaint());
	    disc.setFillPaint(drawingAttributes.getFillPaint());
	    disc.setStroke(drawingAttributes.getStroke());
	}

	omgl.sort();    // in order to put small discs on top (and to easily compute min, median and max values)  
	initMinMaxValues(omgl);  // used for the legend
	if (ompl.size() > 0) // add null sized discs
	    omgl.add(ompl);
	return omgl;
    }

    public JComponent getGUI(final DataStorageIterator dsi) {
	thematicGUI  = (ThematicGUI)super.getGUI(dsi);
	
	/// JCheckBox zoomableSymbolCB = new JCheckBox("Zoomable Symbol");
	/// zoomableSymbolCB.addItemListener(new ItemListener() {
	/// 	public void itemStateChanged(ItemEvent e) {
	/// 	    setZoomableSymbol(((AbstractButton)(e.getItemSelectable())).isSelected());
	/// 	}
	///     });
	/// panel.add(zoomableSymbolCB);
	showNullAsPointCB = new JCheckBox(i18n.get(this, "showNullAsPoint", "Show Null As Point"));
	showNullAsPointCB.setSelected(getShowNullAsPoint());
	showNullAsPointCB.addItemListener(new ItemListener() {
		public void itemStateChanged(ItemEvent e) {
		    setShowNullAsPoint(((AbstractButton)(e.getItemSelectable())).isSelected());
		}
	    });
        showNullAsPointCB.setBorder(BorderFactory.createEmptyBorder());
	//panel.add(showNullAsPointCB);
	//JPanel discPanel = new JPanel();
	//discPanel.setBorder(BorderFactory.createTitledBorder(i18n.get(this, "discTitle", "Disc")));
       /* JComponent cp = resizer.getGUI(dsi);
        cp.add(showNullAsPointCB);
	discPanel.add(cp);
	thematicGUI.add(discPanel);*/
        ThematicConfPanel cpanel = (ThematicConfPanel)resizer.getGUI(dsi);
        //cpanel.getContent().add(showNullAsPointCB);
        JPanel p = new JPanel(new BorderLayout());
        p.add(cpanel.getContent(), BorderLayout.CENTER);
        //p.add(showNullAsPointCB, BorderLayout.SOUTH);
        cpanel.setContent(p);
        thematicGUI.addThematicConfigurationPanel(cpanel);
        JPanel wrap = new JPanel();
        wrap.add(cpanel);
        thematicGUI.add(wrap);
	return thematicGUI;
    }
    
    protected void initMinMaxValues(OMGraphicList omgl) {
	if (omgl == null || omgl.size() == 0) {
	    minMaxValues = new MinMaxValues(0, 0);
	    return;
	}

	double min = 0, max = 0;
	OMGraphic omg = omgl.getOMGraphicAt(0);
	try {
	    min = Util.parseDouble(((HaveWidthAndData) omgl.getOMGraphicAt(0)).getData());
	} catch (java.lang.ClassCastException e) {
	}
	try {
	    max = Util.parseDouble(((HaveWidthAndData) omgl.getOMGraphicAt(omgl.size() - 1)).getData());
	} catch (java.lang.ClassCastException e) {
	}
	minMaxValues = new MinMaxValues(min, max);
    }

    public OMResizeableGraphic getLegend(Refreshable r, MouseControlPointsListener ml) {
	if (resizer.getIsFixedSize()){ // may be one disc with no numerical value would be better ?
	    return null;
        }
	if (minMaxValues == null){
	    return null;
        }
	return new OMResizeableDiscLegend(r, ml, 0, 0, 
					  resizer, minMaxValues, getDataColumnName(), 
					  drawingAttributes.getLinePaint(), drawingAttributes.getFillPaint(), getFullName());
    }

    public void refreshGUI(DataStorageIterator dsi) {
	super.refreshGUI(dsi);
	((RefreshableGUI) resizer).refreshGUI(dsi);
    }

    public void resizerChanged(Resizer resizer) {
	if (this.resizer == resizer)
	    refresh();
    }


}

