// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import java.util.*;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import fr.unicaen.openmap.dataStorage.*;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.*;

/**
 * Multi regular polygons visualization mode.
 */
public class MultiRegularPolygons extends RegularPolygon {

    public static final String elemRadiusPropertyString = "elemRadius";
    public static final String gapPercentagePropertyString = "gapPercentage";
    protected int elemRadius = 5;
    protected int gapPercentage = -10;

    public MultiRegularPolygons(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(MultiRegularPolygons.class, "multiRegPol", "Multi Reg. Polygons"));
    }

    public MultiRegularPolygons(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
        resizer = new SymbolResizer(new DataColumnValueComputer("data"));
        //offseter.addOffseterListener(this);
        interactive = true;
    }

    // JComboBox dataList;
    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);

        setElemRadius(PropUtils.intFromProperties(properties, prefix + elemRadiusPropertyString, getElemRadius()));
        setGapPercentage(PropUtils.intFromProperties(properties, prefix + gapPercentagePropertyString, getGapPercentage()));

    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);

        properties.put(prefix + elemRadiusPropertyString, Integer.toString(getElemRadius()));
        properties.put(prefix + gapPercentagePropertyString, Integer.toString(getGapPercentage()));
        return properties;
    }

    public OMGraphicList generateGraphics(DataStorageIterator dsi) {
        resizer.setMinMaxValue(dsi);
        mouseControlPointsListener.clearControlPoints();
        OMGraphicList omgl = new OMGraphicList();
        float startAngleInRadians = (float) (startAngle * Math.PI / 180.0);
        boolean offVis = offsetsOn ? offseter.isVisible() : false;
        boolean offLineVis = offsetsOn ? offseter.isLineVisible() : false;
        while (dsi.hasNext()) {
            DataObject data = dsi.next();
            OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
            if (omdl == null) {
                continue;
            }

            Centroid centroid = omdl.getCentroid();
            if (centroid == null) {
                continue;
            }

            int nb = (int) resizer.getSize(data);
            float lat = (float) centroid.getLat();
            float lon = (float) centroid.getLon();
            int offX = 0;
            int offY = 0;
            String key = "";
            if (offsetsOn) {
                key = offseter.buildKey(data);
                offX = offseter.getX(key);
                offY = offseter.getY(key);
            }

            OMMultiRegularPolygon p = new OMMultiRegularPolygon(this, key, nb, lat, lon,
                    nbEdges, (float) elemRadius, offX, offY,
                    startAngleInRadians, starCoef, gapPercentage, offVis, offLineVis);
            /*getMultiRegularPolygons(nb, (float) centroid.getLat(), (float) centroid.getLon(),
            nbEdges, (float) elemRadius, offX, offY,
            startAngleInRadians, starCoef);*/

            //    if (nb == 0)
            //	continue;
            //// We add dataValue as AppObject
            //try {
            //	p.setAppObject(data.getData(getDataColumnName()).toString());
            //	p.putAttribute(THEMATIC_DATAVALUE, data.getData(getDataColumnName()).toString());
            //} catch (NullPointerException ex) { p.setAppObject(""); }
            p.setLinePaint(drawingAttributes.getLinePaint());
            p.setFillPaint(drawingAttributes.getFillPaint());
            p.setStroke(drawingAttributes.getStroke());
            // Debug.message("ThematicDataDisplayLayer", "MultiRegPolygon "+nb+" ["+centroid.getLat()+", "+centroid.getLon()+"]");
            omgl.add(p);
            /*if(offsetsOn)
                p.addOffsetableListener(offseter);*/
            mouseControlPointsListener.addOMControlPoints(p.controlPoints);
        }
        // omgl.sort();
        return omgl;
    }

//    protected OMGraphicList getMultiRegularPolygons(int nb, float latPoint, float lonPoint,
//            int nbEdges, float radius, int xOffset, int yOffset,
//            float startAngle, int starCoef) {
//        OMGraphicList omgl = new OMGraphicList();
//
//        int gap = (int) (radius * gapPercentage / 100.0);
//        int width = (int) (2 * radius + gap); // place needed for one poly
//        // nbSqrt is the edge of the biggest possible full square
//        int nbSqrt = (int) Math.sqrt(nb);
//
//        // try to center the group of polygones
//        xOffset -= width * nbSqrt / 2;
//        yOffset -= width * nbSqrt / 2;
//
//        // draw a square of regular polygones
//        for (int x = 0; x < nbSqrt; x++) {
//            for (int y = 0; y < nbSqrt; y++) {
//                omgl.add(new OMRegularPolygon(key, latPoint, lonPoint, nbEdges, radius,
//                        xOffset + x * width,
//                        yOffset + y * width,
//                        startAngle, starCoef, offseter.isVisible()));
//            }
//        }
//        nb -= nbSqrt * nbSqrt;
//        // we still have nb polys to draw
//        OMRegularPolygon omr;
//        // we start by the rigth most vertical line
//        for (int y = 0; nb > 0 && y < nbSqrt; nb--, y++) {
//            omgl.add(omr = new OMRegularPolygon(latPoint, lonPoint, nbEdges, radius,
//                    xOffset + nbSqrt * width,
//                    yOffset + y * width,
//                    startAngle, starCoef));
//               omr.addOffsetableListener(offseter);
//               mouseControlPointsListener.addOMControlPoints(omr.controlPoints);
//        }
//        // we end by the bottom horizontal line
//        for (int x = 0; nb > 0 && x < nbSqrt; nb--, x++) {
//            omgl.add(omr = new OMRegularPolygon(latPoint, lonPoint, nbEdges, radius,
//                    xOffset + x * width,
//                    yOffset + nbSqrt * width,
//                    startAngle, starCoef));
//           omr.addOffsetableListener(offseter);
//           mouseControlPointsListener.addOMControlPoints(omr.controlPoints);
//        }
//        return omgl;
//    }
    public JComponent getGUI(final DataStorageIterator dsi) {

        thematicGUI = (ThematicGUI) super.getGUI(dsi);
        thematicGUI.remove(regpolysPanel);
        //regpolysPanel = new JPanel();
        //regpolysPanel.setLayout(new BoxLayout(regpolysPanel, BoxLayout.X_AXIS));

        //regpolysPanel.setBorder(BorderFactory.createTitledBorder(i18n.get(this,"multiRegularPolygonsTitle","Multi Reg. Polygons")));


        //JPanel dataPanel = new JPanel();
        //dataPanel.setLayout(new BoxLayout(dataPanel, BoxLayout.X_AXIS));
        //dataPanel.add(new JLabel(i18n.get(this,"dataColumnNames","Data Column Name: ")));
        //initDataList(dsi);
        //dataPanel.add(dataList);
        //regpolysPanel.add(dataPanel);
        ThematicConfPanel resizerGUI = (ThematicConfPanel) resizer.getGUI(dsi);

        /*ThematicConfPanel sizeConfPan =
        ThematicConfPanel.createThematicConfPanel(i18n.get(Histogram.class, "size", "Size"));
        sizeConfPan.setContent(resizerGUI);*/
        thematicGUI.addThematicConfigurationPanel(resizerGUI);
        JPanel wrapSize = new JPanel();
        wrapSize.add(resizerGUI);
        thematicGUI.add(wrapSize);

//	final JButton bd = new JButton(i18n.get(this, "draw",
//	"Draw"), Util.loadIcon("visu/triangle.png"));
//	bd.setBackground(Color.LIGHT_GRAY);
//	bd.setBorder(null);
//	bd.setOpaque(false);
//	JPanel p = new JPanel(new BorderLayout());
//	p.add(bd, BorderLayout.WEST);
        //regpolysPanel.add(p);

        final RegularPolygonPanel regPan = getRegularPolygonGUI();
        JPanel wrapRegPan = new JPanel();
        wrapRegPan.add(regPan);
        ThematicConfPanel regConfPan =
                ThematicConfPanel.createThematicConfPanel(i18n.get(this, "draw", "Draw"));
        regConfPan.setContent(wrapRegPan);
        thematicGUI.addThematicConfigurationPanel(regConfPan);
        JPanel wrapReg = new JPanel();
        wrapReg.add(regConfPan);
        thematicGUI.add(wrapReg);


//        //regpolysPanel.add(regPan);
//        JXCollapsiblePane cp = new JXCollapsiblePane(JXCollapsiblePane.Direction.LEFT, new BorderLayout());
//        cp.add(regPan);
//        regpolysPanel.add(new ThematicConfigurationPanel(i18n.get(this, "draw", "Draw"),
//                cp));
        //panel.add(regpolysPanel);
        //regPan.setVisible(false);
//	bd.addActionListener(new ActionListener() {
//		public void actionPerformed(ActionEvent e) {
//			if (!regPan.isVisible()) {
//				bd.setIcon(Util.loadIcon("visu/triangle_down.png"));
//				regPan.setVisible(true);
//			} else {
//				bd.setIcon(Util.loadIcon("visu/triangle.png"));
//				regPan.setVisible(false);
//			}
//		}
//
//	});
        //RegularPolygonPanel regpolyPanel = getRegularPolygonGUI();

        Box sbox;

        sbox = Box.createHorizontalBox();
//        final JSlider elemDiameterField =
//                new JSlider(JSlider.HORIZONTAL, 0, 150, getElemRadius() * 2);
        SpinnerModel model =
                new SpinnerNumberModel(getElemRadius() * 2, //initial value
                0, //min
                150, //max
                5);//step
        final JSpinner elemDiameterField = new JSpinner(model);
//        elemDiameterField.setMajorTickSpacing(50);
//        elemDiameterField.setMinorTickSpacing(10);
//        //	elemDiameterField.setSnapToTicks(true);
//        elemDiameterField.setPaintTicks(true);
//        elemDiameterField.setPaintLabels(true);
        elemDiameterField.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                setElemRadius(((Integer) elemDiameterField.getValue()).intValue());
                getRegularPolygonGUI().repaintSamplePanel();
            }
        });
        //sbox.add();
        //JPanel wrapElemDiameter = new JPanel();
        //wrapElemDiameter.add(elemDiameterField);
        sbox.add(elemDiameterField, "field");
        regPan.addToSliderBox(new JLabel(i18n.get(this, "diameterLabel", "Diameter")), sbox);

        sbox = Box.createHorizontalBox();
//        final JSlider gapPercentageField =
//                new JSlider(JSlider.HORIZONTAL, -100, 100, getGapPercentage());
        model =
                new SpinnerNumberModel(getGapPercentage(), //initial value
                -100, //min
                100, //max
                5);//step
        final JSpinner gapPercentageField = new JSpinner(model);
//        gapPercentageField.setMajorTickSpacing(50);
//        gapPercentageField.setMinorTickSpacing(25);
//        //	gapPercentageField.setSnapToTicks(true);
//        gapPercentageField.setPaintTicks(true);
//        gapPercentageField.setPaintLabels(true);
        gapPercentageField.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                setGapPercentage(((Integer) gapPercentageField.getValue()).intValue());
                //getRegularPolygonGUI().setGap(gapPercentage);
                getRegularPolygonGUI().repaintSamplePanel();
            }
        });
        //sbox.add(new JLabel(i18n.get(this, "gapLabel", "Gap")), "label");
        //JPanel wrapGap = new JPanel();
        //wrapGap.add(gapPercentageField);
        sbox.add(gapPercentageField, "field");
        regPan.addToSliderBox(new JLabel(i18n.get(this, "gapLabel", "Gap")), sbox);


        //regpolysPanel.add(regPan);
        //panel.add(regpolysPanel);

        return thematicGUI;
    }

    public void setElemRadius(int n) {
        elemRadius = n;
        refresh();
    }

    public int getElemRadius() {
        return elemRadius;
    }

    public int getGapPercentage() {
        return gapPercentage;
    }

    public void setGapPercentage(int g) {
        gapPercentage = g;
        refresh();
    }

    //    private OMResizeableMultiRegularPolygonLegend omrmrpl;
    public OMResizeableGraphic getLegend(Refreshable r, MouseControlPointsListener ml) {
        double valuePerUnit = resizer.getValuePerPixel();

        OMResizeableFrame frame = new OMResizeableMultiRegularPolygonLegend(r, ml, 0, 0,
                nbEdges, elemRadius,
                (float) (startAngle * Math.PI / 180.0), starCoef,
                drawingAttributes.getLinePaint(), drawingAttributes.getFillPaint(),//colors.getLineColor(), colors.getFillColor(),
                getDataColumnName(), valuePerUnit, getFullName());
        frame.setVisuFrame(true);
        return frame;
    }

    public void refreshGUI(DataStorageIterator dsi) {
        super.refreshGUI(dsi);
        //initDataList(dsi);
    }
//
//    /** init or reinit the comboBox used to choose dataColumnName */
//    private void initDataList(final DataStorageIterator dsi) {
//        if (dataList == null) {
//	    dataList = new JComboBox(dsi.getDataColumnNames());
//	} else {
//	    dataList.removeAllItems();
//	    for (String item : dsi.getDataColumnNames())
//		dataList.addItem(item);
//	}
//
//    if(getDataColumnName()!=null && !"".equals(getDataColumnName()))
//    	dataList.setSelectedIndex(
//    			dsi.getIndexForColumnName(getDataColumnName()));
//
//	for (ActionListener a : dataList.getActionListeners())
//	    dataList.removeActionListener(a);
//	dataList.addActionListener(new ActionListener() {
//		public void actionPerformed(ActionEvent e) {
//		    try {
//			int index = ((JComboBox)e.getSource()).getSelectedIndex();
//			setDataColumnName(dsi.getRealColumnName(index));
//		    } catch (ArrayIndexOutOfBoundsException ex) { }
//		}});
//    }
}
