/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.unicaen.openmap.omGraphics;

import com.bbn.openmap.I18n;
import com.bbn.openmap.omGraphics.OMColor;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.gui.ThemaMapColorChooser;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeListener;
import java.util.Properties;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

/**
 *
 * @author jchauvea
 */
public class DrawingAttributes extends com.bbn.openmap.omGraphics.DrawingAttributes {

    private boolean withColorMenu;
    private boolean withCapMenu;
    private boolean withJointMenu;
    private boolean withLinePaintMenu;
    private boolean withFillPaintMenu;
    private boolean withStrokeAndDash = true;
    private boolean filled;
    private final Color transparentColor = new Color(0, 0, 0, 0);

    public DrawingAttributes(boolean bColorMenu, boolean bCapMenu, boolean bJointMenu, boolean bLinePaint, boolean bFillPaint) {
        super();
        fillPaint = new Color(255, 255, 255, 200);
        this.withColorMenu = bColorMenu;
        this.withCapMenu = bCapMenu;
        this.withJointMenu = bJointMenu;
        this.withLinePaintMenu = bLinePaint;
        this.withFillPaintMenu = bFillPaint;
        propertyChangeSupport.addPropertyChangeListener(this);

    }

    public DrawingAttributes(boolean bColorMenu, boolean bCapMenu, boolean bJointMenu, boolean bLinePaint, boolean bFillPaint, boolean strokeAndDash) {
        this(bColorMenu, bCapMenu, bJointMenu, bLinePaint, bFillPaint);
        this.withStrokeAndDash = strokeAndDash;

    }

    public DrawingAttributes() {
        this(true, false, false, true, true);
    }

    public void setWithColorMenu(boolean b) {
        withColorMenu = b;
    }

    @Override
    protected JComponent getColorAndLineGUI() {

        if (palette == null || toolbar == null) {
            palette = new JPanel();
        }

        resetGUI();
        palette.removeAll(); // Remove cruft from past OMGraphics

        lineButton = new JButton(getDrawingAttributesIcon(this, icon_width, icon_height,
                true));

        lineButton.setToolTipText(i18n.get(DrawingAttributes.class,
                "drawingAttributesButton", I18n.TOOLTIP,
                "Modify Drawing Parameters"));

        lineButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent ae) {
                JButton button = getLineButton();
                JPopupMenu popup = new JPopupMenu();
                if (withColorMenu) {
                    setPreStrokeMenuOptions(popup);
                }
                setStrokeMenuOptions(popup);
                setPostStrokeMenuOptions(popup);

                popup.show(button, button.getWidth(), 0);
            }
        });

        palette.add(lineButton);
        lineButton.setBackground(Color.gray);
        lineButton.setOpaque(false);
        ComponentSizer.fixComponentSize(lineButton, new Dimension(20, 20));
        palette.setBackground(Color.WHITE);
        palette.setOpaque(false);
        return palette;
    }

    @Override
    protected void setPostStrokeMenuOptions(JPopupMenu popup) {
        int n = popup.getComponentCount();
        Component capMenu = popup.getComponent(n - 2);
        Component jointMenu = popup.getComponent(n - 1);
        if (!withCapMenu) {
            popup.remove(capMenu);
        }
        if (!withJointMenu) {
            popup.remove(jointMenu);
        }
        if (!withStrokeAndDash) {
            popup.remove(2);
            popup.remove(1);
        }
        popup.updateUI();
    }
    public static String filledPropertyString = "filled";

    @Override
    public void setProperties(String prefix, Properties props) {
        PropertyChangeListener[] pcls = null;
        if (propertyChangeSupport != null) {
            pcls = propertyChangeSupport.getPropertyChangeListeners();
        }
        super.setProperties(prefix, props);
        if (pcls != null) {
            for (PropertyChangeListener pcl : pcls) {
                propertyChangeSupport.addPropertyChangeListener(pcl);
            }
        }
        if (props != null)
            filled = PropUtils.booleanFromProperties(props, prefix + filledPropertyString, filled);

    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(properties);
        properties.put(prefix + filledPropertyString, String.valueOf(filled));
        return properties;
    }

    /**
     * Gets the JMenu that has the color control options.
     *
     * @return
     */
    @Override
    public JMenu getColorMenu() {
        JMenu colorMenu = super.getColorMenu();
        //colorMenu.remove(mattedEnabledItem);
        //colorMenu.remove(mattingColorItem);
        colorMenu.remove(selectColorItem);
        if (!withLinePaintMenu) {
            colorMenu.remove(lineColorItem);
        }
        if (!withFillPaintMenu) {
            colorMenu.remove(fillColorItem);
        }
        colorMenu.remove(mattedEnabledItem);
        colorMenu.remove(mattingColorItem);
        return colorMenu;
    }

    @Override
    public Component getGUI() {
        super.getGUI();
        return lineButton;
    }

    @Override
    protected Color getNewPaint(Component source, String title, Color startingColor) {
        Color newPaint = null;
        if (getLock()) {
            newPaint = ThemaMapColorChooser.showDialog(source, title, startingColor);
            releaseLock();
        }
        return newPaint;
    }

    @Override
    public Paint getFillPaint() {
        if (isFilled())
            return super.getFillPaint();
        else
            return transparentColor;
    }

    /**
     * @return the filled
     */
    public boolean isFilled() {
        return filled;
    }

    /**
     * @param filled the filled to set
     */
    public void setFilled(boolean filled) {
        this.filled = filled;
    }
}
