// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

import com.bbn.openmap.I18n;
import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.Layer;
import com.bbn.openmap.omGraphics.FontSizer;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfigurationPanel;
import java.awt.GridLayout;
import javax.swing.JComponent;
import org.jdesktop.swingx.JXCollapsiblePane;

/**
 * Panel to manage a FontSizer.
 * <ul>
 * <li>a FontDialog to select font</li>
 * <li>a button to show the FontDialog and the selected font</li>
 * <li>four spinners to configure a FontSizerShadow</li>
 * </ul>
 * Sample in fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Label.
 * @see FontSizerShadow
 * @see fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Label
 */
public class FontSizerPanel extends JPanel {

    /**
     * Property name for total of ScaleAndSize to keep in properties.
     */
    public final static String TOTAL_PROPERTY_NAME = "fsp.total";
    /**
     * Property name for scale values to keep in properties.
     */
    public final static String SCALE_PROPERTY_NAME = "fsp.scale";
    /**
     * Property name for font size values to keep in properties.
     */
    public final static String SIZE_PROPERTY_NAME = "fsp.size";
    /**
     * Property name used to give a panel title (internationalization).
     */
    public final static String TITLE_PROPERTY_NAME = "title";
    /**
     * Internationalization of GUI.
     */
    public final static String ADD_PROPERTY_NAME = "add";
    /**
     * Internationalization of GUI.
     */
    public final static String DELETE_PROPERTY_NAME = "delete";
    /**
     * Use I18N mecanism to allow internationalization.
     */
    public static I18n i18n = new BasicI18n();
    /**
     * FontSizerShadow associated with this panel.
     */
    protected FontSizerShadow fontsizer;
    /**
     * Use to select a Font.
     */
    protected FontDialog fontDialog;
    /**
     * Able to show the FontDialog.
     */
    protected JButton fontButton;
    /**
     * Contains scale and size settings.
     */
    private Vector<ScaleAndSize> scales = new Vector<ScaleAndSize>();
    Box fontPanel;
    /**
     * contains font and add buttons
     */
    private JPanel panButton;

    /**
     * Basis constructor.
     * @param fontsizer FontSizerShadow used for this FontSizerPanel
     */
    public FontSizerPanel(FontSizerShadow fontsizer) {
        super();
        fontPanel = Box.createVerticalBox();
        this.fontsizer = fontsizer;

        //triangle button
//	final JButton bff = new JButton(i18n.get(FontSizerPanel.class, "title", "Fonts"),Util.loadIcon("visu/triangle.png"));
//	bff.setBackground(Color.LIGHT_GRAY);
//	bff.setBorder(null);
//	bff.setOpaque(false);
//	JPanel panbff = new JPanel(new BorderLayout());
//	panbff.add(bff,BorderLayout.WEST);

        //font dialog
        fontDialog = new FontDialog(this.fontsizer.getFont(), false);

        fontButton = new JButton(i18n.get(FontSizerPanel.class,
                FontSizerShadow.FONT_PROPERTY_NAME,
                "Select font"));
        fontButton.setToolTipText(i18n.get(FontSizerPanel.class,
                FontSizerShadow.FONT_PROPERTY_NAME,
                I18n.TOOLTIP,
                "Select and configure a font. Current : {0}",
                fontsizer.getFont().getName()));
        //fontButton.setPreferredSize(new Dimension(50, 25));
        fontButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent ae) {
                fontDialog.setVisible(true);

                if (!fontDialog.isCanceled()) {
                    getFontSizer().setFont(fontDialog.getSelectedFont());
                    //we want to keep current scale/size values
                    //			getFontSizer().setMinPointSize(fontDialog.getSelectedFont().getSize());
                    //			getFontSizer().setMaxPointSize(fontDialog.getSelectedFont().getSize());
                    refresh();
                    //			fireFontSizerChanged();
                }
            }
        });

        JButton addButton = new JButton(i18n.get(FontSizerPanel.class, ADD_PROPERTY_NAME, "Add scale/size"));
        addButton.setToolTipText(i18n.get(FontSizerPanel.class, ADD_PROPERTY_NAME, I18n.TOOLTIP, "Add a new scale/size."));
        addButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent event) {
                addScale(new ScaleAndSize());
                createOptionPanel();
                updateUI();
                fontPanel.setVisible(false);
                fontPanel.setVisible(true);
            }
        });

        panButton = new JPanel();
        panButton.add(fontButton);
        panButton.add(addButton);

        fontPanel.add(panButton);
        //visual aspect
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        fontPanel.setBorder(BorderFactory.createTitledBorder(""));


        //default
        //add(panbff);
        addScale(new ScaleAndSize(fontsizer.getBaseScale(), fontsizer.getFont().getSize()));
        createOptionPanel();
        add(fontPanel);
        //add(fontPanel);
        /*JXCollapsiblePane cp = new JXCollapsiblePane(JXCollapsiblePane.Direction.LEFT, new BorderLayout());
        cp.add(fontPanel);
        add(new ThematicConfigurationPanel(i18n.get(FontSizerPanel.class, "title", "Fonts"),
        cp));*/
        //fontPanel.setVisible(false);

//	bff.addActionListener(new ActionListener(){
//		public void actionPerformed(ActionEvent e) {
//			if(!fontPanel.isVisible()){
//				bff.setIcon(Util.loadIcon("visu/triangle_down.png"));
//				fontPanel.setVisible(true);
//			}
//			else{
//				bff.setIcon(Util.loadIcon("visu/triangle.png"));
//				fontPanel.setVisible(false);				
//			}
//		}
//		
//	});

    }

    public JComponent getFontPanel() {
        return fontPanel;
    }

    /**
     * Refresh with values from FontSizerShadow.
     */
    public void refresh() {
        fontDialog.selectFont(getFontSizer().getFont());

        //bonus :)
        fontButton.setFont(getFontSizer().getFont());
        fontButton.setText(getFontSizer().getFont().getName());
        fontButton.setToolTipText(i18n.get(FontSizerPanel.class,
                FontSizerShadow.FONT_PROPERTY_NAME,
                I18n.TOOLTIP,
                "Select and configure a font. Current : {0}",
                fontsizer.getFont().getName()));
        fireFontSizerChanged();
    }

    /**
     * Used to set FontSizerShadow with properties resources.
     * @param prefix string prefix used in the properties
     * @param properties used to retrieve properties
     */
    public void setProperties(String prefix, Properties properties) {
        //default for fontsizer
        fontsizer.setProperties(prefix, properties);

        //advanced settings
        int total = PropUtils.intFromProperties(properties, prefix + TOTAL_PROPERTY_NAME, 1);
        if (total > 0) {
            scales.clear();
        }
        for (int i = 0; i < total; i++) {
            addScale(new ScaleAndSize(PropUtils.floatFromProperties(properties, prefix + SCALE_PROPERTY_NAME + String.valueOf(i), 200000000f),
                    PropUtils.intFromProperties(properties, prefix + SIZE_PROPERTY_NAME + String.valueOf(i), fontsizer.getFont().getSize())));
        }

        createOptionPanel();
        refresh();
    }

    /**
     * Used to keep properties from FontSizerShadow.
     * @param prefix string prefix used in the properties
     * @param properties used to load properties
     * @return Properties.
     */
    public Properties getProperties(String prefix, Properties properties) {
        //default for fontsizer
        properties = fontsizer.getProperties(prefix, properties);

        //advanced settings
        properties.put(prefix + TOTAL_PROPERTY_NAME, String.valueOf(scales.size()));
        for (int i = 0; i < scales.size(); i++) {
            properties.put(prefix + SCALE_PROPERTY_NAME + String.valueOf(i), String.valueOf(scales.get(i).getScale()));
            properties.put(prefix + SIZE_PROPERTY_NAME + String.valueOf(i), String.valueOf(scales.get(i).getSize()));
        }

        return properties;
    }
    /**
     * Layer provides Projection to determinate which size font to use.
     */
    private Layer layer;

    /**
     * Set layer.
     */
    public void setLayer(Layer layer) {
        this.layer = layer;
    }

    /**
     * Get the FontSizerShadow with an appropriate font size if any.
     * @return current FontSizerShadow
     */
    public FontSizerShadow getFontSizer() {
        /*       	if (layer != null) {
        getAppropriateFontSize();
        }
        return fontsizer;*/
        return getFontSizer(null);
    }

    /**
     * Get the FontSizerShadow with an appropriate font size for the given projection.
     * @param proj Projection
     * @return current FontSizerShadow
     */
    public FontSizerShadow getFontSizer(Projection proj) {
        //	if (proj != null) {
        getAppropriateFontSize(proj);
        //	System.out.println(fontsizer + " " + this.size);
        //	}
        return fontsizer;
    }

    /**
     * Given the size font according current projection.
     */
    protected int getAppropriateFontSize(Projection proj) {
        //    protected int getAppropriateFontSize() {
        //Default value, out of range : no display, usefull
        int size = 0;
        //	if (layer == null  || layer.getProjection() == null) { return size;}
        if (proj == null) {
            //	    System.out.println("Proj null !!");
            //	    return size;
            size = this.size;
        }

        if (proj != null) {
            //	float scale = layer.getProjection().getScale();
            float scale = proj.getScale();
            fontsizer.setBaseScale(scale);// :

            boolean find = false;
            for (Enumeration<ScaleAndSize> e = scales.elements(); e.hasMoreElements() && !find;) {
                ScaleAndSize current = e.nextElement();
                if (scale < current.getScale()) {
                    size = current.getSize();
                    find = true;
                }
            }
        }

        if (this.size != size) {
            fontsizer.setMinPointSize(size);
            fontsizer.setMaxPointSize(size);
            this.size = size;
            modified = true;
            fireFontSizerChanged();
        } else {
            modified = false;
        }

        //	System.out.println(fontsizer + " " + this.size);

        return size;
    }
    //current font size
    private int size = 0;
    private boolean modified = false;

    public boolean isModified(Projection proj) {
        getAppropriateFontSize(proj);
        //	getAppropriateFontSize();
        return modified;
    }

    /**
     * Store ScaleAndSize behind scale, in ascendant order.
     * @param scaleAndSize contains font size to use with max scale 
     */
    protected void addScale(ScaleAndSize scaleAndSize) {
        if (scales.isEmpty()) {
            scales.add(scaleAndSize);
            return;
        }
        boolean search = true;
        for (int i = 0; i < scales.size() && search; i++) {
            if (scaleAndSize.getScale() < scales.get(i).getScale()) {
                scales.insertElementAt(scaleAndSize, i);
                search = false;
            } else if (i == scales.size() - 1) {
                scales.add(scaleAndSize);
                search = false;
            }
        }
    }

    /**
     * Create... options panel. <br/>
     * Button to choose the default font. <br/>
     * Spinners to manage scales and associated sizes <br/>
     */
    private void createOptionPanel() {
        fontPanel.removeAll(); //mouais

        //JPanel buttonsPanel = new JPanel();

        //Adding a new ScaleAndSize
//        JButton addButton = new JButton(i18n.get(FontSizerPanel.class, ADD_PROPERTY_NAME, "Add scale/size"));
//        addButton.setToolTipText(i18n.get(FontSizerPanel.class, ADD_PROPERTY_NAME, I18n.TOOLTIP, "Add a new scale/size."));
////        addButton.addActionListener(new ActionListener() {
////
////            public void actionPerformed(ActionEvent event) {
////                addScale(new ScaleAndSize());
////                createOptionPanel();
////                updateUI();
////                FontSizerPanel.this.setVisible(false);
////                FontSizerPanel.this.setVisible(true);
////                repaint();
////            }
////        });

        // buttonsPanel.add(fontButton);
        // buttonsPanel.add(addButton);
        fontPanel.add(panButton);

        JPanel panTab = new JPanel(new GridLayout(scales.size() + 1, 3));
        panTab.add(new JLabel(i18n.get(FontSizerPanel.class,
                FontSizerShadow.SCALE_PROPERTY_NAME,
                "Until scale : ")));
        panTab.add(new JLabel(i18n.get(FontSizerPanel.class,
                FontSizerShadow.FONT_SIZE_PROPERTY_NAME,
                "Font size : ")));
        panTab.add(new JPanel());
        //Scales & sizes settings
        for (int i = 0; i < scales.size(); i++) {

            final int index = i;
            JPanel pan = new JPanel();
            Dimension dim = new Dimension(60, 20);
            final JSpinner scaleSpinner = new JSpinner(new SpinnerNumberModel(scales.get(i).getScale(), 1d, 200000000d, 50000d));
            scaleSpinner.setMaximumSize(dim);
            scaleSpinner.setPreferredSize(dim);
            scaleSpinner.addChangeListener(new ChangeListener() {

                public void stateChanged(ChangeEvent ce) {
                    scales.get(index).setScale(((Double) scaleSpinner.getValue()).floatValue());
                    ScaleAndSize scale = scales.get(index);
                    scales.removeElementAt(index);
                    addScale(scale);
                    fireFontSizerChanged();
                    createOptionPanel();
                    FontSizerPanel.this.setVisible(false);
                    FontSizerPanel.this.setVisible(true);
                    repaint();
                }
            });
            scaleSpinner.setToolTipText(i18n.get(FontSizerPanel.class,
                    FontSizerShadow.SCALE_PROPERTY_NAME,
                    I18n.TOOLTIP,
                    "The scale used."));

            final JSpinner sizeSpinner = new JSpinner(new SpinnerNumberModel(scales.get(i).getSize(), 0, 100, 1)); //notes : allow size = 0 => No display :p
            sizeSpinner.setMaximumSize(dim);
            sizeSpinner.setPreferredSize(dim);
            sizeSpinner.addChangeListener(new ChangeListener() {

                public void stateChanged(ChangeEvent ce) {
                    scales.get(index).setSize(((Integer) sizeSpinner.getValue()).intValue());
                    fireFontSizerChanged();
                }
            });
            sizeSpinner.setToolTipText(i18n.get(FontSizerPanel.class,
                    FontSizerShadow.FONT_SIZE_PROPERTY_NAME,
                    I18n.TOOLTIP,
                    "The font size used."));

            //Remove this ScaleAndSize
//            JButton killer = new JButton(i18n.get(FontSizerPanel.class,
//                    DELETE_PROPERTY_NAME,
//                    "Delete"));
            JButton killer = new JButton(Util.loadIcon("cancel2.png"));
            killer.setBorder(null);
            killer.setBorderPainted(false);
            killer.setFocusPainted(false);
            killer.setBackground(Color.WHITE);
            killer.setOpaque(false);
            killer.setToolTipText(i18n.get(FontSizerPanel.class,
                    DELETE_PROPERTY_NAME,
                    I18n.TOOLTIP,
                    "Delete scale/size."));
            killer.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent event) {
                    scales.removeElementAt(index);
                    fireFontSizerChanged();
                    createOptionPanel();
                    fontPanel.setVisible(false);
                    fontPanel.setVisible(true);
                }
            });

//            pan.add(new JLabel(i18n.get(FontSizerPanel.class,
//                    FontSizerShadow.SCALE_PROPERTY_NAME,
//                    "Until scale : ")));
            panTab.add(scaleSpinner);
//            pan.add(new JLabel(i18n.get(FontSizerPanel.class,
//                    FontSizerShadow.FONT_SIZE_PROPERTY_NAME,
//                    "Font size : ")));
            panTab.add(sizeSpinner);
            panTab.add(killer);
        }
        JPanel panTabWrap = new JPanel();
        panTabWrap.add(panTab);
        fontPanel.add(panTabWrap);
    }
    private ArrayList<FontSizerListener> listenerList = new ArrayList<FontSizerListener>();

    public void addFontSizerListener(FontSizerListener l) {
        listenerList.add(l);
    }

    public void removeFooListener(FontSizerListener l) {
        listenerList.remove(l);
    }

    protected void fireFontSizerChanged() {
        for (int i = 0; i < listenerList.size(); i++) {
            listenerList.get(i).fontsizerChanged(getFontSizer());
        }
    }

    /**
     * To keep size font to use until associated scale. Just provide accessors.
     */
    public class ScaleAndSize {

        private float scale;
        private int size;

        /**
         * Use default value : <br/>
         * scale = 1 or current scale if a Projection is available <br/>
         * font size = default font size used by Fontsizer <br/>
         */
        public ScaleAndSize() {
            //	    this(1f, getFontSizer().getFont().getSize());
            this(1f, fontsizer.getFont().getSize());

            /*	    if (layer != null && layer.getProjection() != null)
            setScale(layer.getProjection().getScale());*/
        }

        public ScaleAndSize(float scale, int size) {
            this.scale = scale;
            this.size = size;
        }

        public float getScale() {
            return scale;
        }

        public int getSize() {
            return size;
        }

        public void setScale(float scale) {
            if (scale < 1) {
                return;
            }

            this.scale = scale;
        }

        public void setSize(int size) {
            if (size < 0) {
                return;
            }

            this.size = size;
        }

        public String toString() {
            return String.valueOf(scale) + " -- " + String.valueOf(size);
        }
    }//ScaleAndSize
}
