// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import java.util.ArrayList;
import java.util.Collections;

/**
 * 
 */
public class CustomizedClassification extends ClassificationOfDouble {

    public CustomizedClassification(ArrayList<Double> data, int nbClasses) {
        super(data, nbClasses);

    }

    /*
    public CustomizedClassification(Classification c, int nbClasses) {
    super(c.data, nbClasses);
    }
     */
    public CustomizedClassification(ClassificationOfDouble c) {
        super(c.data, c.getNumberOfClasses());
        setBreaks(c.getBreaks());
    }

    public void reInit(ClassificationOfDouble c) {
        reInit(c.data, c.nbClasses);
        setBreaks(c.getBreaks());
    }

    protected void init() {
        //TO DO : pas trop lobotomisé : tri des valeurs -- conservation des bornes sur changement de nombre de classes...
        // System.out.println("==> CustomizedClassification.init() "+ this);
        if (breaks == null || breaks.length != getNumberOfClasses() + 1) {
            breaks = new double[getNumberOfClasses() + 1];

            for (double d : breaks) {
                d = 0;
            }
        }

        if (data != null) {
            Collections.sort(data);
            double maxDataValue = data.get(data.size() - 1);
            double minDataValue = data.get(0);
            breaks[0] = minDataValue;
            breaks[breaks.length - 1] = maxDataValue;
            initCountsFromBreaks();
            // System.out.println("<== CustomizedClassification.init() "+ this);
        }
    }

    public void setBreaks(double[] breaks) {
//        try {
//            throw new Exception();
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
        if (breaks.length != getNumberOfClasses() + 1) {
            return;
        }

        this.breaks = breaks;
        // shall we sort the breaks?

        init();
    }

    public void setNthBreak(int n, double v) {
        if (breaks == null || n >= breaks.length) {
            init();
        }
        // we try init only once!
        if (breaks == null || n >= breaks.length) {
            return;
        }
        breaks[n] = v;
        init();
    }

    public void loadParametersFrom(Classification classif) {
        reInit(classif);
        if (classif instanceof ClassificationOfDouble) {
            setBreaks(((ClassificationOfDouble) classif).getBreaks());
        }
    }

    @Override
    public void reInit(Classification c, int nbClasses) {
        if (c instanceof ClassificationOfDouble) {
            super.reInit(c, nbClasses);
            setBreaks(((ClassificationOfDouble) c).getBreaks());
        }
        else
            super.reInit(c, nbClasses);
    }
}
