// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.omgraphics;

import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.proj.Projection;

import fr.unicaen.openmap.util.Colors;
import fr.unicaen.openmap.util.find.Distance;


public class OMLegendPiece extends OMGraphicList
{
    /**
     * dessine deux rectangles (par exemple à insérer dans une légende)
     * - un pour couleurs «positives»
     * - un pour couleurs «négatives»
     */
    public OMLegendPiece(double distance, Projection proj, Colors colors, int x, int y)
    {
	LatLonPoint mapCenter = proj.getCenter(); 
	LatLonPoint pt = new LatLonPoint.Double(mapCenter.getLatitude(), mapCenter.getLongitude() + (float)distance); 
	int maxH = Distance.computeDistanceInPixel(mapCenter, pt, proj);
	int width = maxH/3; //parameter ?
	
	//function du genre get??(maxH, width), utilisable pour flowlayer
	int[] pts1 = {
	    x + maxH/2, y + 2*maxH + 10,
	    x + 3*maxH/2, y + 2*maxH + 10,
	    x + 3*maxH/2, y + 2*maxH + width + 10,
	    x + maxH/2, y + 2*maxH + width + 10};
	int[] pts2 = {
	    x + maxH/2, y + 2*maxH + width + 20,
	    x + 3*maxH/2, y + 2*maxH + width + 20,
	    x + 3*maxH/2, y + 2*maxH + 2*width + 20,
	    x + maxH/2, y + 2*maxH + 2*width + 20}; 
	
	OMPoly plus = new OMPoly(pts1);
	plus.setFillPaint(colors.getFillColor());
	plus.setLinePaint(colors.getLineColor()); 
	OMText text1 = new OMText(x + 3*maxH/2 + 3, y + 2*maxH + width + 10, //à ajuster mieux que ça !!
				  "solde positif",
				  OMText.JUSTIFY_LEFT);
	
	OMPoly minus = new OMPoly(pts2);
	OMText text2 = new OMText(x + 3*maxH/2 + 3, y + 2*maxH + 2*width + 20, //à ajuster mieux que ça !!
				  "solde négatif",
				  OMText.JUSTIFY_LEFT);
	
	minus.setFillPaint(colors.getSelectedFillColor());
	minus.setLinePaint(colors.getSelectedLineColor()); 

	add(plus);
	add(text1);
	add(minus);
	add(text2);
    }

}
