// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.sizeable;

import fr.unicaen.openmap.util.Colors;
//import fr.unicaen.openmap.util.classifiable.Classifiable;


//author : Yannick Lange, ylange@etu.info.unicaen.fr, DESS NAPI, juin-septembre 2003.
//upgrated : décembre 2003
abstract public class SizeableElement //implements Classifiable
{

    /**
     * Maximum possible size for this element.
     */
    //    protected double maxSize;

    /**
     * Minimum possible size for this element.
     */
    //    protected double minSize;

    /**
     * Maximum value for this element.
     */
    //    protected double maxValue;

    /**
     * Size for this element. Determinates by maxSize, minSize, value and maxValue
     */
    protected float size; //variable, f(maxSize, maxValue, value)


    /**
     * Returns element's size (width, rayon or anything determinating a size)
     * @return a float representation of size.
     */
    public float getSize()
    {
	return size;
    }

    public void setSize(double _maxSize, double _minSize, double _maxValue, double _value){
	size=calculateSize(_maxSize, _minSize, _maxValue, _value);
    }

    /**
     * Sets the size in the limits given by minSize and maxSize. 
     * If given size is smaller than minSize, SizeableElement's size become minSize. 
     * If given size is bigger than maxSize, SizeableElement's size become maxSize...
     * @param size float representation of new size
     */
    protected void setSize(float size,double _minSize,double _maxSize)
    {
	if (size < _minSize) 
	    {
		this.size = (float) _minSize;
		return;
	    }
	if (size > _maxSize) 
	    {
		this.size = (float) _maxSize;
		return;
	    }

	this.size = size;
    }

    /**
     * Returns max value uses to calculate size of this element.
     * @return a float representation of calculating thickness.
     */
    /*    public double getMaxValue()
    {
	return maxValue;
    }
    */

    /**
     * Sets the max value and adjusts the size behind.
     * @param maxValue the new max (must be positive)
     */
    /*
    public void setMaxValue(double maxValue)
    {
	this.maxValue = maxValue;
	setSize(calculateSize());
    }
    */

    /**
     * Returns maximum size.
     * @return double representation of maximum size.
     */
    /*
    public double getMaxSize()
    {
	return maxSize;
	} */

    /**
     * Sets the possible max size and adjusts the size.
     * @param maxSize max representation of this new max size
     * @return true if sets, false if not.
     */
    /*    public boolean setMaxSize(double maxSize)
    {
	if (maxSize < minSize) 
	    return false;
	
	this.maxSize = maxSize;
	setSize(calculateSize());
	return true;
    }
    */

    /**
     * Returns minimum size.
     * @return double representation of minimum size.
     */
    /*    public double getMinSize()
    {
	return minSize;
    }
    */

    /**
     * Sets the min value and adjusts the size behind.
     * @param minSize the new min (must be positive)
     * @return true if sets, false if not.
     */
    /*
    public boolean setMinSize(double minSize)
    {
	if ((minSize < 0)||(minSize > maxValue)) 
	    return false;

	this.minSize = minSize;
	setSize(calculateSize());
	return true;
	} */

    /**
     * Calculates the size. Uses getValue() and static calculateSize. 
     * Notes this method provides the real size but not necessary used size (due to limits).
     * @return result for calculating size.
     */
    /*
    protected float calculateSize()
    {
	return SizeableElement.calculateSize(maxSize, minSize, getMaxValue(), getValue());
    }
    */

    /**
     * Calculates the place for a value in limits, if superior limit corresponding with a max value.
     * @param maxSize superior limit
     * @param minSize inferior limit
     * @param maxValue max value 
     * @param value value to scale
     * @return result a scale change.
     */
    public static float calculateSize(double maxSize, double minSize, double maxValue, double value)
    {
	return new Float(minSize + Math.abs(value)*(maxSize - minSize)/maxValue).floatValue();
    }

    /**
     * Returns a value used to calculate size.
     * @return a double representation of value associated to sizeable element.
     */
    abstract public double getValue();

    //value associated to colors scale !
    abstract public double getColorationValue();


    //new !!
    public boolean isClassMember(double inf, double sup)
    {
	return (getColorationValue() >= inf)&&(getColorationValue() <= sup);
    }

}
