// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.sizeable;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

import fr.unicaen.openmap.layer.dataDisplayLayer.SimpleDataDisplayLayer;
import fr.unicaen.openmap.util.JActionButton;


/**
 * A panel to insert at your gui if your layer implements SizeableLayer.
 */
//author : Yannick Lange, ylange@etu.info.unicaen.fr, DESS NAPI, juin-septembre 2003.
public class SizeablePanel extends JPanel
{
    private SlidePanel maxSlidePanel; 
    private SlidePanel minSlidePanel; 
    protected SizeableLayer layer;


    /**
     * An empty panel.
     */
    public SizeablePanel()
    {
	super();
	setLayout(new BorderLayout());
    }

    /**
     * A complete Panel with maximum and minimum gestures.
     */
    public SizeablePanel(SizeableLayer layer)
    {
	this();

	this.layer = layer;

	add(getAppropriatedPanel("maximum"), BorderLayout.NORTH);
	add(getAppropriatedPanel("minimum"), BorderLayout.CENTER);
    }

    /**
     * Returns a panel for max gesture.
     * @return SlidePanel.
     */
    private SlidePanel getMaxSlidePanel()
    {
	return maxSlidePanel;
    }

    /**
     * Returns a panel for min gesture.
     * @return SlidePanel.
     */
    private SlidePanel getMinSlidePanel()
    {
	return minSlidePanel;
    }

    /**
     * Returns the layer associated to this panel.
     * @return a SimpleDataSorterLayer representation of layer.
     */
    public SizeableLayer getLayer()
    {
	return layer;
    }

    /**
     * Returns panel for max gesture (if type equals maximum), panel for min gesture.
     * @return JPanel.
     */
    private JPanel getAppropriatedPanel(String type)
    {
	JPanel res = new JPanel();

	JPanel right = new JPanel();

	if (type.equals("maximum"))
	    {
		maxSlidePanel = new SlidePanel(layer.getMaxSize(),"maximum");
		res.add(maxSlidePanel);
	    }
	else
	    {
		minSlidePanel = new SlidePanel(layer.getMinSize(),"minimum");
		res.add(minSlidePanel);
	    }

	right.add(new Submit(type));
	right.add(maxSlidePanel.giveCancel());

	res.add(right);

	return res;
    }

    /**
     * To have a Button with an appropriated purpose.
     */
    public class Submit extends JActionButton
    {
	private SlidePanel slidePanel;
	private double ref;
	private String type;

	public Submit(String type)
	{
	    super("Ok");

	    this.type = type;

	    if (type.equals("minimum")) 
		{
		    slidePanel = getMinSlidePanel();
		    ref = getLayer().getMinSize();
		}
	    else 
		{
		    slidePanel = getMaxSlidePanel();
		    ref = getLayer().getMaxSize();
		}
	}
		
	public void actionPerformed(ActionEvent e)
	{
	    double newValue;
	    double multiple;
	    boolean change;
	    multiple = slidePanel.getSlider().getValue();
	    try
		{
		    newValue = Double.parseDouble(slidePanel.getSliderValue().getText());
		}
	    catch(NumberFormatException nfe)
		{
		    //nfe.printStackTrace();
		    System.err.println("The next time, enter a number please");
		    newValue = ref;
		}
	    
	    if (newValue != ref)
		{   
		    if (type.equals("minimum")) change = getLayer().setMinSize(newValue);
		    else change = getLayer().setMaxSize(newValue);				
		    
		    if (change)
			{
			    ref = newValue;
			    slidePanel.getRefValue().setText(""+ref);
			    getLayer().refreshGraphics();
			}
		    else 
			{
			    System.out.println("DiscPanel : new wished value ("+newValue+") is out of bounds, I do nothing...");
			    slidePanel.getSliderValue().setText(""+ref);
			}
		    
		    slidePanel.getSlider().setValue(100);
		}
	    else System.out.println("I don't replace by the same thing !");
	}
    }//end Submit
}//end SizeablePanel
