// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.sizeable;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

import fr.unicaen.openmap.util.JActionButton;


//author : Yannick Lange, ylange@etu.info.unicaen.fr, DESS NAPI, juin-septembre 2003.
public class SlidePanel extends JPanel
{
    protected JSlider slider;
    protected JTextField sliderValue;
    protected JLabel refValue;


    /**
     * Creates a new SlidePanel.
     * @see SizeablePanel for a full documentation.
     * @param ref relative value 
     * @param type short description
     */ 
    public SlidePanel(double ref, String type)
    {
	super();
	setLayout(new BorderLayout());
	
	slider = new JSlider(JSlider.HORIZONTAL, 0, 200, 100); 
	slider.addChangeListener(new ChangeListener()
	    {
		public void stateChanged(ChangeEvent e) 
		{
		    sliderValue.setText(""+Double.parseDouble(refValue.getText()) * slider.getValue() / 100);
		}   
	    });
	slider.setToolTipText("multiply "+type+" size with this percent");
	slider.setMajorTickSpacing(20);
	slider.setMinorTickSpacing(20);
	slider.setPaintTicks(true);
	slider.setPaintLabels(true);
	slider.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));

	JPanel middle = new JPanel(new GridLayout(2,2));

	JLabel sliderText = new JLabel("wished "+type+" size : ");
	sliderValue = new JTextField(""+ref);

	JLabel maxText = new JLabel("current "+type+" size : ");
	refValue = new JLabel(""+ref);

	middle.add(sliderText);
	middle.add(sliderValue);
	middle.add(maxText);
	middle.add(refValue);

	add(slider, BorderLayout.NORTH);
	add(middle, BorderLayout.CENTER);
    }

    /**
     * @param ref relative value 
     */
    public SlidePanel(double ref)
    {
	this(ref,"");
    }

    /**
     * Returns slider.
     * Returns JSlider representation.
     */
    public JSlider getSlider()
    {
	return slider;
    }

    /**
     * Returns where are decoded adjusted ref with slider value.
     * @return a JTextField representation of wished ref.
     */
    public JTextField getSliderValue()
    {
	return sliderValue;
    }

    /**
     * Returns local where is effective ref.
     * @return a JLabel representation for real ref.
     */
    public JLabel getRefValue()
    {
	return refValue;
    }

    /**
     * Gives a button to initialize slider (at value 100).
     * @return a JActionButton representation of initializing button.
     */
    public JActionButton giveCancel()
    {
	JActionButton cancel = new JActionButton("Cancel")
	    {
		public void actionPerformed(ActionEvent e)
		{
		    getSlider().setValue(100);
		}
	    };
	cancel.setToolTipText("sets the slider at initial value");

	return cancel;
    }
}
